/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __APPLICATION_MESSAGE_H
#define __APPLICATION_MESSAGE_H

#include "container.chain.h"
#include "container.queue.h"
#include "graphics.rectangle.h"
#include "graphics.point.h"


namespace magnum { namespace application {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chain;
using magnum::container::Chainable;
using magnum::container::Queueable;
using magnum::graphics::Rectangle;
using magnum::graphics::Point;

class Handler;
class MessageQueue;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Message : private Queueable<Message> {
friend class MessageQueue;
public:
	enum Type {
		Pointer = 1,
		Integer = 2,
		Float = 3,
		Double = 4,
		Text = 5,
		RectangleInt = 6,
		PointInt = 7,
		RectangleFloat = 8,
		PointFloat = 9
	};


	class Parameter : public Chainable<Parameter> {
	friend class Message;
	private:
		Type					m_Type;
		char					m_Name[48];
		union {
			void*				m_Pointer;
			int					m_Integer;
			float				m_Float;
			double				m_Double;
			String*				m_String;
			int					m_RectangleInt[4];
			int					m_PointInt[4];
			float				m_RectangleFloat[4];
			float				m_PointFloat[4];
		};

	public:
		inline void				refer();
		inline void				release();
		
		inline Type				getType() const;

		inline const char*		getName() const;
		inline int				getInteger() const;
		inline float			getFloat() const;
		inline const String&	getString() const;
		inline double			getDouble() const;
		inline void*			getPointer() const;

		static void*			operator new(size_t );
		static void				operator delete(void* );
	};
	
	typedef Chain<Parameter>::Iterator Iterator;

private:
	int							m_Command;
	unsigned int				m_References;
	Chain<Parameter>			m_Parameters;
	Handler*					m_Target;

public:
	inline Message();
	inline explicit Message(int );
	inline explicit Message(Handler* );
	inline Message(Handler* ,int );
	inline Message(const Message& );
	inline ~Message();

	static void*			operator new(size_t );
	static void				operator delete(void* );

	inline void				refer();
	inline void				release();

	inline int				getCommand() const;
	inline void				setCommand(int cmd);
	
	inline Handler* 		getTarget() const;
	inline void				setTarget(Handler* );
							
	inline Iterator			begin() const;
	inline Iterator 		end() const;
	inline Iterator 		at(const Parameter* ) const;
	
	inline bool				contains(const char* ) const;
	
	inline bool				isEmpty() const;
	inline unsigned int		getSize() const;
	
	inline void				clear();
	
	inline const Message&	operator=(const Message& );

	// Type specific accessors
	inline bool				getPointer(const char* ,void** ) const;
	inline void*			getPointer(const char* ) const;
	void					addPointer(const char* ,void* );
							
	inline bool				getInteger(const char* ,int* ) const;
	void					addInteger(const char* ,int );
							
	inline bool				getFloat(const char* ,float* ) const;
	void					addFloat(const char* ,float );

	inline bool				getDouble(const char* ,double* ) const;
	void					addDouble(const char* ,double );
							
	inline bool				getRectangle(const char*, Rectangle<int>* ) const;
	void					addRectangle(const char*, const Rectangle<int>& );
							
	inline bool				getRectangle(const char*, Rectangle<float>* ) const;
	void					addRectangle(const char*, const Rectangle<float>& );
							
	inline bool				getPoint(const char*, Point<int>* ) const;
	void					addPoint(const char*, const Point<int>& );
							
	inline bool				getPoint(const char*, Point<float>* ) const;
	void					addPoint(const char*, const Point<float>& );
							
	inline bool				getString(const char* ,String* ) const;
	void					addString(const char* ,const char* );
	void					addString(const char* ,const String& );

private:
	Parameter*				findParameter(const char* ) const;
	void			 		cloneParameters(const Chain<Parameter>& );
};

}; };
#endif
