/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __APPLICATION_MESSENGER_H
#define __APPLICATION_MESSENGER_H

#include "system.thread.h"
#include "application.handler.h"
#include "application.message.h"
#include "container.chain.h"
#include "container.allocator.h"


namespace magnum { namespace application {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chain;
using magnum::container::Allocator;
using magnum::system::Thread;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Messenger : public Handler {
DeclareClass(Messenger, Handler);
private:
	Thread*				m_Thread;
	Chain<Handler>		m_Handlers;
	Handler*			m_PreferredHandler;

public:
	typedef Chain<Handler>::Iterator Iterator;

public:
	Messenger();
	~Messenger();

	inline bool			isAlive() const;

	inline bool			synchronize();

	bool				insert(Handler* );
	bool				remove(Handler* );
	void				clear();
	
	inline Iterator 	begin() const;
	inline Iterator 	end() const;
	inline Iterator 	at(const Handler* ) const;
	
	inline Handler*		getPreferredHandler() const;
	inline void			setPreferredHandler(Handler* );

	inline bool			contains(const Handler* ) const;
	inline unsigned int getSize() const;
	inline bool			isEmpty() const;

	bool				postMessage(const Message& msg);
	bool				postMessage(int cmd);
	bool				postMessage(Handler* target, const Message& msg);
	bool				postMessage(Handler* target, int cmd);
	bool				sendMessage(const Message& msg);
	bool				sendMessage(int cmd);
	bool				sendMessage(Handler* target, const Message& msg);
	bool				sendMessage(Handler* target, int cmd);

	inline Thread*		getThread() const;
	
protected:
	inline void			attachThread(Thread* );
	inline void			detachThread();
	
private:
	virtual bool		onSendMessage(Message* ) = 0;
	virtual bool		onPostMessage(Message* ) = 0;
	virtual bool		onSynchronize() = 0;
};

}; };

#endif
