/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __BASE_STRING_INL
#define __BASE_STRING_INL

#include <string.h>

namespace magnum { namespace types {
using namespace magnum::object;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String::String() {
	m_Buffer = NULL;
	m_BufferSize = 0;
	m_Length = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String::operator const char*() const {
	static const char empty[] = "";
	return (m_Buffer ? m_Buffer : empty);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void String::clear() {
	m_Length = 0;
	if (m_Buffer)
		m_Buffer[0] = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String String::getName() const {
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String String::toString() const {
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline size_t String::getLength() const {
	return m_Length;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const char* String::getString() const {
	static const char empty[] = "";
	return (m_Buffer ? m_Buffer : empty);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int String::lastIndexOf(const String& str, unsigned int from) const {
	return lastIndexOf(str.getString(), from);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String String::substring(unsigned int start) const {
	if (start >= m_Length)
		return String();
	String str;
	str.m_Length = m_Length - start;
	str.allocBuffer();
	memcpy(str.m_Buffer, m_Buffer+start, str.m_Length);
	str.m_Buffer[str.m_Length] = 0;
	return str;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String String::substring(unsigned int start, unsigned int end) const {
	if (start >= m_Length)
		return String();
	String str;
	str.m_Length = min(end - start, m_Length - start) + 1;
	str.allocBuffer();
	memcpy(str.m_Buffer, m_Buffer+start, str.m_Length);
	str.m_Buffer[str.m_Length] = 0;
	return str;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String String::toLowerCase() const {
	if (m_Length == NULL)
		return String();
	String str(*this);
#if defined(__FORWINDOWS__)
	_strlwr(str.m_Buffer);
#else
	char* c;
	for(c = str.m_Buffer; *c != 0; c++)
		*c = tolower(*c);
#endif
	return str;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String String::toUpperCase() const {
	if (m_Length == NULL)
		return String();
	String str(*this);
#if defined(__FORWINDOWS__)
	_strupr(str.m_Buffer);
#else
	char* c;
	for(c = str.m_Buffer; *c != 0; c++)
		*c = toupper(*c);
#endif
	return str;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const String& String::append(const String& str) {
	return operator+=(str);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const String& String::append(const char* str) {
	return operator+=(str);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const String& String::append(const Object& str) {
	return operator+=(str);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const String& String::append(char c) {
	return operator+=(c);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String String::trim() {
	return String();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline char& String::operator[](unsigned int i) {
	ASSERT(i <= m_Length);
	return m_Buffer[i];
}

	
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const char& String::operator[](unsigned int i) const {
	ASSERT(i <= m_Length);
	return m_Buffer[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator==(const String& str) const {
	if (str.m_Length != m_Length)
		return false;
	if (m_Length == 0)
		return true;
	return (memcmp(m_Buffer, str.m_Buffer, m_Length) == 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator==(const char* str) const {
	ASSERT(str != NULL);
	if (m_Length == 0)
		return (*str == 0);
	return (strncmp(m_Buffer, str, m_Length) == 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator!=(const String& str) const {
	if (str.m_Length != m_Length)
		return true;
	if (m_Length == 0)
		return false;
	return (memcmp(m_Buffer, str.m_Buffer, m_Length) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator!=(const char* str) const {
	ASSERT(str != NULL);
	if (m_Length == 0)
		return (*str != 0);
	return (strncmp(m_Buffer, str, m_Length) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator>(const String& str) const {
	if (m_Length == 0) 
		return false;
	if (str.m_Length == 0)
		return true;
	return (strcmp(m_Buffer, str.m_Buffer) > 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator>(const char* str) const {
	ASSERT(str != NULL);

	if (m_Length == 0)
		return false;
	if (*str == 0)
		return true;
	return (strncmp(m_Buffer, str, m_Length) > 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator<(const String& str) const {
	if (str.m_Length == 0)
		return false;
	if (m_Length == 0) 
		return true;
	return (strcmp(m_Buffer, str.m_Buffer) < 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator<(const char* str) const {
	ASSERT(str != NULL);

	if (*str == 0)
		return false;
	if (m_Length == 0)
		return true;
	return (strncmp(m_Buffer, str, m_Length) < 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator>=(const String& str) const {
	if (str.m_Length == 0)
		return true;
	if (m_Length == 0)
		return false;
	return (strcmp(m_Buffer, str.m_Buffer) >= 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator>=(const char* str) const {
	ASSERT(str != NULL);

	if (*str == 0)
		return true;
	if (m_Length == 0)
		return false;
	return (strncmp(m_Buffer, str, m_Length) >= 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator<=(const String& str) const {
	if (m_Length == 0)
		return true;
	if (str.m_Length == 0)
		return false;
	return (strcmp(m_Buffer, str.m_Buffer) <= 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool String::operator<=(const char* str) const {
	ASSERT(str != NULL);

	if (m_Length == 0)
		return true;
	if (*str == 0)
		return false;
	return (strncmp(m_Buffer, str, m_Length) <= 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const String& String::operator+=(const Object& str) {
	return operator+=(str.toString());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const String& String::operator=(const Object& str) {
	return operator=(str.toString());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String operator+(const String& str1, const char* str2) {
	ASSERT(str2 != NULL);

	unsigned int strLen = (unsigned int)strlen(str2);
	if (strLen == 0)
		return str1;

	String result;
	result.m_Length = str1.m_Length + strLen;
	result.allocBuffer();
	if (str1.m_Length > 0)
		memcpy(result.m_Buffer, str1.m_Buffer, str1.m_Length);
	if (strLen > 0)
		memcpy(result.m_Buffer + str1.m_Length, str2, strLen);
	result.m_Buffer[result.m_Length] = 0;
	return result;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String operator+(const char* str1, const String& str2) {
	ASSERT(str1 != NULL);

	unsigned int strLen = (unsigned int)strlen(str1);
	if (strLen == 0)
		return str2;

	String result;
	result.m_Length = str2.m_Length + strLen;
	result.allocBuffer();
	if (strLen > 0)
		memcpy(result.m_Buffer, str1, strLen);
	if (str2.m_Length > 0)
		memcpy(result.m_Buffer + strLen, str2.m_Buffer, str2.m_Length);
	result.m_Buffer[result.m_Length] = 0;
	return result;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String operator+(const String& str1, const Object& str2) {
	return str1 + str2.toString();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String operator+(const String& str1, const String& str2) {
	String result;
	result.m_Length = str1.m_Length + str2.m_Length;
	result.allocBuffer();
	if (str1.m_Length > 0)
		memcpy(result.m_Buffer, str1.m_Buffer, str1.m_Length);
	if (str2.m_Length > 0)
		memcpy(result.m_Buffer + str1.m_Length, str2.m_Buffer, str2.m_Length);
	result.m_Buffer[result.m_Length] = 0;
	return result;
}

}; };

#endif
