/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

///////////////////////////////////////////////////////////////////////////////
//
// You MUST supply one of the following defines:
//		__FORWINDOWS__		- For Windows
//		__FORDOS__			- For DOS
//		__FOROS2__			- For OS/2
//		__FORUNIX__			- For unix
//		__FORLINUX__		- For Linux-environment
//
// If your target platform is DOS or WINDOWS, then you can simply define
//		__FORDOS__
//		__FORWINDOWS__
// Then this file will try to determine what word size and processor is
// used.
//
// You may also specify your processor platform:
//		__FORx86__			- Intel 80x86
//		__FORSPARC__		- Sun SPARC
//		__FORRx000__		- MIPS Rx000
//		__FORPPC__			- Power PC
//		__FORALPHA__		- DEC Alpha
// Note that the processor is detected automatically when compiling for
// Windows or DOS. (The later one must be an Intel !)
//
// Often it is helpfull to know about the byte-ordering. Here you can
// define:
//		__LSBFIRST__		- Like Intel,MIPS
//		__MSBFIRST__		- Like SPARC, Motorola
//
// You may also need the floating point format:
//		__FPIEEE__			- Like Intel
//		__FPIBM__			- For IBM workstations
//		__FPVAX__
//		__FPCRAY__
//
// And your compiler:
//		__FORBORLAND__		- Borland C++
//		__FORVISUAL__		- Microsoft Visual C++
//		__FORGNU__			- GNU C++
//		__FORWATCOM__		- Watcom Compiler
//
// UNIX-Options are:
//		__FORX11__			- For unix with X-Windows
//
// One important note for Windows users: If you want to write a portable
// program using void main(void), then you should not define __WINMAIN__,
// but if your program contains an own WinMain function, you MUST define
// __WINMAIN__, so GApplication will not define its own WinMain function !
//
// Notes for usage of threads: Threads will only be available, if the symbol
// __THREADS__ is defined (this is done automatically by Gomez depending on
// you compiling environment). If you use the Borland or Microsoft compiler,
// I suggest to enable the thread library.
//


#ifndef __TARGETSYSTEM_H
#define __TARGETSYSTEM_H


// Autodetect Borland compiler
# if defined(__BORLANDC__)
#	define __FORBORLAND__
#	if defined(__WIN32__)
#		define __FORWIN32__
#	elif defined(_Windows)
#		define __FORWINDOWS__
#	else
#		define __FORDOS__
#	endif
# endif

// Autodetect MS-C
# ifdef _MSC_VER
#	define __FORVISUAL__
#	if (defined(_Windows) || defined(WIN32))
#		define __FORWINDOWS__
#	else
#		define __FORDOS__
#	endif
# endif

// Autodetect GNUC
# ifdef __GNUC__
#	define __FORGNU__
# endif


// Enable UNIX if X11 or MOTIF selected
# if defined(__FORX11__) && !defined(__FORUNIX__)
#	define __FORUNIX__
# endif


// Enable UNIX if Linux
# if defined(__FORLINUX__) && !defined(__FORUNIX__)
#	define __FORUNIX__
# endif

// Enable UNIX and X11 if IRIX
# if defined(__FORIRIX__) && !defined(__FORUNIX__)
#	define __FORUNIX__
# endif
# if defined(__FORIRIX__) && !defined(__FORX11__)
#	define __FORX11__
# endif


# if defined (__FORDOS__)
#	define __CONSOLE__
#	define __32BIT__
#	define __THREADS__
#	if defined(_M_IX86)
#		define __FORx86__				// Intel 80x86
#	elif defined(_M_MRX000)
#		define __FORRx000__			// MIPS Rx000
#	elif defined(_M_ALPHA)
#		define __FORALPHA__			// DEC Alpha
#	elif defined(_M_PPC)
#		define __FORPPC__				// Power PC
#	else
#		define __FORx86__				// Intel 80x86
#	endif
# elif defined (__FORWINDOWS__)		// For Windows NT
#	define __32BIT__
#	define __LONGLONG__
#	define __THREADS__
#	if (defined(_DEBUG) || defined(_Debug))
#		define __FORDEBUG__
#	endif
#	if defined(_M_IX86)
#		define __FORx86__				// Intel 80x86
#	elif defined(_M_MRX000)
#		define __FORRx000__			// MIPS Rx000
#	elif defined(_M_ALPHA)
#		define __FORALPHA__			// DEC Alpha
#	elif defined(_M_PPC)
#		define __FORPPC__				// Power PC
#	else
#		define __FORx86__				// Intel 80x86
#	endif
# elif defined (__FOROS2__)     		// OS/2
#	define __32BIT__
#	define __FORx86__
#	define __THREADS__
# elif defined (__FORUNIX__) 		// UNIX
#	define __32BIT__
#	define __THREADS__
# else
#	error You must define __FOR???__ for your target platform !
# endif


// Check Byte ordering
# if defined (__FORx86__)
#	define __LSBFIRST__
#	define __FPIEEE__
# elif defined (__FORRx000__)
#	define __LSBFIRST__
# elif defined (__FORSPARC__)
#	define __MSBFIRST__
# endif

// Check for native TLS support in compiler
# if defined(__FORVISUAL__)
#	define __NATIVETLS__
# endif


# if defined(__FORWINDOWS__)
#	define STRICT
#   define _WIN32_WINDOWS 0x0410  // Windows 98
#	include <winsock2.h>
#	include <ws2tcpip.h>
#	include <windows.h>
# endif


// Some compilers just take our inline definitions not serious enough
#if defined(__FORVISUAL__)
#	pragma inline_depth(64)
#	pragma inline_recursion(on)
#	define inline __forceinline
#
#	pragma warning(disable:4786)
#	pragma warning(disable:4284)
#
#endif


#define __FOROPENGL__		13

#endif
