/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __BASE_TYPES_H
#define __BASE_TYPES_H

#include <sys/types.h>
#include "base.target.h"

namespace magnum { namespace types {


enum DataType {
	TypeByte	= 0x0001,
	TypeWord	= 0x0002,
	TypeDword	= 0x0004,
	TypeQword	= 0x0008,
	TypeFloat	= 0x1004,
	TypeDouble	= 0x1008
};


#if defined(__FORWINDOWS__)
	typedef unsigned __int8		byte;
	typedef unsigned __int16	word;
	typedef unsigned __int32	dword;
	typedef unsigned __int64	qword;

	typedef signed __int8		int8;
	typedef unsigned __int8		uint8;
	typedef signed __int16		int16;
	typedef unsigned __int16	uint16;
	typedef signed __int32		int32;
	typedef unsigned __int32	uint32;
	typedef signed __int64		int64;
	typedef unsigned __int64	uint64;

	typedef float				float32;
	typedef double				float64;
	typedef long double			float80;
#else
//#	if (__CHAR_BIT__ != 8)
//#		error "Wrong character size (not 8Bits)"
//#	endif

	typedef unsigned char		byte;
	typedef unsigned short int	word;
	typedef unsigned long int	dword;
	typedef unsigned long long	qword;

	typedef signed char			int8;
	typedef unsigned char		uint8;
	typedef signed short int	int16;
	typedef unsigned short int	uint16;
	typedef signed int			int32;
	typedef unsigned int		uint32;
	typedef signed long long	int64;
	typedef unsigned long long	uint64;

	typedef float				float32;
	typedef double				float64;
	typedef long double			float80;
#endif


#if !defined(__FORWINDOWS__)
	typedef byte				BYTE;
	typedef word				WORD;
	typedef dword				DWORD;
	typedef qword				QWORD;
#endif


#ifndef TRUE
 #define TRUE					1
 #define FALSE					0
#endif

#ifndef NULL
 #define NULL					0L
#endif

//
// Simple macros for getting low/high-byte/words
//
inline byte lobyte(word w)		{ return (byte)w; };
inline byte hibyte(word w)		{ return (byte)(w >> 8); };
inline word loword(dword l)		{ return (word)l; };
inline word hiword(dword l)		{ return (word)((dword)l >> 16); };

inline word swapword(word w) {
	return ((w>>8) | (w & 0xff));
}

inline dword swapdword(dword d) {
	return ((d>>24) | ((d>>16) & 0xff00)
		| ((d<<8) & 0xff0000) | ((d << 16) & 0xff000000));
}

inline qword swapqword(qword q) {
	 return (((qword)(byte)(q>>56))) | (((qword)(byte)(q>>48))<<8)
		  | (((qword)(byte)(q>>40))<<16) | (((qword)(byte)(q>>32))<<24)
		  | (((qword)(byte)(q>>24))<<32) | (((qword)(byte)(q>>16))<<40)
		  | (((qword)(byte)(q>>8))<<48) | (((qword)(byte)(q))<<56);
}


//Some LSB/MSB Conversion Routines
#if defined(__LSBFIRST__)
	inline word		lsb2word(word w)	{ return w; };
	inline dword	lsb2dword(dword w)	{ return w; };
	inline qword	lsb2qword(qword w)	{ return w; };
	inline word		msb2word(word w)	{ return swapword(w); };
	inline dword	msb2dword(dword w)	{ return swapdword(w); };
	inline qword	msb2qword(qword w)	{ return swapqword(w); };
#elif defined(__MSBFIRST__)
	inline word		lsb2word(word w)	{ return swapword(w); };
	inline dword	lsb2dword(dword w)	{ return swapdword(w); };
	inline qword	lsb2qword(qword w)	{ return swapqword(w); };
	inline word		msb2word(word w)	{ return w; };
	inline dword	msb2dword(dword w)	{ return w; };
	inline qword	msb2qword(qword w)	{ return w; };
#else
	inline word		lsb2word(word w)	{ return w; };
	inline dword	lsb2dword(dword w)	{ return w; };
	inline qword	lsb2qword(qword w)	{ return w; };
	inline word		msb2word(word w)	{ return w; };
	inline dword	msb2dword(dword w)	{ return w; };
	inline qword	msb2qword(qword w)	{ return w; };
#endif

inline word		word2lsb(word w) { return lsb2word(w); };
inline dword	dword2lsb(dword w) { return lsb2dword(w); };
inline qword	qword2lsb(qword w) { return lsb2qword(w); };
inline word		word2msb(word w) { return msb2word(w); };
inline dword	dword2msb(dword w) { return msb2dword(w); };
inline qword	qword2msb(qword w) { return msb2qword(w); };


}; };

#endif
