/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_ALLOCATOR_H
#define __CONTAINER_ALLOCATOR_H

#include "object.h"


namespace magnum { namespace container {
template<class T> class Allocator;
}; };


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void* operator new(size_t, magnum::container::Allocator<T>& a) {
	return a.alloc(); 
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void operator delete(void* obj, magnum::container::Allocator<T>& a) {
	a.free((T*)obj);
}


namespace magnum { namespace container {
using magnum::object::Object;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Allocator : public Object {
DeclareClass(Allocator, Object);
//friend void* operator new(size_t ,Allocator<T>& );
//friend void operator delete(void* ,Allocator<T>& );
private:
	struct Pool {
		Pool*	m_Next;
		T		m_Data[1];
	};

	enum {
		Tsize	= ((sizeof(T) + sizeof(void*) - 1) / sizeof(void*)) * sizeof(void*)
	};

	Pool*			m_First;
	Pool*			m_Last;
	void*			m_Free;
	unsigned int	m_Size;

public:
	Allocator();
	~Allocator();

	inline T*		alloc();
	inline void		free(T* );

private:
	void			allocate();
	void			deallocate();
};


}; };


#endif
