/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_HASHTABLE_INL
#define __CONTAINER_HASHTABLE_INL

namespace magnum { namespace container {
 

#if 0

/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
inline Hashtable<Key, Item, Hasher>::Hashtable() {
	m_TableSize = m_Hasher.getMax()+1;
	m_Table = new Node*[m_TableSize];
	memset(m_Table, 0 ,m_TableSize * sizeof(Node*));
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
inline Hashtable<Key, Item, Hasher>::~Hashtable() {
	clear();
	delete[] m_Table;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
inline void Hashtable<Key, Item, Hasher>::clear() {
	unsigned int i;
	Node *cur, *next;

	for (i = 0; i < m_TableSize; i++) {
		if (m_Table[i] != NULL) {
			cur = m_Table[i];
			m_Table[i] = NULL;
			do {
				next = cur->m_Next;
				delete(m_Allocator) cur;
				cur = next;
			} while(cur != NULL);
		}
	}
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
void Hashtable<Key, Item, Hasher>::insert(const Item& item,const Key& key) {
	unsigned int hash;
	Node *bucket;

	hash = m_Hasher.getHash(key);
	ASSERT(hash < m_TableSize);

	bucket = new(m_Allocator) Node;
	bucket->m_Item = item;
	bucket->m_Key = key;
	bucket->m_Next = m_Table[hash];
	m_Table[hash] = bucket;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
bool Hashtable<Key, Item, Hasher>::remove(const Key& key) {
	unsigned int hash;
	Node *cur, *last;

	hash = m_Hasher.getHash(key);
	ASSERT(hash < m_TableSize);

	cur = m_Table[hash];
	last = NULL;
	while (cur != NULL) {
		if (cur->m_Key == key) {
			if (last == NULL)
				m_Table[hash] = cur->m_Next;
			else
				last->m_Next = cur->m_Next;
			delete(m_Allocator) cur;
			return;
		}
		last = cur;
		cur = cur->m_Next;
	}
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
void Hashtable<Key, Item, Hasher>::remove(const typename Hashtable<Key, Item, Hasher>::Iterator& item) {
	unsigned int i;
	Node *cur, *last;

	for (i = 0; i < m_TableSize; i++) {
		if (m_Table[i] != NULL) {
			last = NULL;
			cur = m_Table[i];
			do {
				if (cur->m_Item == item) {
					if (last == NULL)
						m_Table[i] = cur->m_Next;
					else
						last->m_Next = cur->m_Next;
					delete(m_Allocator) cur;
				}
				last = cur;
				cur = cur->m_Next;
			} while(cur != NULL);
		}
	}
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
const Item& Hashtable<Key, Item, Hasher>::get(const Key& key) const {
	unsigned int hash;
	Node *bucket;

	hash = m_Hasher.getHash(key);
	ASSERT(hash < m_TableSize);

	bucket = m_Table[hash];
	while (bucket != NULL) {
		if (bucket->m_Key == key) {
			return bucket->m_Item;
		}
		bucket = bucket->m_Next;
	}

	return *(Item*)NULL;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
Item& Hashtable<Key, Item, Hasher>::get(const Key& key) {
	unsigned int hash;
	Node *bucket;

	hash = m_Hasher.getHash(key);
	ASSERT(hash < m_TableSize);

	bucket = m_Table[hash];
	while (bucket != NULL) {
		if (bucket->m_Key == key) {
			return bucket->m_Item;
		}
		bucket = bucket->m_Next;
	}

	return *(Item*)NULL;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
bool Hashtable<Key, Item, Hasher>::set(const Item& item, const Key& key) const {
	unsigned int hash;
	Node *bucket;

	hash = m_Hasher.getHash(key);
	ASSERT(hash < m_TableSize);

	bucket = m_Table[hash];
	while (bucket != NULL) {
		if (bucket->m_Key == key) {
			bucket->m_Item = item;
			return true;
		}
		bucket = bucket->m_Next;
	}
	
	return false;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher> 
bool Hashtable<Key, Item, Hasher>::contains(const Key& key) const {
	unsigned int hash;
	Node *bucket;

	hash = m_Hasher.getHash(key);
	ASSERT(hash < m_TableSize);

	bucket = m_Table[hash];
	while (bucket != NULL) {
		if (bucket->m_Key == key) {
			return true;
		}
		bucket = bucket->m_Next;
	}

	return false;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline HashtableEnumerator<Key, Item, Hasher>::HashtableEnumerator(const Hashtable<Key, Item, Hasher> *hash) {
	m_Hash = hash;
	m_Table = m_Hash->m_Table;
	m_TableSize = m_Hash->m_TableSize;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline HashtableEnumerator<Key, Item, Hasher>::~HashtableEnumerator() {

}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline void HashtableEnumerator<Key, Item, Hasher>::setHash(const Hashtable<Key, Item, Hasher> *hash) {
	m_Hash = hash;
	m_Table = m_Hash->m_Table;
	m_TableSize = m_Hash->m_TableSize;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline const Hashtable<Key, Item, Hasher>* HashtableEnumerator<Key, Item, Hasher>::getHash() const {
	return m_Hash;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline bool HashtableEnumerator<Key, Item, Hasher>::getFirstItem(Item& item) {
	for (m_Index = 0; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			item = m_Node->m_Item;
			return true;
		}
	}

	m_Node = NULL;
	return false;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline bool HashtableEnumerator<Key, Item, Hasher>::getNextItem(Item& item) {
	m_Node = m_Node->m_Next;
	if (m_Node != NULL) {
		item = m_Node->m_Item;
		return true;
	}

	for (; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			item = m_Node->m_Item;
			return true;
		}
	}

	return false;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline bool HashtableEnumerator<Key, Item, Hasher>::getCurrentItem(Item& item) {
	if (m_Node == NULL)
		return false;
	item = m_Node->m_Item;
	return true;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline Item* HashtableEnumerator<Key, Item, Hasher>::getFirstItem() {
	for (m_Index = 0; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			return &m_Node->m_Item;
		}
	}

	m_Node = NULL;
	return NULL;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline Item* HashtableEnumerator<Key, Item, Hasher>::getNextItem() {
	m_Node = m_Node->m_Next;
	if (m_Node != NULL) {
		return &m_Node->m_Item;
	}

	for (; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			return &m_Node->m_Item;
		}
	}

	return NULL;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline Item* HashtableEnumerator<Key, Item, Hasher>::getCurrentItem() {
	if (m_Node == NULL)
		return NULL;
	return &m_Node->m_Item;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline bool HashtableEnumerator<Key, Item, Hasher>::getFirstKey(Key& key) {
	for (m_Index = 0; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			key = m_Node->m_Key;
			return true;
		}
	}

	m_Node = NULL;
	return false;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline bool HashtableEnumerator<Key, Item, Hasher>::getNextKey(Key& key) {
	m_Node = m_Node->m_Next;
	if (m_Node != NULL) {
		key = m_Node->m_Key;
		return true;
	}

	for (; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			key = m_Node->m_Key;
			return true;
		}
	}

	return false;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline bool HashtableEnumerator<Key, Item, Hasher>::getCurrentKey(Key& key) {
	if (m_Node == NULL)
		return false;
	key = m_Node->m_Key;
	return true;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline const Key* HashtableEnumerator<Key, Item, Hasher>::getFirstKey() {
	for (m_Index = 0; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			return &m_Node->m_Key;
		}
	}

	m_Node = NULL;
	return NULL;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline const Key* HashtableEnumerator<Key, Item, Hasher>::getNextKey() {
	m_Node = m_Node->m_Next;
	if (m_Node != NULL) {
		return &m_Node->m_Key;
	}

	for (; m_Index < m_TableSize; m_Index++) {
		if (m_Table[m_Index] != NULL) {
			m_Node = m_Table[m_Index];
			return &m_Node->m_Key;
		}
	}

	return NULL;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Hasher>
inline const Key* HashtableEnumerator<Key, Item, Hasher>::getCurrentKey() {
	if (m_Node == NULL)
		return NULL;
	return &m_Node->m_Key;
}

#endif

}; };
#endif
