/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_LIST_INL
#define __CONTAINER_LIST_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline List<T>::List() {
	m_First = &m_Terminator;
	m_Size = 0;
	m_Terminator.m_Next = m_Terminator.m_Prev = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline List<T>::List(const List& l) {
	m_First = &m_Terminator;
	m_Size = 0;
	m_Terminator.m_Next = m_Terminator.m_Prev = NULL;
	operator=(l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline List<T>::~List() {
	clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> const List<T>& List<T>::operator=(const List& l) {
	clear();

	m_Size = l.m_Size;
	Item *newNode, *last, *node = l.m_First;
	if (node == NULL) {
		return *this;
	}	

	m_First = new(m_Allocator) Item();
	m_First->m_Item = node->m_Item;
	m_First->m_Prev = m_First->m_Next = NULL;;
	last = m_First;
	node = node->m_Next;

	while (node != &l.m_Terminator) {
		newNode = new(m_Allocator) Item(*node);
		newNode->m_Prev = last;
		newNode->m_Item = node->m_Item;
		last->m_Next = newNode;
		last = newNode;
		node = node->m_Next;
	}

	last->m_Next = &m_Terminator;
	m_Terminator.m_Prev = last;

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> bool List<T>::contains(const T& item) const {
	Item *node = m_First;
	while (node != &m_Terminator) {
		if (node->m_Item == item)
			return true;
		node = node->m_Next;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> typename List<T>::Iterator List<T>::append() {
	Item *node = new(m_Allocator) Item();
	Item *last = m_Terminator.m_Prev;
	node->m_Next = &m_Terminator;
	node->m_Prev = last;
	if (m_First == &m_Terminator) 
		m_First = node;
	else
		last->m_Next = node;
	m_Terminator.m_Prev = node;
	m_Size++;
	return Iterator(node);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> typename List<T>::Iterator List<T>::insert(const T& item) {
	Item *node = new(m_Allocator) Item(item);
	Item *last = m_Terminator.m_Prev;
	node->m_Next = &m_Terminator;
	node->m_Prev = last;
	if (m_First == &m_Terminator) 
		m_First = node;
	else
		last->m_Next = node;
	m_Terminator.m_Prev = node;
	m_Size++;
	return Iterator(node);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> typename List<T>::Iterator List<T>::insertFirst(const T& item) {
	Item *node = new(m_Allocator) Item(item);
	node->m_Next = m_First;
	node->m_Prev = NULL;
	m_First->m_Prev = node;
	m_First = node;
	m_Size++;
	return Iterator(node);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> bool List<T>::remove(const T& item) {
	Item *node = m_First;
	while (node != &m_Terminator) {
		if (node->m_Item == item) {
			if (m_First != node) {
				node->m_Prev->m_Next = node->m_Next;
			} else {
				m_First = node->m_Next;
			}
			node->m_Next->m_Prev = node->m_Prev;
			operator delete(node, m_Allocator);
			m_Size--;
			return true;
		}
		node = node->m_Next;
	}

	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void List<T>::remove(const typename List<T>::Iterator& iter) {
	Item* node = iter.m_Item;
	if (m_First != node) {
		node->m_Prev->m_Next = node->m_Next;
	} else {
		m_First = node->m_Next;
	}
	node->m_Next->m_Prev = node->m_Prev;
	operator delete(node, m_Allocator);
	m_Size--;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void List<T>::clear() {
	Item *node;
	while (m_First != &m_Terminator) {
		node = m_First;
		m_First = m_First->m_Next;
		operator delete(node, m_Allocator);
		m_Size--;
	}

	m_Terminator.m_Next = m_Terminator.m_Prev = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline unsigned int List<T>::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename List<T>::Iterator List<T>::begin() const {
	return Iterator(m_First);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename List<T>::Iterator List<T>::end() const {
	return Iterator((Item*)&m_Terminator);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename List<T>::Iterator List<T>::at(const T& item) const {
	Item *node = m_First;

	while (node != &m_Terminator) {
		if (node->m_Item == item) 
			return Iterator(node);
		node = node->m_Next;
	}

	return Iterator((Item*)&m_Terminator);
}


}; };
#endif
