/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_MAP_H
#define __CONTAINER_MAP_H


#include "base.types.h"
#include "object.h"
#include "base.assert.h"
#include "container.iterator.h"
#include "container.allocator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

template<class Key, class Item> class Multimap;

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item> class Map : public Object {
DeclareClass(Map, Object);
friend class Multimap<Key, Item>;
private:
	typedef struct Node {
		Item			m_Item;
		Key				m_Key;
		unsigned int	m_Rand;
		Node*			m_LSon;
		Node*			m_RSon;
		Node*			m_Father;
	} Node;


	Node*				m_Root;
	unsigned int		m_Size;
	unsigned int		m_Rand;
	Allocator<Node>		m_Allocator;

public:
	class Iterator : public IteratorBase<Iterator> {
	friend class Map<Key, Item>;
	friend class Multimap<Key, Item>;
	public:
		inline Iterator();
		inline				operator Item*();
		inline				operator const Item*() const;
		inline Item&		operator*() const;
		inline Item*		operator->() const;
		inline Iterator&	operator++();
		inline Iterator		operator++(int );
		inline Iterator&	operator--();
		inline Iterator		operator--(int );
		inline bool 		operator==(const Iterator& iter) const;
		inline bool 		operator!=(const Iterator& iter) const;
		inline bool			operator<(const Iterator& iter) const;
		inline const Item&	item() const;
		inline const Key&	key() const;

	private:
		inline Iterator(Node* n);

		Node*	m_Node;
	};


public:
	inline Map();
	inline ~Map();

	inline bool			contains(const Key& k) const;
	Iterator			insert(const Key& key, const Item& item);
	void				remove(const Key& key);
	void				remove(const Iterator& item);
	void				clear();
						
	inline const Item&	get(const Key& key) const;
	inline Item&		get(const Key& key);
	inline void			set(const Key&, const Item& elem);
						
	inline unsigned		int getSize() const;
						
	inline bool			isEmpty() const;
						
	inline Iterator		begin() const;
	inline Iterator		at(const Key& ) const;
	inline Iterator		end() const;

	inline Item&		operator [](const Key& );
	inline const Item&	operator [](const Key& ) const;


private:
	 void			removeNode(Node *node, Node **father);
	 void			repairUp(Node *node);
	 inline unsigned int rand();
};


}; };

#endif
