/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_SCHEDULER_INL
#define __CONTAINER_SCHEDULER_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> Scheduler<T>::Scheduler() {
	m_Root.m_Scheduler = this;
	m_Root.m_Next = &m_Root;
	m_Root.m_Prev = &m_Root;
	m_Size = 0;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> Scheduler<T>::~Scheduler() {
	clear();
	m_Root.m_Scheduler = NULL;
	m_Root.m_Next = NULL;
	m_Root.m_Prev = NULL;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline Task<T>::Task() {
	m_Scheduler = NULL;
	m_Prev = NULL;
	m_Next = NULL;
	m_Priority = Normal;
	m_Visited = false;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline Task<T>::~Task() {
	if (m_Scheduler != NULL) {
		m_Prev->m_Next = m_Next;
		m_Next->m_Prev = m_Prev;
		m_Scheduler->m_Size--;
		m_Scheduler = NULL;
		m_Prev = NULL;
		m_Next = NULL;
	}
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> void Task<T>::setDependency(T* task) {
	m_Dependency = task;
	if (task == NULL)
		return;
	if (m_Scheduler == NULL)
		return;
	if (((Task<T>*)task)->m_Scheduler == NULL)
		m_Scheduler->insert(task);
	ASSERT(((Task<T>*)task)->m_Scheduler == m_Scheduler);
	m_Prev->m_Next = m_Next;
	m_Next->m_Prev = m_Prev;
	m_Next = ((Task<T>*)task)->m_Next;
	m_Prev = task;
	((Task<T>*)task)->m_Next = this;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline T* Task<T>::getDependency() const {
	return m_Dependency;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline typename Task<T>::Priority Task<T>::getPriority() const {
	return m_Priority;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline void Task<T>::setPriority(typename Task<T>::Priority p) {
	m_Priority = p;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline Scheduler<T>* Task<T>::getScheduler() const {
	return m_Scheduler;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> typename Scheduler<T>::Iterator Scheduler<T>::insert(T* task) {
	ASSERT(task != NULL);
	ASSERT(((Task<T>*)task)->m_Scheduler == NULL);
	if (((Task<T>*)task)->m_Dependency != NULL) {
		if (((Task<T>*)task)->m_Dependency->m_Scheduler == NULL)
			insert(((Task<T>*)task)->m_Dependency);
		Task<T>* prev = ((Task<T>*)task)->m_Dependency;
		((Task<T>*)task)->m_Next = prev->m_Next;
		((Task<T>*)task)->m_Prev = prev;
		((Task<T>*)task)->m_Prev->m_Next = task;
		((Task<T>*)task)->m_Next->m_Prev = task;
	} else {
		((Task<T>*)task)->m_Scheduler = this;
		((Task<T>*)task)->m_Next = &m_Root;
		((Task<T>*)task)->m_Prev = m_Root.m_Prev;
		((Task<T>*)task)->m_Prev->m_Next = task;
		((Task<T>*)task)->m_Next->m_Prev = task;
	}
	task->refer();
	m_Size++;
	return Iterator(task);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> bool Scheduler<T>::remove(T* task) {
	ASSERT(task != NULL);
	ASSERT(((Task<T>*)task)->m_Scheduler == this)
//	ASSERT(noDependencies(task));

	((Task<T>*)task)->m_Prev->m_Next = ((Task<T>*)task)->m_Next;
	((Task<T>*)task)->m_Next->m_Prev = ((Task<T>*)task)->m_Prev;
	((Task<T>*)task)->m_Prev = NULL;
	((Task<T>*)task)->m_Next = NULL;
	((Task<T>*)task)->m_Scheduler = NULL;
	task->release();
	m_Size--;
	return true;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool Scheduler<T>::contains(const T* task) const {
	return (((Task<T>*)task)->m_Scheduler == this);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline unsigned int Scheduler<T>::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool Scheduler<T>::isEmpty() const {
	return (m_Root.m_Next == &m_Root);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> void Scheduler<T>::clear() {
	while (m_Root.m_Next != &m_Root) {
		remove ((T*)m_Root.m_Next);
	}
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline typename Scheduler<T>::Iterator Scheduler<T>::begin() const {
	return Iterator(m_Root.m_Next);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline typename Scheduler<T>::Iterator Scheduler<T>::end() const {
	return Iterator((Task<T>*)&m_Root);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline typename Scheduler<T>::Iterator Scheduler<T>::at(const T* t) const {
	return Iterator(t);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T>::TaskIterator() {
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T>::TaskIterator(Task<T>* task) {
	m_Task = task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T>::operator T*() {
	return (T*)m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T>::operator const T*() const {
	return (const T*)m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline T* TaskIterator<T>::operator->() {
	return (T*)m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline const T* TaskIterator<T>::operator->() const {
	return (const T*)m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline T& TaskIterator<T>::operator*() {
	return *(T*)m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline const T& TaskIterator<T>::operator*() const {
	return *(const T*)m_Task;
}
	

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T>& TaskIterator<T>::operator++() {
	m_Task = m_Task->m_Next;
	return *this;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T> TaskIterator<T>::operator++(int ) {
	Task<T>* task = m_Task;
	m_Task = m_Task->m_Next;
	return TaskIterator<T>(task);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T>& TaskIterator<T>::operator--() {
	m_Task = m_Task->m_Prev;
	return *this;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline TaskIterator<T> TaskIterator<T>::operator--(int ) {
	Task<T>* task = m_Task;
	m_Task = m_Task->m_Prev;
	return TaskIterator<T>(task);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool TaskIterator<T>::operator==(const TaskIterator<T>& i) const {
	return m_Task == i.m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool TaskIterator<T>::operator!=(const TaskIterator<T>& i) const {
	return m_Task != i.m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool TaskIterator<T>::operator<(const TaskIterator<T>& i) const {
	return m_Task < i.m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool TaskIterator<T>::operator>(const TaskIterator<T>& i) const {
	return m_Task > i.m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool TaskIterator<T>::operator<=(const TaskIterator<T>& i) const {
	return m_Task <= i.m_Task;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> inline bool TaskIterator<T>::operator>=(const TaskIterator<T>& i) const {
	return m_Task >= i.m_Task;
}

}; };

#endif
