/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_STACK_INL
#define __CONTAINER_STACK_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Stack<T>::Stack(int size) {
	m_Size = size;
	m_Position = 0;
	m_Stack = new T[m_Size];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Stack<T>::~Stack() {
	delete[] m_Stack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Stack<T>::push(const T& t) {
	ASSERT(m_Position < m_Size);
	m_Stack[m_Position] = t;
	m_Position++;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T Stack<T>::pop() {
	ASSERT(m_Position > 0);
	m_Position--;
	return m_Stack[m_Position];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T& Stack<T>::peek() {
	ASSERT(m_Position > 0);
	return m_Stack[m_Position-1];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const T& Stack<T>::peek() const {
	ASSERT(m_Position > 0);
	return m_Stack[m_Position-1];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Stack<T>::clear() {
	m_Position = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Stack<T>::isEmpty() const {
	return (m_Position == 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Stack<T>::isFull() const {
	return (m_Position == m_Size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline unsigned int Stack<T>::getSize() const {
	return m_Position;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename Stack<T>::Iterator Stack<T>::begin() const {
	return Iterator(m_Stack);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename Stack<T>::Iterator Stack<T>::at(unsigned int i) const {
	ASSERT(i < m_Position);
	return Iterator(m_Stack + i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename Stack<T>::Iterator Stack<T>::end() const {
	return Iterator(m_Stack + m_Position);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T& Stack<T>::operator[](unsigned int i) {
	ASSERT(i < m_Position);
	return m_Stack[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const T& Stack<T>::operator[](unsigned int i) const {
	ASSERT(i < m_Position);
	return m_Stack[i];
}

}; };
#endif
