/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_TUPLE_INL
#define __CONTAINER_TUPLE_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline Tuple<T,n>::Tuple() {
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline Tuple<T,n>::Tuple(const T* tuple) {
	for (unsigned int i = 0; i < n; i++)
		m_Tuple[i] = tuple[i];
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline bool Tuple<T,n>::contains(const T& item) const {
	for (unsigned int i = 0; i < n; i++)
		if (m_Tuple[i] == item)
			return true;
	return false;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline T Tuple<T,n>::min() const {
	T min = m_Tuple[0];
	for (unsigned int i = 1; i < n; i++) {
		if (m_Tuple[i] < min)
			min = m_Tuple[i];
	}
	return min;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline T Tuple<T,n>::max() const {
	T max = m_Tuple[0];
	for (unsigned int i = 1; i < n; i++) {
		if (m_Tuple[i] > max)
			max = m_Tuple[i];
	}
	return max;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline bool Tuple<T,n>::operator==(const Tuple<T,n>& tuple) const {
	for (unsigned int i = 0; i < n; i++)
		if (m_Tuple[i] != tuple.m_Tuple[i])
			return false;
	return true;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline bool Tuple<T,n>::operator!=(const Tuple<T,n>& tuple) const {
	for (unsigned int i = 0; i < n; i++)
		if (m_Tuple[i] != tuple.m_Tuple[i])
			return true;
	return false;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline bool Tuple<T,n>::operator<(const Tuple<T,n>& tuple) const {
	for (unsigned int i = 0; i < n; i++) {
		if (m_Tuple[i] < tuple.m_Tuple[i])
			return true;
		if (m_Tuple[i] > tuple.m_Tuple[i])
			return false;
	}
	return false;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline bool Tuple<T,n>::operator<=(const Tuple<T,n>& tuple) const {
	for (unsigned int i = 0; i < n-1; i++) {
		if (m_Tuple[i] < tuple.m_Tuple[i])
			return true;
		if (m_Tuple[i] > tuple.m_Tuple[i])
			return false;
	}
	return (m_Tuple[n-1] <= tuple.m_Tuple[n-1]);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline bool Tuple<T,n>::operator>(const Tuple<T,n>& tuple) const {
	for (unsigned int i = 0; i < n; i++) {
		if (m_Tuple[i] > tuple.m_Tuple[i])
			return true;
		if (m_Tuple[i] < tuple.m_Tuple[i])
			return false;
	}
	return false;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline bool Tuple<T,n>::operator>=(const Tuple<T,n>& tuple) const {
	for (unsigned int i = 0; i < n-1; i++) {
		if (m_Tuple[i] > tuple.m_Tuple[i])
			return true;
		if (m_Tuple[i] < tuple.m_Tuple[i])
			return false;
	}
	return (m_Tuple[n-1] >= tuple.m_Tuple[n-1]);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline Tuple<T,n>::operator const T*() const {
	return m_Tuple;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T, unsigned int n> inline Tuple<T,n>::operator T*() {
	return m_Tuple;
}


}; };

#endif
