/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __FREETYPE_FONT_H
#define __FREETYPE_FONT_H

#include "graphics.image.h"
#include "freetype.glyph.h"
#include "freetype.descriptor.h"

namespace magnum { namespace freetype {
using magnum::graphics::Image;
using magnum::object::Object;
using magnum::types::String;

class FontFactory;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Font : public Object {
DeclareClass(Font, Object);
friend class FontFactory;
public:
	enum Flags {
		Scalable		= FT_FACE_FLAG_SCALABLE,
		FixedWidth		= FT_FACE_FLAG_FIXED_WIDTH,
		FixedSizes		= FT_FACE_FLAG_FIXED_SIZES,
		Kerning			= FT_FACE_FLAG_KERNING,
	};
	
	static const char		Bold[];
	static const char		Regular[];
	static const char		Italic[];

private:
	Descriptor				m_Descriptor;
	Image					m_Image;
	Glyph					m_Glyphs[256];
	unsigned int			m_Flags;
	
public:
	Font(const Descriptor& );
	Font(const Descriptor& ,const Dimension<int>& );
	
	inline const Descriptor& getDescriptor() const;

	inline String			getName() const;
	inline const String& 	getFamily() const;
	inline const String& 	getStyle() const;
	
	inline const Dimension<float>& getSize() const;
	inline Encoding::Type	getEncoding() const;
	inline bool				getKerning() const;
	
	inline const Image*		getBitmap() const;
	inline Image*			getBitmap();
	
	inline const Glyph&		operator[](char ) const;
	inline Glyph&			operator[](char );
};

}; };

#endif
