/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __FREETYPE_RENDERER_INL
#define __FREETYPE_RENDERER_INL

namespace magnum { namespace freetype {


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
inline void Renderer::selectFont(Font* f) {
	ASSERT(f != NULL);
	m_Font = f;
	m_Size = m_Font->getDescriptor().getSize();
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
inline void	Renderer::deselectFont() {
	m_Font = NULL;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
inline Font* Renderer::getFont() const {
	return m_Font;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
Dimension<float> Renderer::getSize(char c) {
	switch(c) {
		case 0: 
			return m_Descriptor.getSize();
		case '\t':
			return Dimension<float>(m_Descriptor.getWidth()*m_Tabsize, m_Descriptor.getHeight());
		case ' ':
			return Dimension<float>(m_Descriptor.getWidth(), m_Descriptor.getHeight());
		default:
			return (*m_Font)[c].getSize();
	}
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
Dimension<float> Renderer::getSize(const char* string) {
	float width = 0;
	float height = 0;
	int lines = 1;
	float length = 0.0f;
	const char *s = str;
	while(*s) {
		switch (*s) {
			case 27:
				if (*(s+1) != '[') {
					length += (*m_Font)[*s].getWidth();
					break;
				}
				while(*s && *s != 'm') s++;
				break;
			case '\n':
				lines++;
				if(width < length) 
					width = length;
				length = 0.0f;
				break;
			case '\r':
				if(width < length) 
					width = length;
				length = 0.0f;
				break;
			case '\t':
				length += m_Size.getWidth() * m_Tabsize;
				break;
			case ' ':
				length += m_Size.getWidth();
				break;
			default:
				length += (*m_Font)[*s].getWidth();
		}
		s++;
	}
	if(width < length) 
		width = length;
	height = m_Size.getHeight() * lines;
	return Dimension<float>(width, height);
}



/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void Renderer::puts(const Point<float>& pos,const char *s) {
	float row = pos.getY();
	onLocation(pos);	
	while(*s) {
		switch(*s) {
			case 27:
				s += 2;
				int code;
				sscanf(s,"%dm",&code);
				if (s == 0) {
					renderColour(Reset);
				} else if ((s >= 30 ) && (s <=37)) {
					renderColour((ColourCode)(s - 30 + Gray));
				}
				while(*s && *s != 'm') s++;
				break;
			case '\n':
				row += m_Size.getHeight());
				onLocation(Point<float>(pos.getX(), row));
				break;
			case '\r':
				onLocation(Point<float>(pos.getX(), row));
				break;
			case '\t':
				onMove(Point<float>(m_Size.getWidth() * m_Tabsize, 0.0f));
				break;
			case ' ':
				onMove(Point<float>(m_Size.getWidth(), 0.0f));
				break;
			default:
				onRender((*m_Font)[*s]);
		}
		s++;
	}
}


}; };

#endif
