/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_CUBE_H
#define __GEOMETRY_CUBE_H

#include "math.vector.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "math.vector4.h"
#include "geometry.body.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::math::Vector4;

template<class T, int dim> class Sphere;
template<class T, int dim> class Box;
template<class T, int dim> class AlignedBox;

template<class T> class Cube2;
template<class T> class Cube3;
template<class T> class Cube4;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Cube : public Body<T,dim> {
friend class Cube2<T>;
friend class Cube3<T>;
friend class Cube4<T>;
private:
	Vector<T,dim>	m_Origin;
	T				m_Radius;

public:
	inline Cube();
	inline Cube(const Vector<T,dim>& ,const T& );

	inline void			setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void			setRadius(const T& );
	inline const T&		getRadius() const;

	inline bool			isEmpty() const;
	inline void			empty();

	inline bool			isInside(const Vector<T,dim>& ) const; 
	T					getDistance(const Vector<T,dim>& ) const;
	
	bool				intersect(const Sphere<T,dim>& ) const;
	bool				intersect(const Cube<T,dim>& ) const;
	inline bool			intersect(const AlignedBox<T,dim>& ) const;
	inline bool			intersect(const Box<T,dim>& ) const;

	Box<T,dim>			toBox() const;
	Sphere<T,dim>		toSphere() const;
	AlignedBox<T,dim>	toAlignedBox() const;

	inline void			bound(const Vector<T,dim>* ,unsigned int n);
	void				bound(const Vector<T,dim>* ,unsigned int stride,unsigned int n);
	
	void				merge(const Cube<T,dim>& );
	
	void				getVertices(Vector<T,dim>* ) const;
};



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Cube2 : public Cube<T,2> {
public:	
	inline Cube2();
	inline Cube2(const Vector2<T>& ,const T& );
	inline Cube2(const Vector<T,2>& ,const T& );
	inline Cube2(const Cube<T,2>& );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Cube3 : public Cube<T,3> { 
public:	
	inline Cube3();
	inline Cube3(const Vector3<T>& ,const T& );
	inline Cube3(const Vector<T,3>& ,const T& );
	inline Cube3(const Cube<T,3>& );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Cube4 : public Cube<T,4> { 
public:	
	inline Cube4();
	inline Cube4(const Vector4<T>& ,const T& );
	inline Cube4(const Vector<T,4>& ,const T& );
	inline Cube4(const Cube<T,4>& );
};



}; };

#endif
