/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_LINE_H
#define __GEOMETRY_LINE_H

#include "math.vector.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Line {
private:
	Vector<T,dim>		m_Origin;
	Vector<T,dim>		m_Direction;

public:
	inline Line();
	inline Line(const Vector<T,dim>& ,const Vector<T,dim>& );

	inline void			setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void			setDirection(const Vector<T,dim>& );
	inline const Vector<T,dim>& getDirection() const;

	inline T			getDistance2(const Vector<T,dim>& ) const;
	inline T			getDistance(const Vector<T,dim>& ) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Line2 : public Line<T,2> {
public:
	inline Line2() {};
	inline Line2(const Line<T,2>& l) : Line(l) {};
	inline Line2(const Vector<T,2>& o,const Vector<T,2>& d) : Line(o,d) {};
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Line3 : public Line<T,3> {
public:
	inline Line3() {};
	inline Line3(const Line<T,3>& l) : Line(l) {};
	inline Line3(const Vector<T,3>& o,const Vector<T,3>& d) : Line(o,d) {};
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Line4 : public Line<T,4> {
public:
	inline Line4() {};
	inline Line4(const Line<T,4>& l) : Line(l) {};
	inline Line4(const Vector<T,4>& o,const Vector<T,4>& d) : Line(o,d) {};
};


}; };

#endif
