/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_SEGMENT_H
#define __GEOMETRY_SEGMENT_H

#include "math.vector.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Segment {
private:
	Vector<T,dim>		m_Origin;
	Vector<T,dim>		m_Direction;

public:
	inline Segment();
	inline Segment(const Vector<T,dim>& ,const Vector<T,dim>& );

	inline void			setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void			setDirection(const Vector<T,dim>& );
	inline const Vector<T,dim>& getDirection() const;

	inline void			setLength(const T& );
	inline T			getLength() const;

	inline T			getDistance2(const Vector<T,dim>& ) const;
	inline T			getDistance(const Vector<T,dim>& ) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Segment2 : public Segment<T,2> {
public:
	inline Segment2() {};
	inline Segment2(const Segment<T,2>& l) : Segment(l) {};
	inline Segment2(const Vector<T,2>& o,const Vector<T,2>& d) : Segment(o,d) {};
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Segment3 : public Segment<T,3> {
public:
	inline Segment3() {};
	inline Segment3(const Segment<T,3>& l) : Segment(l) {};
	inline Segment3(const Vector<T,3>& o,const Vector<T,3>& d) : Segment(o,d) {};
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Segment4 : public Segment<T,4> {
public:
	inline Segment4() {};
	inline Segment4(const Segment<T,4>& l) : Segment(l) {};
	inline Segment4(const Vector<T,4>& o,const Vector<T,4>& d) : Segment(o,d) {};
};

}; };

#endif
