/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_SPHERELIMITER_H
#define __GEOMETRY_SPHERELIMITER_H

#include "geometry.sphere.h"
#include "container.tuple.h"
#include "stats.permutation.h"
#include "math.linearsystem.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::container::Tuple;
using magnum::stats::Permutation;
using magnum::math::LinearSystem;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class SphereLimiter {
friend class Sphere<T,dim>;
private:
	Sphere<T,dim>				m_Sphere;
	Tuple<unsigned int, dim+1>	m_Support;
	unsigned int				m_SupportSize;
	Permutation					m_Permutation;
	const Vector<T,dim>*		m_Points;
	LinearSystem<T>				m_LinearSystem[dim+1];
	unsigned int				m_Stride;

	Sphere<T,dim>				m_TempSphere;


public:
	inline bool			limit(const Vector<T,dim>* ,unsigned int n);
	bool				limit(const Vector<T,dim>* ,unsigned int stride,unsigned int n);

	inline const Sphere<T,dim>&	getSphere() const;

private:
	inline const Vector<T,dim>& getPoint(unsigned int ) const;
	
	inline bool			contains(unsigned int ) const;
	inline bool			supports(unsigned int ) const;
	void				update(unsigned int );			

	inline bool			exactSphere2(unsigned int, unsigned int );
	bool				exactSphere3(unsigned int, unsigned int, unsigned int );
	bool				exactSphere4(unsigned int, unsigned int, unsigned int , unsigned int );
	bool				exactSphereN(unsigned int* ,unsigned int n);
	
	bool				updateSphere1(unsigned int );
	bool				updateSphere2(unsigned int );
	bool				updateSphere3(unsigned int );
	bool				updateSphereN(unsigned int ,unsigned int );
};


}; };

#endif
