/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_TRIANGLEMESH_H
#define __GEOMETRY_TRIANGLEMESH_H

#include "geometry.triangle.h"
#include "container.array.h"

namespace magnum { namespace geometry {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Array;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class TriangleMesh : public Object {
DeclareClass(TriangleMesh, Object);
private:
	typedef Vector<T,dim>	vec;

	struct Item {
		vec					m_Vector;
		unsigned int		m_Count;
		unsigned int		m_Index;
	};

	Array<vec>				m_Vertices;
	Array<unsigned int>		m_Indices;

public:
	inline TriangleMesh();
	inline ~TriangleMesh();

	void						append(const TriangleMesh* );
	inline void					setTriangle(unsigned int n, unsigned int ,unsigned int ,unsigned int );
	inline void					setTriangle(unsigned int n, const Vector<T,dim>& ,const Vector<T,dim>& ,const Vector<T,dim>& );
	inline void					setTriangle(unsigned int n, const Triangle<T,dim>& );
	void						clear();

	void						optimize(T = (T)0);

	inline unsigned int			getSize();

	inline const Array<unsigned int>& getIndices() const;
	inline const Array<Vector<T,dim> >&	getVertices() const;

	inline void					setIndices(const unsigned int* ,unsigned int );
	inline void					setIndices(unsigned int );
	inline void					getIndices(unsigned int* ,unsigned int );
	inline void					setVertices(unsigned int );
	inline void					setVertices(const Vector<T,dim>* ,unsigned int );
	inline void					getVertices(Vector<T,dim>* ,unsigned int );

	inline Vector<unsigned int,3> getIndices(unsigned int ) const;
	inline Triangle<T,dim>		getVertices(unsigned int ) const;

	inline Vector<T,dim>		getVertex(unsigned int ) const;
	inline void					setVertex(unsigned int ,const Vector<T,dim>& );

	inline Triangle<T,dim>		operator[](unsigned int ) const;
};


template<class T> class TriangleMesh2 : public TriangleMesh<T,2> {};
template<class T> class TriangleMesh3 : public TriangleMesh<T,3> {};
template<class T> class TriangleMesh4 : public TriangleMesh<T,4> {};


}; };

#endif
