/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS_POINT_H
#define __GRAPHICS_POINT_H

#include "base.h"

namespace magnum { namespace graphics {

template<class T> class Rectangle;
template<class T> class Dimension;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Point  {
friend class Rectangle<T>;
public:
	T					x;
	T					y;

public:
	inline Point();
	inline Point(const T& ,const T& );

	inline void 		setX(const T& );
	inline void 		setY(const T& );

	inline T 			getX() const;
	inline T 			getY() const;

	inline const Point& operator=(const Point& );

	inline bool 		operator==(const Point<T>& ) const;
	inline bool 		operator!=(const Point<T>& ) const;

	inline Point 		operator+(const Point<T>& ) const;
	inline Point 		operator-(const Point<T>& ) const;
	inline Point 		operator+(const Dimension<T>& ) const;
	inline Point 		operator-(const Dimension<T>& ) const;

	inline const Point& operator+=(const Point<T>& );
	inline const Point& operator-=(const Point<T>& );
	inline const Point& operator+=(const Dimension<T>& );
	inline const Point& operator-=(const Dimension<T>& );
	
	inline T&			operator[](unsigned int );
	inline const T&		operator[](unsigned int ) const;

	template<class S>
	inline operator Point<S>();
};


}; };
#endif
