/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS_RECTANGLE_H
#define __GRAPHICS_RECTANGLE_H

#include "graphics.point.h"
#include "graphics.dimension.h"
#include "math.consts.h"


namespace magnum { namespace graphics {
using magnum::math::min;
using magnum::math::max;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Rectangle {
private:
	T	m_X;
	T	m_Y;
	T	m_Width;
	T	m_Height;


public:
	inline Rectangle();
	inline Rectangle(const T& ,const T& ,const T& ,const T& );
	inline Rectangle(const Point<T>& ,const Dimension<T>& );

	inline bool			isInside(const Point<T>& ) const;
	inline bool			isInside(const T&, const T& ) const;
	inline bool			isInside(const Rectangle<T>& ) const;
	inline bool			isInside(const T&, const T& ,const T&, const T& ) const;

	inline bool			intersects(const Rectangle<T>& ) const;

	inline bool			isEmpty() const;

	inline T			getWidth() const;
	inline T			getHeight() const;
	inline Dimension<T> getSize() const;

	inline void			setWidth(const T& );
	inline void			setHeight(const T& );
	inline void			setSize(const Dimension<T>& );
	inline void			setSize(const T& ,const T& );

	inline T			getX() const;
	inline T			getY() const;
	inline void			setX(const T& );
	inline void			setY(const T& );

	inline Point<T>		getOrigin() const;
	inline void			setOrigin(const Point<T>& );
	inline void			setOrigin(const T& ,const T& );

    inline Rectangle	operator+(const Point<T> &) const;
    inline Rectangle	operator-(const Point<T> &) const;
    inline Rectangle	operator+(const Dimension<T> &) const;
    inline Rectangle	operator-(const Dimension<T> &) const;
    inline Rectangle	operator&(const Rectangle &) const;
    inline Rectangle	operator|(const Rectangle &) const;


    inline const Rectangle& operator+=(const Point<T> &);
    inline const Rectangle& operator-=(const Point<T> &);
    inline const Rectangle& operator+=(const Dimension<T> &);
    inline const Rectangle& operator-=(const Dimension<T> &);
    inline const Rectangle& operator&=(const Rectangle &);
    inline const Rectangle& operator|=(const Rectangle &);

	inline bool			operator==(const Rectangle<T>& ) const;
	inline bool			operator!=(const Rectangle<T>& ) const;

	template<class S>
	inline operator Rectangle<S>() const;
};
}; };
#endif
