/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS_RECTANGLE_INL
#define __GRAPHICS_RECTANGLE_INL


namespace magnum { namespace graphics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T>::Rectangle() {
	m_X = m_Y = m_Width = m_Height = T(0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T>::Rectangle(const T& x,const T& y,const T& w,const T& h) {
	m_X = x;
	m_Y = y;
	m_Width = w;
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T>::Rectangle(const Point<T>& p,const Dimension<T>& d) {
	m_X = p.x;
	m_Y = p.y;
	m_Width = d.width;
	m_Height = d.height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::isInside(const Point<T>& point) const {
	return ((point.x >= m_X) && (point.y >= m_Y) && (point.x <= m_X+m_Width) && (point.y <= m_Y+m_Height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::isInside(const T& x,const T& y) const {
	return ((x >= m_X) && (y >= m_Y) && (x <= m_X+m_Width) && (y <= m_Y+m_Height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::isInside(const Rectangle<T>& r) const {
	return ((r.m_X >= m_X) && (r.m_Y >= m_Y) && (r.m_X+r.m_Width <= m_X+m_Width) && (r.m_Y+r.m_Height <= m_Y+m_Height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::isInside(const T& x, const T& y, const T& width, const T& height) const {
	return ((x >= m_X) && (y >= m_Y) && (x+width <= m_X+m_Width) && (y+height <= m_Y+m_Height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::intersects(const Rectangle<T>& r) const {
	T ax2 = m_X + m_Width;
	if ((m_X < r.m_X) && (ax2 <= r.m_X))
		return false;

	T ay2 = m_Y + m_Height;
	if ((m_Y < r.m_Y) && (ay2 <= r.m_Y))
		return false;

	T bx2 = r.m_X + r.m_Width;
	if ((m_X >= bx2) && (ax2 > bx2))
		return false;

	T by2 = r.m_Y + r.m_Height;
	if ((m_Y >= by2) && (ay2 > by2))
		return false;

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::isEmpty() const {
	return ((m_Width == 0) || (m_Height == 0));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T Rectangle<T>::getWidth() const {
	return m_Width;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T Rectangle<T>::getHeight() const {
	return m_Height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setWidth(const T& w) {
	m_Width = w;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setHeight(const T& h) {
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Dimension<T> Rectangle<T>::getSize() const {
	return Dimension<T>(m_Width, m_Height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setSize(const Dimension<T>& d) {
	m_Width = d.width;
	m_Height = d.height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setSize(const T& w, const T& h) {
	m_Width = w;
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T Rectangle<T>::getX() const {
	return m_X;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T Rectangle<T>::getY() const {
	return m_Y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setX(const T& w) {
	m_X = w;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setY(const T& h) {
	m_Y = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Point<T> Rectangle<T>::getOrigin() const {
	return Point<T>(m_X, m_Y);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setOrigin(const Point<T>& d) {
	m_X = d.x;
	m_Y = d.y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Rectangle<T>::setOrigin(const T& x, const T& y) {
	m_X = x;
	m_Y = y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T> Rectangle<T>::operator+(const Point<T> &p) const {
	return Rectangle(m_X+p.x, m_Y+p.y, m_Width, m_Height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T> Rectangle<T>::operator-(const Point<T> &p) const {
	return Rectangle(m_X-p.x, m_Y-p.y, m_Width, m_Height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T> Rectangle<T>::operator+(const Dimension<T> &d) const {
	return Rectangle(m_X, m_Y, m_Width+d.width, m_Height+d.height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T> Rectangle<T>::operator-(const Dimension<T> &d) const {
	return Rectangle(m_X, m_Y, m_Width-d.width, m_Height-d.height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T> Rectangle<T>::operator&(const Rectangle<T> &r) const {
	T x = max(m_X, r.m_X);
	T y = max(m_Y, r.m_Y);
	T w = max(min(m_X + m_Width, r.m_X + r.m_Width) - x, T(0));
	T h = max(min(m_Y + m_Height, r.m_Y + r.m_Height) - y, T(0));
	return Rectangle(x,y,w,h);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Rectangle<T> Rectangle<T>::operator|(const Rectangle<T> &r) const {
	T x = min(m_X, r.m_X);
	T y = min(m_Y, r.m_Y);
	return Rectangle(x, y, max(m_X + m_Width, r.m_X + r.m_Width) - x, max(m_Y + m_Height, r.m_Y + r.m_Height) - y);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Rectangle<T>& Rectangle<T>::operator+=(const Point<T> &p) {
	m_X += p.x;
	m_Y += p.y;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Rectangle<T>& Rectangle<T>::operator-=(const Point<T> &p) {
	m_X -= p.x;
	m_Y -= p.y;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Rectangle<T>& Rectangle<T>::operator+=(const Dimension<T> &d) {
	m_Width += d.width;
	m_Height += d.height;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Rectangle<T>& Rectangle<T>::operator-=(const Dimension<T> &d) {
	m_Width -= d.width;
	m_Heigh -= d.height;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Rectangle<T>& Rectangle<T>::operator&=(const Rectangle<T> &r) {
	T x = max(m_X, r.m_X);
	T y = max(m_Y, r.m_Y);
	T w = max(min(m_X + m_Width, r.m_X + r.m_Width) - x, T(0));
	T h = max(min(m_Y + m_Height, r.m_Y + r.m_Height) - y, T(0));
	m_X = x;
	m_Y = y;
	m_Width = w;
	m_Height = h;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Rectangle<T>& Rectangle<T>::operator|=(const Rectangle<T> &r) {
	T x = min(m_X, r.m_X);
	T y = min(m_Y, r.m_Y);
	T w = max(m_X + m_Width, r.m_X + r.m_Width) - x;
	T h = max(m_Y + m_Height, r.m_Y + r.m_Height) - y;
	m_X = x;
	m_Y = y;
	m_Width = w;
	m_Height = h;
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::operator==(const Rectangle<T>& r) const {
	return ((m_X == r.m_X) && (m_Y == r.m_Y) && (m_Width == r.m_Width) && (m_Height == r.m_height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Rectangle<T>::operator!=(const Rectangle<T>& r) const {
	return ((m_X != r.m_X) || (m_Y != r.m_Y) || (m_Width != r.m_Width) || (m_Height != r.m_height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class S> 
inline Rectangle<T>::operator Rectangle<S>() const {
	return Rectangle<S>((S)m_X, (S)m_Y, (S)m_Width, (S)m_Height);
}

}; };
#endif
