/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS_TRANSFORM_INL
#define __GRAPHICS_TRANSFORM_INL

namespace magnum { namespace graphics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Transform<T>::setIdentity() {
	m_Matrix[0][0] =
	m_Matrix[1][1] = T(1);
	m_Matrix[0][1] =
	m_Matrix[0][2] =
	m_Matrix[1][0] =
	m_Matrix[1][2] = T(0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T> Transform<T>::getTransformX() const {
	return Point<T>(m_Matrix[0][0], m_Matrix[0][1]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Transform<T>::setTransformX(const Point<T>& p) {
	m_Matrix[0][0] = p.getX();
	m_Matrix[0][1] = p.getY();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Transform<T>::setTransformX(T x, T y) {
	m_Matrix[0][0] = x;
	m_Matrix[0][1] = y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T> Transform<T>::getTransformY() const {
	return Point<T>(m_Matrix[1][0], m_Matrix[1][1]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Transform<T>::setTransformY(const Point<T>& p) {
	m_Matrix[1][0] = p.getX();
	m_Matrix[1][1] = p.getY();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Transform<T>::setTransformY(T x, T y) {
	m_Matrix[1][0] = x;
	m_Matrix[1][1] = y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T> Transform<T>::getTranslation() const {
	return Point<T>(m_Matrix[0][2], m_Matrix[1][2]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Transform<T>::setTranslation(const Point<T>& p) {
	m_Matrix[0][2] = p.getX();
	m_Matrix[1][2] = p.getY();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Transform<T>::setTranslation(T x, T y) {
	m_Matrix[0][2] = x;
	m_Matrix[1][2] = y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T>	Transform<T>::getScale() const {
	return Point<T>(
		(T)sqrt(m_Matrix[0][0]*m_Matrix[0][0] + m_Matrix[0][1]*m_Matrix[0][1]),
		(T)sqrt(m_Matrix[1][0]*m_Matrix[1][0] + m_Matrix[1][1]*m_Matrix[1][1]));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void	Transform<T>::setScale(const Point<T>& p) {
	m_Matrix[0][0] = p.getX();
	m_Matrix[0][1] = 
	m_Matrix[1][0] = T(0);
	m_Matrix[1][1] = p.getY();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void	Transform<T>::setScale(T x, T y) {
	m_Matrix[0][0] = x;
	m_Matrix[0][1] = 
	m_Matrix[1][0] = T(0);
	m_Matrix[1][1] = y;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T>	Transform<T>::operator*(const Point<T>& p) const {
	return Point<T>(
		p.getX()*m_Matrix[0][0] + p.getY()*m_Matrix[0][1] + m_Matrix[0][2],
		p.getX()*m_Matrix[1][0] + p.getY()*m_Matrix[1][1] + m_Matrix[1][2]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T> Transform<T>::operator%(const Point<T>& p) const {
	T x = p.getX() - m_Matrix[0][2];
	T y = p.getY() - m_Matrix[1][2];
	T det = m_Matrix[1][1]*m_Matrix[0][0] - m_Matrix[0][1]*m_Matrix[1][0];
	return Point<T>(x * m_Matrix[1][1] / det, y * m_Matrix[0][0] / det);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Rectangle<T> Transform<T>::operator*(const Rectangle<T>& r) const {
	return Rectangle<T>(
		r.getX()*m_Matrix[0][0] + r.getY()*m_Matrix[0][1] + m_Matrix[0][2],
		r.getX()*m_Matrix[1][0] + r.getY()*m_Matrix[1][1] + m_Matrix[1][2],
		r.getWidth()*m_Matrix[0][0] + r.getHeight()*m_Matrix[0][1],
		r.getWidth()*m_Matrix[1][0] + r.getHeight()*m_Matrix[1][1]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Rectangle<T> Transform<T>::operator%(const Rectangle<T>& p) const {
	T x = p.getX() - m_Matrix[0][2];
	T y = p.getY() - m_Matrix[1][2];
	T det = m_Matrix[1][1]*m_Matrix[0][0] - m_Matrix[0][1]*m_Matrix[1][0];
	return Rectangle<T>(
		x * m_Matrix[1][1] / det, 
		y * m_Matrix[0][0] / det,
		p.getWidth() *  m_Matrix[1][1] / det,
		p.getHeight() *  m_Matrix[0][0] / det);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Dimension<T> Transform<T>::operator*(const Dimension<T>& d) const {
	return Dimension<T>(
		d.getWidth()*m_Matrix[0][0] + d.getHeight()*m_Matrix[0][1],
		d.getWidth()*m_Matrix[1][0] + d.getHeight()*m_Matrix[1][1]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Dimension<T> Transform<T>::operator%(const Dimension<T>& p) const {
	T det = m_Matrix[1][1]*m_Matrix[0][0] - m_Matrix[0][1]*m_Matrix[1][0];
	return Dimension<T>(
		p.getWidth() *  m_Matrix[1][1] / det,
		p.getHeight() *  m_Matrix[0][0] / det);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Transform<T> Transform<T>::operator+(const Point<T>& p) const  {
	Transform<T> t;
	t.m_Matrix[0][0] = m_Matrix[0][0];
	t.m_Matrix[0][1] = m_Matrix[0][1];
	t.m_Matrix[0][2] = m_Matrix[0][2] + p.getX();
	t.m_Matrix[1][0] = m_Matrix[1][0];
	t.m_Matrix[1][1] = m_Matrix[1][1];
	t.m_Matrix[1][2] = m_Matrix[1][2] + p.getY();
	return t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Transform<T> Transform<T>::operator-(const Point<T>& p) const {
	Transform<T> t;
	t.m_Matrix[0][0] = m_Matrix[0][0];
	t.m_Matrix[0][1] = m_Matrix[0][1];
	t.m_Matrix[0][2] = m_Matrix[0][2] - p.getX();
	t.m_Matrix[1][0] = m_Matrix[1][0];
	t.m_Matrix[1][1] = m_Matrix[1][1];
	t.m_Matrix[1][2] = m_Matrix[1][2] - p.getY();
	return t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const Transform<T>&	Transform<T>::operator+=(const Point<T>& p) {
	m_Matrix[0][2] += p.getX();
	m_Matrix[1][2] += p.getY();
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const Transform<T>& Transform<T>::operator-=(const Point<T>& p) {
	m_Matrix[0][2] -= p.getX();
	m_Matrix[1][2] -= p.getY();
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Transform<T> Transform<T>::operator*(const Transform<T>& t) const {
	Transform<T> s;
	s.m_Matrix[0][0] = m_Matrix[0][0]*t.m_Matrix[0][0] + m_Matrix[0][1]*t.m_Matrix[1][0];
	s.m_Matrix[0][1] = m_Matrix[0][0]*t.m_Matrix[0][1] + m_Matrix[0][1]*t.m_Matrix[1][1];
	s.m_Matrix[1][0] = m_Matrix[1][0]*t.m_Matrix[0][0] + m_Matrix[1][1]*t.m_Matrix[1][0];
	s.m_Matrix[1][1] = m_Matrix[1][0]*t.m_Matrix[0][1] + m_Matrix[1][1]*t.m_Matrix[1][1];
	s.m_Matrix[0][2] = m_Matrix[0][0]*t.m_Matrix[0][2] + m_Matrix[0][1]*t.m_Matrix[1][2] + m_Matrix[0][2];
	s.m_Matrix[1][2] = m_Matrix[1][0]*t.m_Matrix[0][2] + m_Matrix[1][1]*t.m_Matrix[1][2] + m_Matrix[1][2];
	return s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const Transform<T>&	Transform<T>::operator*=(const Transform<T>& t) {
	T m[2][3];
	m[0][0] = m_Matrix[0][0]*t.m_Matrix[0][0] + m_Matrix[0][1]*t.m_Matrix[1][0];
	m[0][1] = m_Matrix[0][0]*t.m_Matrix[0][1] + m_Matrix[0][1]*t.m_Matrix[1][1];
	m[1][0] = m_Matrix[1][0]*t.m_Matrix[0][0] + m_Matrix[1][1]*t.m_Matrix[1][0];
	m[1][1] = m_Matrix[1][0]*t.m_Matrix[0][1] + m_Matrix[1][1]*t.m_Matrix[1][1];
	m[0][2] = m_Matrix[0][0]*t.m_Matrix[0][2] + m_Matrix[0][1]*t.m_Matrix[1][2] + m_Matrix[0][2];
	m[1][2] = m_Matrix[1][0]*t.m_Matrix[0][2] + m_Matrix[1][1]*t.m_Matrix[1][2] + m_Matrix[1][2];
	memcpy(m_Matrix, m, sizeof(m_Matrix));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T* Transform<T>::operator[](unsigned int i) const {
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T* Transform<T>::operator[](unsigned int i) {
	return m_Matrix[i];
}


}; };

#endif
