/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS_TRANSLATOR_JPG_H
#define __GRAPHICS_TRANSLATOR_JPG_H

#include "graphics.image.h"
#include "io.translator.h"


#if defined(__FORWINDOWS__)
#define XMD_H
#undef FAR
#endif

#include <stdio.h>

extern "C" {
#include "jpeglib.h"
#include "jerror.h"
};

#if defined(__FORWINDOWS__)
#undef XMD_H
#endif


#include <setjmp.h>


namespace magnum { namespace graphics {
using magnum::object::Object;
using magnum::types::String;
using magnum::io::File;
using magnum::io::InputStream;
using magnum::io::OutputStream;
using magnum::io::Translator;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorJPG : public Translator<Image> {
DeclareClass(TranslatorJPG, Translator<Image>);
private:
	struct SourceMgr {
		struct jpeg_source_mgr pub;	/* public fields */
		InputStream*	infile;		/* source stream */
		byte*			buffer;		/* start of buffer */
		bool			start_of_file;	/* have we gotten any data yet? */
	};
	
	struct ErrorMgr {
		struct jpeg_error_mgr pub;	/* "public" fields */
		jmp_buf setjmp_buffer;	/* for return to caller */
	};

	jpeg_decompress_struct	m_Decompressor;
	SourceMgr				m_Source;
	ErrorMgr				m_ErrorMgr;


public:
	TranslatorJPG();
	~TranslatorJPG();

protected:
	Image*	onDecode(InputStream* ,long size = Autosize);
	long	onEncode(OutputStream* ,const Image* );
	bool	onIdentify(File* );


private:
	static void			initSource(j_decompress_ptr );
	static boolean		fillSourceBuffer(j_decompress_ptr );
	static void			skipSourceData(j_decompress_ptr , long );
	static void			termSource(j_decompress_ptr );
	static void			errorExit(j_common_ptr cinfo);
};


}; };

#endif
