/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_PLAINMATERIAL_INL
#define __GRAPHICS3D_PLAINMATERIAL_INL



namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline ColourMaterial::ColourMaterial() {
	m_Shininess = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> ColourMaterial::getAmbient() const {
	return m_Ambient;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::setAmbient(const Colour<float>& c) {
	m_Ambient = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> ColourMaterial::getDiffuse() const {
	return m_Diffuse;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::setDiffuse(const Colour<float>& c) {
	m_Diffuse = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> ColourMaterial::getSpecular() const {
	return m_Specular;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::setSpecular(const Colour<float>& c) {
	m_Specular = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> ColourMaterial::getEmission() const {
	return m_Emission;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::setEmission(const Colour<float>& c) {
	m_Emission = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float ColourMaterial::getShininess() const {
	return m_Shininess;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::setShininess(float s) {
	m_Shininess = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::selectAmbient() const {
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, (const float *)m_Ambient);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::selectDiffuse() const {
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, (const float *)m_Diffuse);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::selectSpecular() const {
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, (const float *)m_Specular);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::selectEmission() const {
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, (const float *)m_Emission);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ColourMaterial::selectShininess() const {
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, m_Shininess);
}


}; };

#endif