/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_CONTEXT_H
#define __GRAPHICS3D_CONTEXT_H

#include "object.h"
#include "container.chain.h"
#include "system.thread.h"
#include "graphics3d.framebuffer.h"
#include "graphics3d.matrixstack.h"
#include "graphics3d.materialstack.h"
#include "graphics3d.lightstack.h"
#include "graphics3d.texturestack.h"
#include "graphics3d.shaderstack.h"
#include "graphics3d.modestack.h"
#include "graphics3d.extensions.h"
#include "graphics3d.mesh.h"


namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::system::Thread;
using magnum::container::Chain;
using magnum::container::Chainable;

class Resource;
class IndexBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Context : public Object, private Chainable<Context> {
DeclareClass(Context, Object);
friend class Resource;
friend class IndexBuffer;
private:
	class Startup {
	public:
		Startup();
	};
	friend Startup;

	static Startup			g_Startup;
	static Chain<Context>	g_Contexts;

#if defined(__NATIVETLS__)
	static __declspec(thread) Context*	g_Current;
#else
	static Thread::Storage	g_Current;
#endif

	Extensions				m_Extensions;
	FrameBuffer				m_FrameBuffer;

	MatrixStack				m_MatrixStack;
	MaterialStack			m_MaterialStack;
	LightStack				m_LightStack;
	TextureStack			m_TextureStack;
	ShaderStack				m_ShaderStack;
	ModeStack				m_ModeStack;

	// This is a dirty hack, because Index buffer are not part of the internal
	// OpenGL state, except for IBOs.
	void*					m_Indices;
	unsigned int			m_IndexType;
	unsigned int			m_IndexSize;

#if defined(__FORWINDOWS__)
	HDC						m_DeviceHandle;
	HWND					m_WindowHandle;
	HGLRC					m_OglContext;
#endif

public:
	enum CullMode {
		CullFront = 1,
		CullBack = 2,
		CullNone = 3
	};

private:
	Context();

public:
	static Context*			create();

	inline static Context*	getCurrent();
	bool					makeCurrent();

	inline FrameBuffer*		getFrameBuffer();

	inline MaterialStack*	getMaterialStack();
	inline LightStack*		getLightStack();
	inline MatrixStack*		getMatrixStack();
	inline TextureStack*	getTextureStack();
	inline ShaderStack*		getShaderStack();
	inline ModeStack*		getModeStack();
	inline const Extensions* getExtensions() const;

	inline void				flush();
	inline void				swap();

	inline void				setCulling(CullMode );
	inline CullMode			getCulling() const;

	inline void				setPolygonOffset(float ,float );

	inline void				renderVertices(Mesh::Primitive p, unsigned int start, unsigned int size);
	inline void				renderVertices(Mesh::Primitive p, unsigned int size);
	inline void				renderIndices(Mesh::Primitive p, unsigned int start, unsigned int size);
	inline void				renderIndices(Mesh::Primitive p, unsigned int size);
	
#if defined(__FORWINDOWS__)
	inline HDC				getDeviceHandle() const;
	inline HWND				getWindowHandle() const;
#endif

private:
	static Context*			searchCurrent();
	static Context*			createCurrent();
};

}; };


#endif
