/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_CONTEXT_INL
#define __GRAPHICS3D_CONTEXT_INL


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Context* Context::getCurrent() {
#if defined(__NATIVETLS__)
	Context* c = g_Current;
#else
	Context* c = (Context *)Thread::getTLS(g_Current);
#endif

	if (c == NULL)
		return searchCurrent();
	ASSERT(c->m_OglContext == wglGetCurrentContext());

	return c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Extensions* Context::getExtensions() const {
	return &m_Extensions;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FrameBuffer* Context::getFrameBuffer() {
	return &m_FrameBuffer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline MaterialStack* Context::getMaterialStack() {
	return &m_MaterialStack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline LightStack* Context::getLightStack() {
	return &m_LightStack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline MatrixStack* Context::getMatrixStack() {
	return &m_MatrixStack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline TextureStack* Context::getTextureStack() {
	return &m_TextureStack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline ShaderStack* Context::getShaderStack() {
	return &m_ShaderStack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline ModeStack* Context::getModeStack() {
	return &m_ModeStack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::flush() {
	glFlush();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::swap() {
	SwapBuffers(m_DeviceHandle);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::setCulling(CullMode m) {
	if (m == CullFront) {
		glEnable(GL_CULL_FACE);
		glCullFace(GL_FRONT);
	} else if (m == CullBack) {
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
	} else {
		glDisable(GL_CULL_FACE);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Context::CullMode Context::getCulling() const {
	if (glIsEnabled(GL_CULL_FACE) == GL_FALSE)
		return CullNone;
	int m;
	glGetIntegerv(GL_CULL_FACE_MODE, &m);
	if (m == GL_FRONT)
		return CullFront;
	else
		return CullBack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::setPolygonOffset(float m,float r) {
	ASSERT(glPolygonOffsetEXT);
	if ((m == 0) && (r == 0.0f)) {
		glDisable(GL_POLYGON_OFFSET_FILL);
	} else {
		glPolygonOffsetEXT(m, r);
		glEnable(GL_POLYGON_OFFSET_FILL);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Context::renderVertices(Mesh::Primitive p, unsigned int start, unsigned int size) {
	glDrawArrays(p, start, size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Context::renderVertices(Mesh::Primitive p, unsigned int size) {
	glDrawArrays(p, 0, size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Context::renderIndices(Mesh::Primitive p, unsigned int start, unsigned int size) {
	glDrawElements(p, size, m_IndexType, (byte *)m_Indices + start*m_IndexSize);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Context::renderIndices(Mesh::Primitive p, unsigned int size) {
	glDrawElements(p, size, m_IndexType, (byte *)m_Indices);
}


#if defined(__FORWINDOWS__)
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HWND Context::getWindowHandle() const {
	return m_WindowHandle;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HDC Context::getDeviceHandle() const {
	return m_DeviceHandle;
}
#endif

}; };


#endif
