/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_INDEXBUFFER_INL
#define __GRAPHICS3D_INDEXBUFFER_INL

namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* IndexBuffer::lock(LockFlag f) {
	ASSERT((f >= ReadOnly) && (f <= Discard));
	ASSERT(!m_Locked);
	if (m_OglBuffer != 0) {
		glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER, m_OglBuffer);
		m_Buffer = glMapBufferARB(GL_ELEMENT_ARRAY_BUFFER, g_OglAcces[f]);
		if (m_Buffer)
			m_Locked = true;
	} else {
		m_Locked = true;
	}
	return m_Buffer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	IndexBuffer::unlock() {
	ASSERT(m_Locked);
	if (m_OglBuffer != 0) {
		glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER, m_OglBuffer);
		glUnmapBufferARB(GL_ELEMENT_ARRAY_BUFFER);
		m_Buffer = NULL;
	}
	m_Locked = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool IndexBuffer::isLocked() const {
	return (m_Locked);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int IndexBuffer::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline IndexBuffer::Type IndexBuffer::getType() const {
	return (Type)m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	IndexBuffer::select() const {
	Context* context = getContext();
	context->m_Indices = bindBuffer();
	context->m_IndexType = m_OglType;
	context->m_IndexSize = m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* IndexBuffer::bindBuffer() const {
	if (m_OglBuffer) {
		glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER, m_OglBuffer);
		return NULL;
	} else {
		if (glBindBufferARB)
			glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER, NULL);
		return m_Buffer;
	}
}


}; };

#endif
