/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_LIGHTSTACK_INL
#define __GRAPHICS3D_LIGHTSTACK_INL

namespace magnum {namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline LightStack::LightStack() {
	for (int i = 0; i < Light::MaxLights; i++) {
		m_Light[i] = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Light* LightStack::getLight(int i) const {
	ASSERT((i >= 0) && (i < Light::MaxLights));
	return m_Light[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void LightStack::enableLighting(unsigned int flags) {
	glEnable(GL_LIGHTING);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, (flags & LocalViewer) ? GL_TRUE : GL_FALSE);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE , (flags & TwoSided) ? GL_TRUE : GL_FALSE);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void LightStack::disableLighting() {
	glDisable(GL_LIGHTING);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool LightStack::isLighting() const {
	GLboolean b;
	glGetBooleanv(GL_LIGHTING, &b);
	return (b == GL_TRUE);
}

}; };

#endif
