/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_LOCATION_INL
#define __GRAPHICS3D_LOCATION_INL


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline LocationArray<L>::LocationArray() {
	m_Length = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline void LocationArray<L>::clear() {
	m_Locations.clear();
	m_Length = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline unsigned int LocationArray<L>::getSize() const {
	return m_Locations.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline unsigned int LocationArray<L>::getLength() const {
	return m_Length;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline Types::Type LocationArray<L>::getType(const char* name) const {
	return at(name).m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline L LocationArray<L>::getLocation(const char* name) const {
	return at(name).m_Location;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline unsigned int LocationArray<L>::getSize(const char* name) const {
	return ((at(name).m_Type >> Types::SizeShift) & Types::SizeMask);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline const char* LocationArray<L>::getDescription(const char* name) const {
	return at(name).m_Description;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline const LocationArray<L>& LocationArray<L>::operator=(const LocationArray& fmt) {
	m_Locations = fmt.m_Locations;
	m_Length = fmt.m_Length;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline LocationArray<L>::Iterator LocationArray<L>::begin() const {
	return m_Locations.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> inline LocationArray<L>::Iterator LocationArray<L>::end() const {
	return m_Locations.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> void LocationArray<L>::shrink() {
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (m_Locations[i].m_Type == 0) {
			m_Locations.remove(i);
			i--;
		}
	}
	m_Locations.shrink();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> void LocationArray<L>::append(const LocationArray<L>& fmt) {
	unsigned int i = m_Locations.getSize();
	unsigned int offset = m_Length;

	m_Locations.append(fmt.m_Locations);
	m_Length += fmt.m_Length;
	for (; i < m_Locations.getSize(); i++) {
		m_Locations[i].m_Offset += offset;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> void LocationArray<L>::add(const char* name, const char* desc, Types::Type t, L l) {
	Location cmp;
	strncpy(cmp.m_Description, desc, sizeof(cmp.m_Description));
	strncpy(cmp.m_Name, name, sizeof(cmp.m_Name));
	cmp.m_Type = t;
	cmp.m_Location = l;
	m_Length += ((t >> Types::SizeShift) & Types::SizeMask);
	m_Locations.insert(cmp);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> void LocationArray<L>::remove(const char* name) {
	unsigned int size = 0;
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (strncmp(m_Locations[i].m_Name, name, sizeof(m_Locations[i].m_Name)) == 0) {
			size = ((m_Locations[i].m_Type >> Types::SizeShift) & Types::SizeMask);
			m_Length -= size;
			m_Locations.remove(i);
			break;
		}
	}
	for (;i < m_Locations.getSize(); i++) {
		m_Locations[i].m_Offset -= size;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> bool LocationArray<L>::contains(const char* name) const {
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (strncmp(m_Locations[i].m_Name, name, sizeof(m_Locations[i].m_Name)) == 0)
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> bool LocationArray<L>::contains(L l) const {
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (m_Locations[i].m_Location == l)
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> const LocationArray<L>::Location& LocationArray<L>::at(const char* name) const {
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (strncmp(m_Locations[i].m_Name, name, sizeof(m_Locations[i].m_Name)) == 0)
			return m_Locations[i];
	}
	ASSERT(false);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> const LocationArray<L>::Location& LocationArray<L>::operator[](const char* name) const {
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (strncmp(m_Locations[i].m_Name, name, sizeof(m_Locations[i].m_Name)) == 0)
			return m_Locations[i];
	}
	ASSERT(false);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> const LocationArray<L>::Location& LocationArray<L>::at(L l) const {
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (m_Locations[i].m_Location == l)
			return m_Locations[i];
	}
	ASSERT(false);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> const LocationArray<L>::Location& LocationArray<L>::operator[](L l) const {
	for (unsigned int i = 0; i < m_Locations.getSize(); i++) {
		if (m_Locations[i].m_Location == l)
			return m_Locations[i];
	}
	ASSERT(false);
}

}; };

#endif
