/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MATRIXSTACK_H
#define __GRAPHICS3D_MATRIXSTACK_H

#include "object.h"
#include "math.matrix.h"
#include "math.vector3.h"

namespace magnum {namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::math::Vector3;

class Context;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MatrixStack : public Object {
DeclareClass(MatrixStack, Object);
friend Context;
public:
	enum Mode {
		Texture = GL_TEXTURE,
		Projection = GL_PROJECTION,
		ModelView = GL_MODELVIEW,
	};

private:
	Mode			m_Mode;

private:
	inline MatrixStack();

public:
	inline void		setProjection(const Matrix4<float>& );
	inline void		identityProjection();
	inline void		pushProjection();
	inline void		popProjection();

	inline void		setView(const Matrix4<float>& );
					
	inline void		setModel(const Matrix4<float>& );
	inline void		pushModel();
	inline void		popModel();
	inline void		multModel(const Matrix4<float>& );
	inline void		rotateModel(float , const Vector3<float>& );
	inline void		translateModel(const Vector3<float>& );
	inline void		scaleModel(const Vector3<float>& );
	inline void		identityModel();
					
	inline void		setTexture(const Matrix4<float>& );
	inline void		pushTexture();
	inline void		popTexture();
	inline void		multTexture(const Matrix4<float>& );
	inline void		rotateTexture(float , const Vector3<float>& );
	inline void		translateTexture(const Matrix4<float>& );
	inline void		scaleTexture(const Matrix4<float>& );
	inline void		identityTexture();
					
	inline void		selectMode(Mode );
	inline void		forceSelectMode(Mode );
	inline Mode		getMode() const;

	inline void		set(const Matrix4<float>& );
	inline Matrix4<float> get() const;
	inline void		push();
	inline void		pop();
	inline void		mult(const Matrix4<float>& );
	inline void		translate(const Vector3<float>& );
	inline void		scale(const Vector3<float>& );
	inline void		rotate(float , const Vector3<float>& );
	inline void		identity();
	inline void		ortho(float ,float, float ,float ,float ,float );
	inline void		ortho(const Rectangle<float>& ,float ,float );
};


}; };

#endif
