/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESHBUILDER_H
#define __GRAPHICS3D_MESHBUILDER_H

#include "math.vector2.h"
#include "math.vector3.h"
#include "container.array.h"
#include "graphics.colour.h"
#include "graphics3d.vertexformat.h"
#include "graphics3d.indexmesh.h"

namespace magnum { namespace graphics3d {
using magnum::math::Vector3;
using magnum::math::Vector2;
using magnum::graphics::Colour;
using magnum::container::Array;
using magnum::object::Pointer;

class Appearance;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MeshBuilder : public Object {
DeclareClass(MeshBuilder, Object);
private:
	struct Vertex {
		Vector3<float>		m_Vertex;
		Vector3<float>		m_Normal;
		Colour<float>		m_Colour;
		Vector2<float>		m_TexCoord;
	};

	Mesh::Primitive		m_Primitive;
	Pointer<Appearance>	m_Appearance;
	unsigned int		m_Attributes;
	VertexFormat		m_VertexFormat;
	Vertex				m_Current;
	
	Array<unsigned int>	m_Indices;
	Array<Vertex>		m_Vertices;
	unsigned int		m_MaxIndex;

public:
	MeshBuilder();
	
	inline void			setPrimitive(Mesh::Primitive );
	inline Mesh::Primitive getPrimitive() const;
	
	inline void			setAppearance(Appearance* );
	inline Appearance*	getAppearance();
	
	void				setAttributes(unsigned int );
	inline unsigned int getAttributes() const;
	inline const VertexFormat& getVertexFormat() const;
	
	inline void			vertex(const Vector3<float>& );
	inline void			vertex(float ,float ,float );
	inline void			normal(const Vector3<float>& );
	inline void			normal(float ,float ,float );
	inline void			texCoord(const Vector2<float>& );
	inline void			texCoord(float ,float );
	inline void			colour(const Colour<float>& );
	inline void			colour(float ,float ,float );
	
	inline void			index(unsigned int );
	
	void				clear();
	void				finish();
	Mesh*				build();
};

}; };

#endif
