/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESHBUILDER_INL
#define __GRAPHICS3D_MESHBUILDER_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::setAppearance(Appearance* app) {
	m_Appearance = app;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Appearance* MeshBuilder::getAppearance() {
	return m_Appearance;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::setPrimitive(Mesh::Primitive p) {
	m_Primitive = p;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mesh::Primitive MeshBuilder::getPrimitive() const {
	return m_Primitive;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int MeshBuilder::getAttributes() const {
	return m_Attributes;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const VertexFormat& MeshBuilder::getVertexFormat() const {
	return m_VertexFormat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::vertex(const Vector3<float>& v) {
	m_Current.m_Vertex = v;
	m_Vertices.insert(m_Current);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::vertex(float x,float y,float z) {
	m_Current.m_Vertex.x = x;
	m_Current.m_Vertex.y = y;
	m_Current.m_Vertex.z = z;
	m_Vertices.insert(m_Current);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::normal(const Vector3<float>& n) {
	m_Current.m_Normal = n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::normal(float x,float y,float z) {
	m_Current.m_Normal.x = x;
	m_Current.m_Normal.y = y;
	m_Current.m_Normal.z = z;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::texCoord(const Vector2<float>& t) {
	m_Current.m_TexCoord = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::texCoord(float x,float y) {
	m_Current.m_TexCoord.x = x;
	m_Current.m_TexCoord.y = y;
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::colour(const Colour<float>& n) {
	m_Current.m_Colour = n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::colour(float r,float g,float b) {
	m_Current.m_Colour.R = r;
	m_Current.m_Colour.G = g;
	m_Current.m_Colour.B = b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MeshBuilder::index(unsigned int idx) {
	if (idx > m_MaxIndex)
		m_MaxIndex = idx;
	m_Indices.insert(idx);
}

}; };

#endif
