/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESHSTRIPPER_H
#define __GRAPHICS3D_MESHSTRIPPER_H

#include "math.graph.h"
#include "object.pointer.h"
#include "container.array.h"
#include "container.tuple.h"
#include "container.pair.h"
#include "container.map.h"
#include "container.compound.h"
#include "container.priorityheap.h"
#include "algorithm.sort.h"
#include "algorithm.find.h"

namespace magnum { namespace graphics3d {
using namespace magnum::container;
using namespace magnum::math;
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::container::Array;
using magnum::container::Tuple;
using magnum::container::Pair;
using magnum::container::Map;
using magnum::container::Compound;
using magnum::container::PriorityHeap;
using magnum::algorithm::sort;
using magnum::algorithm::find;
using magnum::algorithm::findfirst;
using magnum::math::Graph;

class IndexMesh;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class TriangleStripper : public Object {
DeclareClass(TriangleStripper, Object);
private:
	typedef Pair<T,T>			Edge;
	typedef Compound<Edge, int>	TriangleEdge;
	typedef Array<T>			Strip;
	typedef Pointer<Strip>		StripPointer;

	struct Triangle {
		T						v0, v1, v2;
		PriorityHeap<int>::Iterator	m_Degree;
		int						m_Index;
	};

	Graph<Triangle, Edge>		m_Mesh;
	Array<TriangleEdge>			m_Edges;
	PriorityHeap<int>			m_Degrees;
	Array<StripPointer>			m_Strips;
	unsigned int				m_StripSize;
	Array<unsigned int>			m_Cache;
	unsigned int				m_CacheSize;
	T							m_Index[2];

public:
	void				clear();
	void				insert(const T* ,unsigned int numIndices);
	void				finish();

	Array<T>*			strip();						

	inline void			setCacheSize(unsigned int );
	inline unsigned int getCacheSize() const;

private:
	inline bool			findCache(unsigned int );
	inline void			clearCache();
	inline void			insertCache(unsigned int );

	void				connect(const TriangleEdge& edge);
	void				removeTriangle(unsigned int );
	T					getConnection(unsigned int tri, Edge* edge, bool odd);

	Array<T>*			findStrip();
	unsigned int		getStripSize(unsigned int first, const typename Graph<Triangle, Edge>::Edge& );
	Array<T>*			buildStrip(unsigned int first, const typename Graph<Triangle, Edge>::Edge& );
	Array<T>*			buildSingleton(unsigned int first);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MeshStripper : public Object {
DeclareClass(MeshStripper, Object);
private:

public:
	bool		process(IndexMesh* );
};

}; };

#endif
