/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODE_H
#define __GRAPHICS3D_MODE_H

#include "math.range.h"
#include "graphics.colour.h"
#include "graphics3d.resource.h"
#include "graphics3d.opengl.h"

namespace magnum { namespace graphics3d {
using magnum::math::Range;
using magnum::graphics::Colour;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Mode : public Resource {
DeclareClass(Mode, Resource);
public:
	enum TestFunction {
		Never = GL_NEVER,
		Less = GL_LESS,
		Equal = GL_EQUAL,
		LessEqual = GL_LEQUAL,
		Greater = GL_GREATER,
		GreaterEqual = GL_GEQUAL,
		NotEqual = GL_NOTEQUAL,
		Always = GL_ALWAYS,
		Disable = -1,
	};

	enum StencilMode {
		Keep = GL_KEEP,
//		Zero = GL_ZERO,
		Replace = GL_REPLACE,
		Increment = GL_INCR,
		Decrement = GL_DECR,
		IncrementWrap = GL_INCR_WRAP_EXT,
		DecrementWrap = GL_DECR_WRAP_EXT,
		Invert = GL_INVERT,
	};

	enum Equation {
		Add = GL_FUNC_ADD,
		Subtract = GL_FUNC_SUBTRACT,
		ReverseSubtract = GL_FUNC_REVERSE_SUBTRACT,
		Min = GL_MIN,
		Max = GL_MAX
	};

	enum BlendMode {
		Zero = GL_ZERO,
		One = GL_ONE,
		DstColour = GL_DST_COLOR,
		DstColourInv = GL_ONE_MINUS_DST_COLOR, 
		DstAlpha = GL_DST_ALPHA, 
		DstAlphaInv = GL_ONE_MINUS_DST_ALPHA,
		SrcColour = GL_SRC_COLOR,
		SrcColourInv = GL_ONE_MINUS_SRC_COLOR, 
		SrcAlpha = GL_SRC_ALPHA, 
		SrcAlphaInv = GL_ONE_MINUS_SRC_ALPHA,
		SrcAlphaSaturate = GL_SRC_ALPHA_SATURATE,
		ConstAlpha = GL_CONSTANT_ALPHA,
		ConstAlphaInv = GL_ONE_MINUS_CONSTANT_ALPHA,
		ConstColour = GL_CONSTANT_COLOR,
		ConstColourInv = GL_ONE_MINUS_CONSTANT_COLOR,
	};

private:
	Range<float>			m_DepthRange;
	TestFunction			m_DepthTest;
	bool					m_DepthWrite;
	StencilMode				m_StencilFail;
	StencilMode				m_StencilZFail;
	StencilMode				m_StencilPass;
	TestFunction			m_StencilTest;
	unsigned int			m_StencilRef;
	unsigned int			m_StencilMask;
	unsigned int			m_StencilWrite;
	BlendMode				m_BlendSource;
	BlendMode				m_BlendDestination;
	Equation				m_BlendEquation;
	Colour<float>			m_BlendColour;
	bool					m_RedWrite;
	bool					m_GreenWrite;
	bool					m_BlueWrite;
	bool					m_AlphaWrite;
	TestFunction			m_AlphaTest;
	float					m_AlphaRef;

public:
	Mode();
	inline ~Mode();

	inline void				select() const;
	inline void				forceSelect() const;
	inline void				deselect() const;
	static inline void		selectNone();

	// Depth Buffer
	inline void				setDepthRange(float , float);
	inline void				setDepthRange(const Range<float>& );
	inline Range<float>		getDepthRange() const;
	inline void				setDepthTest(TestFunction  );
	inline TestFunction		getDepthTest() const;
	inline void				setDepthWrite(bool );
	inline bool				getDepthWrite() const;

	// Stencil Buffer							
	inline void				setStencilMode(StencilMode ,StencilMode ,StencilMode );
	inline void				setStencilTest(TestFunction );
	inline void				setStencilTest(TestFunction , unsigned int ref, unsigned int mask);
	inline TestFunction		getStencilTest() const;
	inline void				setStencilReference(unsigned int );
	inline unsigned int		getStencilReference() const;
	inline void				setStencilMask(unsigned int );
	inline unsigned int		getStencilMask() const;
	inline void				setStencilWrite(unsigned int mask = 0xffffffff);
	inline unsigned int		getStencilWrite() const;

	// Blending
	inline void				setBlendMode(BlendMode ,BlendMode );
	inline void				setBlendEquation(Equation );
	inline Equation			getBlendEquation() const;
	inline void				setBlendColour(const Colour<float>& );
	inline Colour<float>	getBlendColour() const;
							
	// Colour/Alpha writing
	inline void				setColourWrite(bool r, bool g, bool b, bool a);
	inline void				setColourWrite(bool r);
			
	// Alphatest				
	inline void				setAlphaTest(TestFunction ,float ref);
	inline void				setAlphaTest(TestFunction );
	inline TestFunction		getAlphaTest() const;


protected:
	virtual void			onSelect() const;
	virtual void			onDeselect() const;
};

}; };


#endif
