/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODE_INL
#define __GRAPHICS3D_MODE_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mode::~Mode() {
	// Remove from all shader stacks!
	ModeStack* ss = getContext()->getModeStack();
	if (ss->m_Mode == this) {
		ss->m_Mode = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::select() const {
	ModeStack* ss = getContext()->getModeStack();
	const Mode* cur = ss->m_Mode;
	if (cur == this)
		return;
	ss->m_Mode = this;
	if (cur != NULL) {
		cur->onDeselect();
	}
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::forceSelect() const {
	ModeStack* ss = getContext()->getModeStack();
	const Mode* cur = ss->m_Mode;
	ss->m_Mode = this;
	if (cur != NULL) {
		cur->onDeselect();
	}
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::deselect() const {
	ModeStack* ss = getContext()->getModeStack();
	if (ss->m_Mode == this) {
		ss->m_Mode = NULL;
		onDeselect();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::selectNone() {
	ModeStack* ss = getContext()->getModeStack();
	if (ss->m_Mode != NULL) {
		ss->m_Mode->onDeselect();
		ss->m_Mode = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::setDepthRange(const Range<float>& r) {
	m_DepthRange = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::setDepthRange(float min, float max) {
	m_DepthRange.setStart(min);
	m_DepthRange.setEnd(max);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Range<float> Mode::getDepthRange() const {
	return m_DepthRange;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setDepthTest(TestFunction  t) {
	m_DepthTest = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mode::TestFunction	Mode::getDepthTest() const {
	return m_DepthTest;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setDepthWrite(bool b) {
	m_DepthWrite = b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	Mode::getDepthWrite() const {
	return m_DepthWrite;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::setStencilMode(Mode::StencilMode fail,Mode::StencilMode zfail,Mode::StencilMode pass) {
	m_StencilFail = fail;
	m_StencilZFail = zfail;
	m_StencilPass = pass;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::setStencilTest(Mode::TestFunction t, unsigned int ref, unsigned int mask) {
	m_StencilTest = t;
	m_StencilRef = ref;
	m_StencilMask = mask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::setStencilTest(Mode::TestFunction t) {
	m_StencilTest = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mode::TestFunction Mode::getStencilTest() const {
	return m_StencilTest;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setStencilReference(unsigned int ref) {
	m_StencilRef = ref;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	Mode::getStencilReference() const {
	return m_StencilRef;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setStencilMask(unsigned int mask) {
	m_StencilMask = mask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	Mode::getStencilMask() const {
	return m_StencilMask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setStencilWrite(unsigned int mask) {
	m_StencilWrite = mask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	Mode::getStencilWrite() const {
	return m_StencilWrite;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mode::setBlendMode(Mode::BlendMode src,Mode::BlendMode dst) {
	m_BlendSource = src;
	m_BlendDestination = dst;
}
					

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setBlendEquation(Mode::Equation e) {
	m_BlendEquation = e;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mode::Equation Mode::getBlendEquation() const {
	return m_BlendEquation;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setBlendColour(const Colour<float>& c) {
	m_BlendColour = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> Mode::getBlendColour() const {
	return m_BlendColour;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setColourWrite(bool r, bool g, bool b, bool a) {
	m_RedWrite = r;
	m_GreenWrite = g;
	m_BlueWrite = b;
	m_AlphaWrite = a;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setColourWrite(bool a) {
	m_RedWrite =
	m_GreenWrite = 
	m_BlueWrite = 
	m_AlphaWrite = a;
}
		

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setAlphaTest(Mode::TestFunction t,float ref) {
	m_AlphaTest = t;
	m_AlphaRef = ref;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mode::setAlphaTest(Mode::TestFunction t) {
	m_AlphaTest = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mode::TestFunction Mode::getAlphaTest() const {
	return m_AlphaTest;
}


}; };

#endif
