/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODESTACK_H
#define __GRAPHICS3D_MODESTACK_H

#include "graphics3d.mode.h"

namespace magnum {namespace graphics3d {
class Context;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ModeStack : public Object {
DeclareClass(ModeStack, Object);
friend Mode;
friend Context;
private:
	const Mode*				m_Mode;
	Mode::TestFunction		m_StencilTest;
	unsigned int			m_StencilRef;
	unsigned int			m_StencilMask;

private:
	inline ModeStack();

public:
	inline const Mode*		getMode() const;

	void					push();
	void					pop();

	// Depth Buffer
	inline void				setDepthRange(float , float);
	inline void				setDepthRange(const Range<float>& );
	inline void				setDepthTest(Mode::TestFunction  );
	inline void				setDepthWrite(bool );

	// Stencil Buffer							
	inline void				setStencilMode(Mode::StencilMode ,Mode::StencilMode ,Mode::StencilMode );
	inline void				setStencilTest(Mode::TestFunction , unsigned int ref, unsigned int mask);
	inline void				setStencilReference(unsigned int ref);
	inline void				setStencilWrite(unsigned int = 0xffffffff);

	// Blending
	inline void				setBlendMode(Mode::BlendMode ,Mode::BlendMode );
	inline void				setBlendEquation(Mode::Equation );
	inline void				setBlendColour(const Colour<float>& );
							
	// Colour/Alpha writing
	inline void				setColourWrite(bool r, bool g, bool b, bool a);
	inline void				setColourWrite(bool r);
			
	// Alphatest				
	inline void				setAlphaTest(Mode::TestFunction ,float ref);

	// Hardware Support Queries
	inline bool				supportsBlendEquation() const;
	inline bool				supportsBlendColour() const;
};

}; };

#endif
