/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_OPTIMIZER_H
#define __GRAPHICS3D_OPTIMIZER_H

#include "object.pointer.h"
#include "graphics3d.mesh.h"
#include "graphics3d.texture.h"
#include "graphics3d.indexbuffer.h"
#include "graphics3d.vertexbuffer.h"
#include "graphics3d.compiler.h"
#include "container.array.h"
#include "container.tuple.h"

namespace magnum {namespace graphics3d {
using magnum::object::Pointer;
using magnum::container::Array;
using magnum::container::Tuple;

class Shader;
class Material;
class Mode;
class RenderStream;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Optimizer : public Object {
DeclareClass(Optimizer, Object);
public:
	enum Strategy {
		SortShader = 0x0001,
		SortMaterial = 0x0002,
		SortMode = 0x0004,
		SortBuffer = 0x0008,
		MergeBuffer = 0x0010
	};

private:
    struct Job {
        Pointer<Mesh>			m_Mesh;
        Pointer<Material>		m_Material;
        Pointer<Shader>			m_Shader;
		Pointer<Texture>		m_Textures[Texture::MaxTextures];
        Pointer<Mode>			m_Mode;
        Pointer<VertexBuffer>	m_VertexBuffers[Attributes::MaxAttributes];
        Pointer<IndexBuffer>	m_IndexBuffer;
        unsigned int			m_Start;
        unsigned int			m_Count;
        Mesh::Primitive			m_Primitive;
    };

    Array<Job>			m_Jobs;

	Compiler			m_Compiler;
						
	Strategy			m_Strategy;

	Texture*			m_PreselectedTextures[Texture::MaxTextures];						
    Material*			m_PreselectedMaterial;
    Shader*				m_PreselectedShader;
    Mode*				m_PreselectedMode;
    VertexBuffer*		m_PreselectedVertexBuffers[Attributes::MaxAttributes];
    IndexBuffer*		m_PreselectedIndexBuffer;

	Texture*			m_CurrentTextures[Texture::MaxTextures];						
    Material*			m_CurrentMaterial;
    Shader*				m_CurrentShader;
    Mode*				m_CurrentMode;
    VertexBuffer*		m_CurrentVertexBuffers[Attributes::MaxAttributes];
    IndexBuffer*		m_CurrentIndexBuffer;
    
public:
    Optimizer();
    ~Optimizer();

	inline void			setStrategy(Strategy );
	inline Strategy		getStrategy() const;
						
    inline void			preselectVertexBuffer(VertexBuffer* , unsigned int attribs = 0xffffffff);
    inline void			preselectIndexBuffer(IndexBuffer* );
    inline void			preselectMaterial(Material* );
    inline void			preselectTexture(Texture* ,unsigned int );
    inline void			preselectShader(Shader* );
    inline void			preselectAppearance(Appearance* );
    inline void			preselectMode(Mode* );

    inline void			selectVertexBuffer(VertexBuffer* , unsigned int attribs = 0xffffffff);
    inline void			selectIndexBuffer(IndexBuffer* );
    inline void			selectMaterial(Material* );
    inline void			selectTexture(Texture* ,unsigned int );
    inline void			selectShader(Shader* );
    inline void			selectAppearance(Appearance* );
    inline void			selectMode(Mode* );

    inline void			deselectMaterial();
    inline void			deselectTexture(unsigned int );
    inline void			deselectShader();
    inline void			deselectAppearance();
    inline void			deselectMode();

    void				renderMesh(Mesh* );
    void				renderVertices(Mesh::Primitive, unsigned int start, unsigned int size);
    void				renderIndices(Mesh::Primitive, unsigned int start, unsigned int size);
						
    void				clear();
	void				finish();

	RenderStream*		optimize();

private:
	static int			cmpJobs(const void* ,const void* );

	void				mergeJobs();
	void				mergeBuffers();
	void				splitBuffers();

	RenderStream*		createStream();

	Tuple<VertexBuffer*,VertexBuffer::MaxBuffers>	collectVertexBuffers(const Pointer<VertexBuffer>* );
	unsigned int		collectAttributes(const Tuple<VertexBuffer*,Attributes::MaxAttributes>& );
	unsigned int		collectAttributes(const Tuple<VertexBuffer*,Attributes::MaxAttributes>& ,const VertexBuffer*);
};

}; };

#endif
