/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_OPTIMIZER_INL
#define __GRAPHICS3D_OPTIMIZER_INL

namespace magnum {namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::setStrategy(Optimizer::Strategy s) {
	m_Strategy = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Optimizer::Strategy Optimizer::getStrategy() const {
	return m_Strategy;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::selectVertexBuffer(VertexBuffer* vb, unsigned int attribs) {
	ASSERT(vb != NULL);
	attribs &= vb->getAttributes();
	for (unsigned int i = 0; i < Attributes::MaxAttributes; i++) {
		if (attribs & (1 << i))
			m_CurrentVertexBuffers[i] = vb;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::selectIndexBuffer(IndexBuffer* ib) {
	ASSERT(ib != NULL)
	m_CurrentIndexBuffer = ib;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::selectMaterial(Material* mat) {
	ASSERT(mat != NULL)
	m_CurrentMaterial = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::deselectMaterial() {
	m_CurrentMaterial = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::selectTexture(Texture* tex, unsigned int i) {
	ASSERT(tex != NULL);
	ASSERT(i < Texture::MaxTextures);
	m_CurrentTextures[i] = tex;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::deselectTexture(unsigned int i) {
	ASSERT(i < Texture::MaxTextures);
	m_CurrentTextures[i] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::selectShader(Shader* shader) {
	ASSERT(shader != NULL)
	m_CurrentShader = shader;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::deselectShader() {
	m_CurrentShader = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::selectMode(Mode* mode) {
	ASSERT(mode != NULL)
	m_CurrentMode = mode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::deselectMode() {
	m_CurrentMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::selectAppearance(Appearance* app) {
	ASSERT(app != NULL)
	if (app->getShader() != NULL)
		selectShader(app->getShader());
	if (app->getMaterial() != NULL)
		selectMaterial(app->getMaterial());
	for (unsigned int i = 0; i < Texture::MaxTextures; i++) {
		if (app->getTexture(i) != NULL)
			selectTexture(app->getTexture(i), i);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::deselectAppearance() {
	deselectShader();
	deselectMaterial();
	for (unsigned int i = 0; i < Texture::MaxTextures; i++) {
		deselectTexture(i);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::preselectVertexBuffer(VertexBuffer* vb, unsigned int attribs) {
	ASSERT(vb != NULL);
	attribs &= vb->getAttributes();
	for (unsigned int i = 0; i < Attributes::MaxAttributes; i++) {
		if (attribs & (1 << i))
			m_PreselectedVertexBuffers[i] = vb;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::preselectIndexBuffer(IndexBuffer* ib) {
	ASSERT(ib != NULL)
	m_PreselectedIndexBuffer = ib;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::preselectMaterial(Material* mat) {
	ASSERT(mat != NULL)
	m_PreselectedMaterial = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimizer::preselectTexture(Texture* tex, unsigned int i) {
	ASSERT(tex != NULL);
	ASSERT(i < Texture::MaxTextures);
	m_PreselectedTextures[i] = tex;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::preselectShader(Shader* shader) {
	ASSERT(shader != NULL)
	m_PreselectedShader = shader;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimizer::preselectAppearance(Appearance* app) {
	ASSERT(app != NULL)
	if (app->getShader() != NULL)
		preselectShader(app->getShader());
	if (app->getMaterial() != NULL)
		preselectMaterial(app->getMaterial());
	for (unsigned int i = 0; i < Texture::MaxTextures; i++) {
		if (app->getTexture(i) != NULL)
			preselectTexture(app->getTexture(i), i);
	}
}



}; };

#endif
