#ifndef __GRAPHICS3D_PARAMETER_H
#define __GRAPHICS3D_PARAMETER_H

#include "object.h"
#include "math.vector.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "math.vector4.h"
#include "math.matrix.h"
#include "graphics.colour.h"

namespace magnum {namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Vector;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::math::Vector4;
using magnum::math::Matrix;
using magnum::math::Matrix2;
using magnum::math::Matrix3;
using magnum::math::Matrix4;
using magnum::graphics::Colour;

class ParameterFormat;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ParameterBuffer : public Object {
DeclareClass(ParameterBuffer, Object);
public:
	typedef int				Location;

private:
	const ParameterFormat*	m_Format;
	byte					m_Parameters[1];

private:	
	inline ParameterBuffer();

public:
	static ParameterBuffer* create(const ParameterFormat& );
	
	inline unsigned int		getSize() const;
	inline const ParameterFormat* getFormat() const;
	inline void*			getPointer(Location );
	inline const void*		getPointer(Location ) const;
	inline void*			getBuffer();
	inline const void*		getBuffer() const;

	inline void				setVector(Location l, const Vector3<float>& );
	inline void				setVector(Location l, const Vector4<float>& );
	inline void				setVector(Location l, const Vector3<double>& );
	inline void				setVector(Location l, const Vector4<double>& );
	inline void				setMatrix(Location l, const Matrix3<float>& );
	inline void				setMatrix(Location l, const Matrix4<float>& );
	inline void				setMatrix(Location l, const Matrix3<double>& );
	inline void				setMatrix(Location l, const Matrix4<double>& );
	inline void				setColour(Location l, const Colour<float>& );
	inline void				setColour(Location l, const Colour<byte>& );
	inline void				setInteger(Location l, int );
	inline int				getInteger(Location l) const;
	inline void				setFloat(Location l, float );
	inline float			getFloat(Location l) const;
	inline void				setDouble(Location l, double );
	inline double			getDouble(Location l) const;
};


}; };

#endif
