/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_SHADER_H
#define __GRAPHICS3D_SHADER_H

#include "object.pointer.h"
#include "system.assembler.h"
#include "graphics3d.resource.h"
#include "graphics3d.constant.h"


namespace magnum { namespace graphics3d {
using magnum::object::Pointer;
using magnum::system::Assembler;

class PixelProgram;
class VertexProgram;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Shader : public Resource {
DeclareClass(Shader, Resource);
private:
	ConstantList			m_Constants;
	Pointer<PixelProgram>	m_PixelProgram;
	Pointer<VertexProgram>	m_VertexProgram;
	unsigned int			m_BaseAttributes;
	unsigned int			m_Attributes;

protected:
	inline					Shader();

public:
	inline void				select() const;
	inline void				forceSelect() const;
	inline void				deselect() const;
	static inline void		selectNone();

	void					compileSelect(Assembler& ) const;
	void					compileDeselect(Assembler& ) const;

	inline const ConstantList&	getConstants() const;
	inline unsigned int		getAttributes() const;

    inline void				attachPixelProgram(PixelProgram* );
    inline void				detachPixelProgram();
    inline PixelProgram*	getPixelProgram() const;

    inline void				attachVertexProgram(VertexProgram* );
    inline void				detachVertexProgram();
    inline VertexProgram*	getVertexProgram() const;

protected:
	virtual void			onSelect() const;
	virtual void			onDeselect() const;
	
	virtual void			onCompileSelect(Assembler& ) const;
	virtual void			onCompileDeselect(Assembler& ) const;

	inline void				publishAttributes(unsigned int );
	inline void				publishConstants(const ConstantList& );

private:
	void					selectAttributes() const;
	void					updateAttributes();
};


}; };

#endif
