/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_TEXTURE_H
#define __GRAPHICS3D_TEXTURE_H

#include "system.assembler.h"
#include "container.list.h"
#include "graphics.palette.h"
#include "graphics.image.h"
#include "graphics3d.resource.h"
#include "graphics3d.opengl.h"


namespace magnum { namespace graphics3d {
using magnum::graphics::Palette;
using magnum::types::byte;
using magnum::types::String;
using magnum::container::List;
using magnum::system::Assembler;

class Texture1D;
class Texture2D;
class Texture3D;
class Context;
class TextureStack;
class FrameBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Texture : public Resource {
DeclareClass(Texture, Resource);
friend Texture1D;
friend Texture2D;
friend Texture3D;
friend FrameBuffer;
public:
	enum {
		MaxTextures = 8,
	};

	enum LockFlag {
		ReadOnly = 1,
		WriteOnly = 2,
		ReadWrite = 3,
		Discard = 4
	};

	enum Filter {
		Nearest = GL_NEAREST,
		Linear = GL_LINEAR,
		NearestMipmapNearest = GL_NEAREST_MIPMAP_NEAREST,
		LinearMipmapNearest = GL_LINEAR_MIPMAP_NEAREST,
		NearestMipmapLinear = GL_NEAREST_MIPMAP_LINEAR,
		LinearMipmapLinear = GL_LINEAR_MIPMAP_LINEAR
	};

	enum Wrapping {
		Clamp = GL_CLAMP,
		ClampEdge = GL_CLAMP_TO_EDGE,
		Repeat = GL_REPEAT,
		Mirror = GL_MIRRORED_REPEAT
	};

	enum MipMaps {
		CopyAll	= -1,
		GenerateAll = -2
	};


	typedef magnum::graphics::Image::Type Type;

private:
	Palette<byte>*	m_Palette;	
	Type			m_Type;
	Filter			m_MagFilter;
	Filter			m_MinFilter;
	Wrapping		m_WrapS;
	Wrapping		m_WrapT;
	Wrapping		m_WrapU;
	unsigned int	m_OglTexture;
	unsigned int	m_OglTarget;


protected:
	Texture(unsigned int tex, unsigned int target);

public:
	inline ~Texture();

	inline void			select(unsigned int stage = 0) const;
	inline void			deselect(unsigned int stage = 0) const;
	inline void			forceSelect(unsigned int stage = 0) const;
	static inline void	selectNone(unsigned int stage = 0);

	void				compileSelect(Assembler& ,unsigned int stage = 0) const;

	inline bool			isIndexed() const;
	inline Type			getType() const;
	inline unsigned int	getTarget() const;
						
	inline void			setFilter(Filter min, Filter max);
	inline Filter		getMagFilter() const;
	inline Filter		getMinFilter() const;
						
	inline void			setWrapping(Wrapping , Wrapping = Clamp, Wrapping = Clamp);
	inline Wrapping		getWrappingS() const;
	inline Wrapping		getWrappingT() const;
	inline Wrapping		getWrappingU() const;

	bool				getPalette(Palette<byte>& pal) const;
	bool				setPalette(const Palette<byte>& pal);
	
protected:
	static int			mapOutputFormat(Type );
	static int			mapInputFormat(Type );
};

typedef List<Texture*>	TextureList;

}; };

#endif
