/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_TEXTURE_INL
#define __GRAPHICS3D_TEXTURE_INL

namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::~Texture() {
	// Remove texture from all contexts!
	TextureStack* ts = getContext()->getTextureStack();
	for (int i = 0; i < MaxTextures; i++) {
		if (ts->m_Texture[i] == this)
			ts->m_Texture[i] = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Texture::select(unsigned int idx) const {
	ASSERT(idx < MaxTextures);
	TextureStack* ts = getContext()->getTextureStack();
	if (ts->m_Texture[idx] == this)
		return;
	ts->m_Texture[idx] = this;
	glActiveTextureARB(GL_TEXTURE0_ARB + idx);
	if (m_OglTarget != ts->m_OglTarget[idx]) {
		if (glIsEnabled(ts->m_OglTarget[idx])) {
			glDisable(ts->m_OglTarget[idx]);
			glEnable(m_OglTarget);
		}
		ts->m_OglTarget[idx] = m_OglTarget;
	}
	glBindTexture(m_OglTarget, m_OglTexture);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Texture::forceSelect(unsigned int idx) const {
	ASSERT(idx < MaxTextures);
	TextureStack* ts = getContext()->getTextureStack();
	ts->m_Texture[idx] = this;
	glActiveTextureARB(GL_TEXTURE0_ARB + idx);
	if (m_OglTarget != ts->m_OglTarget[idx]) {
		if (glIsEnabled(ts->m_OglTarget[idx])) {
			glDisable(ts->m_OglTarget[idx]);
			glEnable(m_OglTarget);
		}
		ts->m_OglTarget[idx] = m_OglTarget;
	}
	glBindTexture(m_OglTarget, m_OglTexture);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Texture::deselect(unsigned int idx) const {
	ASSERT(idx < MaxTextures);
	TextureStack* ts = getContext()->getTextureStack();
	if (ts->m_Texture[idx] != this)
		return;
	ts->m_Texture[idx] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Texture::selectNone(unsigned int idx) {
	ASSERT(idx < MaxTextures);
	TextureStack* ts = getContext()->getTextureStack();
	ts->m_Texture[idx] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Type Texture::getType() const {
	return m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Texture::getTarget() const {
	return m_OglTarget;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Texture::setFilter(Filter min, Filter mag) {
	m_MinFilter = min;
	m_MagFilter = mag;
	int oldtex;
	glGetIntegerv(m_OglTarget, &oldtex);
	glBindTexture(m_OglTarget, m_OglTexture);
	glTexParameteri(m_OglTarget, GL_TEXTURE_MIN_FILTER, min);
	glTexParameteri(m_OglTarget, GL_TEXTURE_MAG_FILTER, mag);
	glBindTexture(m_OglTarget, oldtex);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Filter Texture::getMagFilter() const {
	return m_MagFilter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Filter Texture::getMinFilter() const {
	return m_MinFilter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Texture::setWrapping(Wrapping s, Wrapping t, Wrapping u) {
	m_WrapS = s;
	m_WrapT = t;
	m_WrapU = u;
	int oldtex;
	glGetIntegerv(m_OglTarget, &oldtex);
	glBindTexture(m_OglTarget, m_OglTexture);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_S, s);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_T, t);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_R, u);
	glBindTexture(m_OglTarget, oldtex);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Wrapping Texture::getWrappingS() const {
	return m_WrapS;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Wrapping Texture::getWrappingT() const {
	return m_WrapT;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Wrapping Texture::getWrappingU() const {
	return m_WrapU;
}


}; };

#endif
