/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_TEXTURE2D_H
#define __GRAPHICS3D_TEXTURE2D_H

#include "graphics3d.context.h"
#include "graphics3d.texture.h"
#include "graphics.dimension.h"
#include "graphics.rectangle.h"
#include "graphics.palette.h"
#include "graphics.image.h"


namespace magnum { namespace graphics3d {
using magnum::graphics::Image;
using magnum::graphics::Dimension;
using magnum::graphics::Rectangle;
using magnum::graphics::Palette;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Texture2D : public Texture {
DeclareClass(Texture2D, Texture);
private:
	int					m_Levels;
	Dimension<int>		m_Size;
	bool				m_Locked;

protected:
	Texture2D(unsigned int tex);

public:
	inline static Dimension<int> getMinSize();
	inline static Dimension<int> getMaxSize();

	static Texture2D*	create(Type ,Dimension<int>& d, int levels = GenerateAll);
	static Texture2D*	create(Type ,unsigned int w, unsigned int h, int levels = GenerateAll);
	static Texture2D*	create(const Image* , int levels = GenerateAll);

	~Texture2D();

	bool				update(const Image* , int level = 0);

	bool				copyTo(Image* img, const Point<int>& ,const Rectangle<int>& ) const;
	bool				copyFrom(const Image* img, const Point<int>& ,const Rectangle<int>& );
	bool				copyFrom(const Image* img, const Point<int>& );

	void*				lock(int level, const Rectangle<int>& , LockFlag );
	void				unlock();
	inline bool			isLocked() const;

	inline Dimension<int> getSize() const;
	inline unsigned int	getWidth() const;
	inline unsigned int	getHeight() const;
	inline unsigned int	getLevels() const;
};

}; };

#endif
