#ifndef __GRAPHICS3D_TEXTUREMATERIAL_INL
#define __GRAPHICS3D_TEXTUREMATERIAL_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline TextureMaterial::TextureMaterial() {
	memset(m_Texture, 0, sizeof(m_Texture));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void TextureMaterial::attachTexture(Texture* t, unsigned int stage) {
	ASSERT((stage >= 0) && (stage < Texture::MaxTextures));
	ASSERT(t != NULL);
	t->refer();
	if (m_Texture[stage] != NULL)
		m_Texture[stage]->release();
	m_Texture[stage] = t;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void TextureMaterial::detachTexture(unsigned int stage) {
	ASSERT((stage >= 0) && (stage < Texture::MaxTextures));
	if (m_Texture[stage] != NULL)
		m_Texture[stage]->release();
	m_Texture[stage] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture*	TextureMaterial::getTexture(unsigned int stage) const {
	ASSERT((stage >= 0) && (stage < Texture::MaxTextures));
	return m_Texture[stage];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void TextureMaterial::selectTextures() const {
	for (int i = 0; i < Texture::MaxTextures; i++) {
		if (m_Texture[i] != NULL) {
			m_Texture[i]->select(i);
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void TextureMaterial::selectTexture(unsigned int i) const {
	ASSERT(m_Texture[i] != NULL);
	m_Texture[i]->select(i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void TextureMaterial::deselectTexture(unsigned int i) const {
	ASSERT(m_Texture[i] != NULL);
	m_Texture[i]->deselect(i);
}

}; };

#endif

