/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GUI_CONTEXT_H
#define __GUI_CONTEXT_H

#include "graphics.transform.h"
#include "graphics.polygon.h"
#include "graphics.rectangle.h"
#include "graphics.image.h"
#include "graphics.palette.h"

namespace magnum { namespace gui {
using magnum::object::Object;
using magnum::types::String;
using magnum::graphics::Colour;
using magnum::graphics::Rectangle;
using magnum::graphics::Point;
using magnum::graphics::Transform;
using magnum::graphics::Palette;
using magnum::graphics::Polygon;
using magnum::graphics::Image;

class Window;
class View;
class Bitmap;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Context : public Object {
DeclareClass(Context, Object);
friend class Window;
friend class View;
private:
	Context();
	~Context();

public:
	inline void			drawRectangle(const Rectangle<float>& );
	inline void			fillRectangle(const Rectangle<float>& );
	inline void			drawPoint(const Point<float>& );
	inline void			drawLine(const Point<float>& , const Point<float>& );
	inline void			drawPolygon(const Polygon<float>& );
	inline void			drawBitmap(const Point<float>& , const Bitmap* );
	inline void			drawBitmap(const Point<float>& , const Bitmap* ,const Rectangle<float>& );
	inline void			drawBitmap(const Rectangle<float>& , const Bitmap* );
	inline void			drawBitmap(const Rectangle<float>& , const Bitmap* ,const Point<float>& );
	inline void			stretchBitmap(const Rectangle<float>& , const Bitmap* ,const Rectangle<float>& );


	inline const Colour<byte>& getFillColour() const;
	void				setFillColour(const Colour<byte>& );

	inline const Colour<byte>& getLineColour() const;
	void				setLineColour(const Colour<byte>& );

	inline int			getLineWidth() const;
	void				setLineWidth(int );

	Transform<float>	getTransform() const;
	void				setTransform(const Transform<float>& );

	void				setClipping(const Rectangle<float>& );
	Rectangle<float>	getClipping() const;


	// Some system specific calls
#if defined(__FORWINDOWS__)
	inline HWND			getWindowHandle() const;
	inline HDC			getDeviceHandle() const;
#elif defined(__FORX11__)
	inline Display* 	getDisplayHandle() const;
	inline Window		getWindowHandle() const;
	inline C			getDeviceHandle() const;
#endif


private:
#if defined(__FORWINDOWS__)
	HWND				m_WindowHandle;
	HDC					m_DeviceHandle;
	Colour<byte>		m_FillColour;
	Colour<byte>		m_LineColour;
	int					m_LineWidth;
	
#elif defined(__FORX11__)
	Display*			m_DisplayHandle;
	Window				m_WindowHandle;
    C					m_DeviceHandle;
#endif

};

}; };

#endif
