/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GUI_VIEW_H
#define __GUI_VIEW_H

#include "application.handler.h"
#include "application.keyboard.h"
#include "container.chain.h"
#include "graphics.rectangle.h"
#include "graphics.colour.h"
#include "graphics.transform.h"
#include "gui.context.h"

# if defined(__FORX11__)
#	include <X11/Xlib.h>
# endif

namespace magnum { namespace gui {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::byte;
using magnum::container::Chain;
using magnum::container::Chainable;
using magnum::application::Handler;
using magnum::application::Message;
using magnum::application::Keyboard;
using magnum::application::Character;
using magnum::application::Mouse;
using magnum::graphics::Rectangle;
using magnum::graphics::Dimension;
using magnum::graphics::Colour;
using magnum::graphics::Point;
using magnum::graphics::Transform;

class Window;
class Container;
class OpenGL;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class View : public Handler, private Chainable<View> {
DeclareClass(View, Handler);
friend class Window;
friend class Container;
friend class OpenGL;
private:
	Window*					m_Window;
	Container*				m_Parent;
	Context					m_Context;
	Rectangle<float>		m_Bounds;
	Rectangle<float>		m_Frame;
	int						m_Alignement;
	String					m_Name;
	Rectangle<float>		m_UpdateBounds;
	Transform<float>		m_Transform;
	Transform<float>		m_AbsTransform;
	
#if defined(__FORWINDOWS__)
	HWND					m_WindowHandle;
	HDC						m_DeviceHandle;
	bool					m_MouseTracking;
#endif
	

public:
	enum Alignement {
		FollowLeft = 0x01,
		FollowRight = 0x02,
		FollowLeftRight = 0x03,
		FollowHCenter = 0x04,
		FollowTop = 0x10,
		FollowBottom = 0x20,
		FollowTopBottom = 0x30,
		FollowVCenter = 0x40,
		FollowAll = 0x33
	};

	View(const Rectangle<float>& frame);
	~View();

	inline Window*			getWindow() const;
	inline Container*		getParent() const;

	inline void				setName(const char* );
	inline String			getName(void) const;

	inline Rectangle<float> getFrame() const;
	inline void				setFrame(const Rectangle<float>& );

	inline Dimension<float> getSize() const;
	inline void				setSize(const Dimension<float>& );

	inline Point<float>		getLocation() const;
	inline void				setLocation(const Point<float>& );

	inline Rectangle<float> getBounds() const;
	void					setBounds(const Rectangle<float>& );

	inline Point<float>		getOrigin() const;
	void					setOrigin(const Point<float>& );

	inline Point<float>		getScale() const;
	void					setScale(const Point<float>& );
	
	inline Context*			getContext();

	inline bool				getFocus();
	inline bool				hasFocus() const;
	inline bool				releaseFocus();

	bool					invalidate(const Rectangle<float>& rect);
	bool					invalidate(void); 

	inline  Point<float>	convertToParent(const Point<float>& localPoint) const;
	inline  Dimension<float> convertToParent(const Dimension<float>& localDimension) const;
	inline  Rectangle<float> convertToParent(const Rectangle<float>& localRect) const; 

	inline  Point<float>	convertFromParent(const Point<float>& localPoint) const; 
	inline  Dimension<float> convertFromParent(const Dimension<float>& localDimension) const; 
	inline  Rectangle<float> convertFromParent(const Rectangle<float>& localRect) const; 

	inline  Point<float>	convertToWindow(const Point<float>& localPoint) const;
	inline  Dimension<float> convertToWindow(const Dimension<float>& localDimension) const;
	inline  Rectangle<float> convertToWindow(const Rectangle<float>& localRect) const; 

	inline  Point<float>	convertFromWindow(const Point<float>& localPoint) const; 
	inline  Dimension<float> convertFromWindow(const Dimension<float>& localDimension) const; 
	inline  Rectangle<float> convertFromWindow(const Rectangle<float>& localRect) const; 

	inline  int				getAlignement() const;
	inline  void			setAlignement(int );

	virtual Dimension<float> getPreferredSize() const;
	virtual bool			setPreferredSize(void);
	
#if defined(__FORWINDOWS__)
	inline HWND				getWindowHandle() const;
	inline HDC				getDeviceHandle() const;
#endif
	

protected:
	virtual bool			onPaint(Context* ,const Rectangle<float>& );

	// Creation, open and close methods
	virtual bool			onAttachWindow();
	virtual bool			onDetachWindow();

	// Activation / Deactivation
	virtual bool			onActivateWindow();
	virtual bool			onDeactivateWindow();

	// Resizing
	virtual bool			onResize(const Dimension<float>& );
							
	// Keyboard input
	virtual bool			onGetFocus();
	virtual bool			onReleaseFocus();
	virtual bool			onKeyDown(Keyboard::Key );
	virtual bool			onKeyUp(Keyboard::Key );
	virtual bool			onChar(Character::Code ,Character::Modifier );

	// Mouse input
	virtual bool			onMouseMove(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseButton(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseEnter(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseLeave();
	virtual bool			onMouseWheel(float ,Mouse::Button );
							
private:					
	void					updateFrame();
	virtual void			updateTransform();

#if defined(__FORWINDOWS__)
	static LRESULT CALLBACK windowProc(HWND hwnd, UINT Message,WPARAM wParam, LPARAM lParam);
	static char				windowClass[];
	static void				registerClass();
#elif defined(__FORX11__)		
	virtual bool			dispatchEvent(const XEvent& event);
#endif
};

typedef Chain<View>	ViewList;

}; };
#endif
