/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GUI3D_VIEW_INL
#define __GUI3D_VIEW_INL

namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Root* View::getRoot() const {
	return m_Root;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Container* View::getParent() const {
	return m_Parent;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Context* View::getContext() const {
	return m_Context;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setName(const char* n) {
	m_Name = n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String View::getName(void) const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::isEnabled() const {
	return m_Enabled;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::isHovered() const {
	if (m_Root == NULL)
		return false;
	return (m_Root->m_MouseView == this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::getFocus() {
	if (m_Root == NULL)
		return false;
	return (m_Root->setFocus(this));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::hasFocus() const {
	if (m_Root == NULL)
		return false;
	return (m_Root->m_Focus == this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::releaseFocus() {
	if (m_Root == NULL)
		return false;
	return m_Root->releaseFocus(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Rectangle<float>& View::getFrame() const {
	return m_Frame;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setFrame(const Rectangle<float>& frame) {
	m_Frame = frame;
	updateFrame();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::getSize() const {
	return m_Frame.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setSize(const Dimension<float>& size) {
	m_Frame.setSize(size);
	updateFrame();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::getLocation() const {
	return m_Frame.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setLocation(const Point<float>& size) {
	m_Frame.setOrigin(size);
	updateFrame();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Rectangle<float>& View::getBounds() const {
	return m_Bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::getOrigin() const {
	return m_Transform.getTranslation();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::getScale() const {
	return m_Transform.getScale();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int View::getAlignement() const {
	return m_Alignement;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setAlignement(int r) {
	m_Alignement = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::getPreferredSize() const {
	return onGetPreferredSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::setPreferredSize(void) {
	return onSetPreferredSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertToParent(const Point<float>& p) const {
	return m_Transform * p + m_Frame.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertToParent(const Dimension<float>& d) const {
	return m_Transform * d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline  Rectangle<float> View::convertToParent(const Rectangle<float>& r) const {
	return (m_Transform * r) + m_Frame.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertFromParent(const Point<float>& localPoint) const {
	return m_Transform % (localPoint - m_Frame.getOrigin());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertFromParent(const Dimension<float>& localDimension) const {
	return m_Transform % localDimension;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<float> View::convertFromParent(const Rectangle<float>& localRect) const {
	return m_Transform % (localRect - m_Frame.getOrigin());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertToRoot(const Point<float>& p) const {
	return m_AbsTransform * p;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertToRoot(const Dimension<float>& localDimension) const {
	return m_AbsTransform * localDimension;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline  Rectangle<float> View::convertToRoot(const Rectangle<float>& localRect) const {
	return m_AbsTransform * localRect;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertFromRoot(const Point<float>& localPoint) const {
	return m_AbsTransform % localPoint;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertFromRoot(const Dimension<float>& localDimension) const {
	return m_AbsTransform % localDimension;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<float> View::convertFromRoot(const Rectangle<float>& localRect) const {
	return m_AbsTransform % localRect;
}

}; };

#endif
