/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_DIRECTORY_H
#define __IO_DIRECTORY_H

#include "object.h"
#include "container.array.h"
#include "system.timestamp.h"

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Array;
using magnum::system::Timestamp;

class Directory;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class DirectoryNode : public Object {
DeclareClass(DirectoryNode, Object);
friend class Directory;
private:
#if defined(__FORWINDOWS__)
	WIN32_FIND_DATA			m_Entry;
#endif

public:
	inline DirectoryNode();
	
	inline String			getName() const;
	inline unsigned long	getSize() const;
	inline Timestamp		getCreationTime() const;
	inline Timestamp		getModificationTime() const;
	inline Timestamp		getAccessTime() const;
	
	inline bool				isDirectory() const;
	inline bool				isWriteable() const;
	inline bool				isHidden() const;

	inline bool 			operator==(const DirectoryNode& iter) const;
	inline bool 			operator!=(const DirectoryNode& iter) const;
	inline bool 			operator>=(const DirectoryNode& iter) const;
	inline bool 			operator<=(const DirectoryNode& iter) const;
	inline bool 			operator<(const DirectoryNode& iter) const;
	inline bool 			operator>(const DirectoryNode& iter) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Directory : public Object {
DeclareClass(Directory, Object);
public:
	typedef DirectoryNode			Node;
	typedef Array<Node>::Iterator	Iterator;
	
private:
	String				m_Directory;
	Array<Node>			m_Nodes;
	
public:
	inline Directory();
	inline Directory(const char* );
	
	inline String		getName() const;
	
	bool				refresh();
	bool				open(const char* );
	void				clear();
	
	inline unsigned int	getSize() const;
	
	inline Iterator		begin() const;
	inline Iterator		at(unsigned int ) const;
	inline Iterator		end() const;
	
	static bool			create(const char* );
	static bool			remove(const char* );
	static String		getCurrent();
};

}; };

#endif
