/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_DIRECTORY_INL
#define __IO_DIRECTORY_INL

namespace magnum { namespace io {
using magnum::types::uint64;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Directory::Directory() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Directory::Directory(const char* name) {
	open(name);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Directory::getName() const {
	return m_Directory;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Directory::Iterator Directory::begin() const {
	return m_Nodes.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Directory::Iterator Directory::end() const {
	return m_Nodes.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Directory::Iterator Directory::at(unsigned int i) const {
	return m_Nodes.at(i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Directory::getSize() const {
	return m_Nodes.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
DirectoryNode::DirectoryNode() {
#if defined(__FORWINDOWS__)
	memset(&m_Entry, 0, sizeof(m_Entry));
#endif	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String DirectoryNode::getName() const {
#if defined(__FORWINDOWS__)
	return String(m_Entry.cFileName);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned long DirectoryNode::getSize() const {
#if defined(__FORWINDOWS__)
	return m_Entry.nFileSizeLow;
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Timestamp DirectoryNode::getCreationTime() const {
#if defined(__FORWINDOWS__)
	return Timestamp(((uint64)(m_Entry.ftCreationTime.dwHighDateTime) << 32) + m_Entry.ftCreationTime.dwLowDateTime);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Timestamp DirectoryNode::getModificationTime() const {
#if defined(__FORWINDOWS__)
	return Timestamp(((uint64)(m_Entry.ftLastWriteTime.dwHighDateTime) << 32) + m_Entry.ftLastWriteTime.dwLowDateTime);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Timestamp DirectoryNode::getAccessTime() const {
#if defined(__FORWINDOWS__)
	return Timestamp(((uint64)(m_Entry.ftLastAccessTime.dwHighDateTime) << 32) + m_Entry.ftLastAccessTime.dwLowDateTime);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::isDirectory() const {
#if defined(__FORWINDOWS__)
	return ((m_Entry.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::isWriteable() const {
#if defined(__FORWINDOWS__)
	return ((m_Entry.dwFileAttributes & FILE_ATTRIBUTE_READONLY) == 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	DirectoryNode::isHidden() const {
#if defined(__FORWINDOWS__)
	return ((m_Entry.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) != 0);
#endif
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::operator==(const DirectoryNode& iter) const {
#if defined(__FORWINDOWS__)
	return (strncmp(m_Entry.cFileName, iter.m_Entry.cFileName, MAX_PATH) == 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::operator!=(const DirectoryNode& iter) const {
#if defined(__FORWINDOWS__)
	return (strncmp(m_Entry.cFileName, iter.m_Entry.cFileName, MAX_PATH) != 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::operator>=(const DirectoryNode& iter) const {
#if defined(__FORWINDOWS__)
	return (strncmp(m_Entry.cFileName, iter.m_Entry.cFileName, MAX_PATH) >= 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::operator<=(const DirectoryNode& iter) const {
#if defined(__FORWINDOWS__)
	return (strncmp(m_Entry.cFileName, iter.m_Entry.cFileName, MAX_PATH) <= 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::operator<(const DirectoryNode& iter) const {
#if defined(__FORWINDOWS__)
	return (strncmp(m_Entry.cFileName, iter.m_Entry.cFileName, MAX_PATH) > 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DirectoryNode::operator>(const DirectoryNode& iter) const {
#if defined(__FORWINDOWS__)
	return (strncmp(m_Entry.cFileName, iter.m_Entry.cFileName, MAX_PATH) < 0);
#endif
}


}; };

#endif
