/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_FILE_H
#define __IO_FILE_H

#include "object.h"
#include "system.timestamp.h"
#include "io.inputstream.h"
#include "io.outputstream.h"

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;
using magnum::system::Timestamp;


/*----------------------------------------------------------------------------
This class is the basic I/O object for reading or writing files, memory 
blocks or database objects. This is a pure virtual implementation, as
the real implementation is very system-specific
----------------------------------------------------------------------------*/
class File : public StreamFactory {
DeclareClass(File, StreamFactory);
class FileInputStream;
class FileOutputStream;
private:
	class FileInputStream : public InputStream {
	DeclareClass(FileInputStream, InputStream);
		friend class File;
		private:
			File*	m_File;

		public:
			inline unsigned long read(void *buffer, unsigned long  size);
			inline unsigned long skip(unsigned long  size);
			inline unsigned long available();
			inline bool		close();
	};


	class FileOutputStream : public OutputStream {
	DeclareClass(FileOutputStream, OutputStream);
		friend class File;
		private:
			File*	m_File;

		public:
			inline unsigned long write(const void *buffer, unsigned long size);
			inline bool		flush();
			inline bool		close();
	};



	FileInputStream		m_InputStream;
	FileOutputStream	m_OutputStream;

public:
	enum PositionMode {
		Absolute = 1,
		Current = 2,
		End = 3
	};

	enum OpenFlags {
		ReadOnly = 1,
		WriteOnly = 2,
		ReadWrite = 3
	};

	inline InputStream*		getInputStream();
	inline OutputStream*	getOutputStream();

	virtual unsigned long 	write(const void *buf, unsigned long size) = 0;
	virtual unsigned long 	read(void *buf, unsigned long size) = 0;

	virtual unsigned long 	setPosition(long ,PositionMode = Absolute) = 0;
	virtual unsigned long 	getPosition() const = 0;
	virtual bool			rewind() = 0;

	virtual bool			flush() = 0;

	virtual unsigned long 	getSize() const = 0;
	virtual unsigned long 	setSize(unsigned long newSize) = 0;

	virtual bool 			isWriteable() const = 0;
	virtual bool 			isReadable() const = 0;

	virtual Timestamp		getCreationTime() const = 0;
	virtual Timestamp		getModificationTime() const = 0;
	virtual Timestamp		getAccessTime() const = 0;

protected:
	inline File();
	inline ~File();
};

}; };

#endif
