/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MAGNUM_GRAPHICS3D_H
#define __MAGNUM_GRAPHICS3D_H

// OpenGL Stuff
#include "graphics3d.exception.h"
#include "graphics3d.context.h"
#include "graphics3d.resource.h"
#include "graphics3d.matrixstack.h"
#include "graphics3d.lightstack.h"
#include "graphics3d.texturestack.h"
#include "graphics3d.materialstack.h"
#include "graphics3d.shaderstack.h"
#include "graphics3d.modestack.h"
#include "graphics3d.framebuffer.h"
#include "graphics3d.indexbuffer.h"
#include "graphics3d.vertexbuffer.h"
#include "graphics3d.vertexformat.h"
#include "graphics3d.vertexconverter.h"
#include "graphics3d.mesh.h"
#include "graphics3d.meshgroup.h"
#include "graphics3d.vertexmesh.h"
#include "graphics3d.indexmesh.h"
#include "graphics3d.states.h"
#include "graphics3d.texture.h"
#include "graphics3d.texture1d.h"
#include "graphics3d.texture2d.h"
#include "graphics3d.texture3d.h"
#include "graphics3d.texturemanager.h"
#include "graphics3d.texturefactory.h"
#include "graphics3d.camera.h"
#include "graphics3d.material.h"
#include "graphics3d.shader.h"
#include "graphics3d.shaderfactory.h"
#include "graphics3d.shadermanager.h"
#include "graphics3d.pixelprogram.h"
#include "graphics3d.vertexprogram.h"
#include "graphics3d.rendertarget.h"
#include "graphics3d.mode.h"
#include "graphics3d.renderstream.h"
#include "graphics3d.meshmerger.h"
#include "graphics3d.meshstripper.h"
#include "graphics3d.optimizer.h"
#include "graphics3d.compiler.h"
#include "graphics3d.constant.h"
#include "graphics3d.parameter.h"
#include "graphics3d.parameterformat.h"
#include "graphics3d.appearance.h"
#include "graphics3d.bounds.h"
#include "graphics3d.geometryrenderer.h"
#include "graphics3d.meshbuilder.h"
#include "graphics3d.meshfactory.h"

#include "graphics3d.context.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.matrixstack.inl"
#include "graphics3d.lightstack.inl"
#include "graphics3d.texturestack.inl"
#include "graphics3d.materialstack.inl"
#include "graphics3d.shaderstack.inl"
#include "graphics3d.modestack.inl"
#include "graphics3d.framebuffer.inl"
#include "graphics3d.indexbuffer.inl"
#include "graphics3d.vertexbuffer.inl"
#include "graphics3d.vertexformat.inl"
#include "graphics3d.vertexconverter.inl"
#include "graphics3d.mesh.inl"
//#include "graphics3d.meshgroup.inl"
#include "graphics3d.vertexmesh.inl"
#include "graphics3d.indexmesh.inl"
#include "graphics3d.states.inl"
#include "graphics3d.texture.inl"
#include "graphics3d.texture1d.inl"
#include "graphics3d.texture2d.inl"
#include "graphics3d.texture3d.inl"
#include "graphics3d.texturemanager.inl"
#include "graphics3d.texturefactory.inl"
#include "graphics3d.camera.inl"
#include "graphics3d.mode.inl"
#include "graphics3d.shaderfactory.inl"
#include "graphics3d.shadermanager.inl"
#include "graphics3d.pixelprogram.inl"
#include "graphics3d.vertexprogram.inl"
#include "graphics3d.rendertarget.inl"
#include "graphics3d.renderstream.inl"
#include "graphics3d.meshmerger.inl"
#include "graphics3d.meshstripper.inl"
#include "graphics3d.optimizer.inl"
#include "graphics3d.compiler.inl"
#include "graphics3d.constant.inl"
#include "graphics3d.parameter.inl"
#include "graphics3d.parameterformat.inl"
#include "graphics3d.appearance.inl"
#include "graphics3d.bounds.inl"
#include "graphics3d.geometryrenderer.inl"
#include "graphics3d.meshbuilder.inl"
#include "graphics3d.meshfactory.inl"

// Materials
#include "graphics3d.material.h"
#include "graphics3d.colourmaterial.h"

#include "graphics3d.material.inl"
#include "graphics3d.colourmaterial.inl"

// Models
#include "graphics3d.model.h"
#include "graphics3d.staticmodel.h"
#include "graphics3d.framemodel.h"
#include "graphics3d.dynamicmodel.h"
#include "graphics3d.springmodel.h"

#include "graphics3d.model.inl"
#include "graphics3d.staticmodel.inl"
#include "graphics3d.framemodel.inl"
#include "graphics3d.dynamicmodel.inl"
#include "graphics3d.springmodel.inl"

// Lights
#include "graphics3d.light.h"
#include "graphics3d.pointlight.h"
#include "graphics3d.spotlight.h"
#include "graphics3d.dirlight.h"

#include "graphics3d.light.inl"
#include "graphics3d.pointlight.inl"
#include "graphics3d.spotlight.inl"
#include "graphics3d.dirlight.inl"

// Shaders
#include "graphics3d.shader.h"
#include "graphics3d.plainshader.h"
#include "graphics3d.textureshader.h"
#include "graphics3d.colourshader.h"
#include "graphics3d.envshader.h"
#include "graphics3d.texenvshader.h"
#include "graphics3d.texenvbumpshader.h"
#include "graphics3d.envbumpshader.h"
#include "graphics3d.bumpshader.h"
#include "graphics3d.texbumpshader.h"

#include "graphics3d.shader.inl"
#include "graphics3d.plainshader.inl"
#include "graphics3d.textureshader.inl"
#include "graphics3d.colourshader.inl"
#include "graphics3d.envshader.inl"
#include "graphics3d.texenvshader.inl"
#include "graphics3d.texenvbumpshader.inl"
#include "graphics3d.envbumpshader.inl"
#include "graphics3d.bumpshader.inl"
#include "graphics3d.texbumpshader.inl"

// Translators
#include "graphics3d.translator.3ds.h"
#include "graphics3d.translator.ifs.h"
#include "graphics3d.translator.x.h"
#include "graphics3d.translator.ms3d.h"
#include "graphics3d.translator.md2.h"
#include "graphics3d.translator.md3.h"
#include "graphics3d.translator.md4.h"
#include "graphics3d.translator.mds.h"


// Algorithms
#include "graphics3d.technique.h"
#include "graphics3d.projectionvolume.h"
#include "graphics3d.stencilshadow.h"

#include "graphics3d.technique.inl"
#include "graphics3d.projectionvolume.inl"
#include "graphics3d.stencilshadow.inl"


#endif
