/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MATH_LINEARSYSTEM_H
#define __MATH_LINEARSYSTEM_H

#include "object.h"
#include "math.matrix.h"

namespace magnum { namespace math {
using magnum::types::String;
using magnum::object::Object;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class LinearSystem : public Object {
DeclareClass(LinearSystem, Object);
private:
	T*						m_System;
	T*						m_Inverse;
	unsigned int			m_Size;

public:
	LinearSystem(unsigned int size = 0);
	LinearSystem(const LinearSystem& );
	~LinearSystem();
	
	inline void				setSize(unsigned int );
	inline unsigned int		getSize() const;

	template<int dim> 
	inline					LinearSystem(const Matrix<T,dim>& );
	template<int dim> 
	inline bool				solve(const Vector<T,dim>& ,Vector<T,dim>* ) const;
	template<int dim> 
	inline LinearSystem&	operator=(const Matrix<T,dim>& );
	template<int dim> 
	void					apply(const Vector<T,dim>& ,Vector<T,dim>* ) const;
	
	inline bool				solve(const T* ,T* ) const;
	inline bool				solve(T* ) const;
	inline void				apply(const T* ,T* ) const;
	inline void				apply(T* ) const;
	inline bool				invert();

	inline const T&			operator()(unsigned int, unsigned int) const;
	inline T&				operator()(unsigned int, unsigned int);
	
	inline const T*			operator[](unsigned int) const;
	inline T*				operator[](unsigned int);

	LinearSystem&			operator=(const LinearSystem& );
};


}; };

#endif
