/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MATH_VECTOR_H
#define __MATH_VECTOR_H

#include "base.h"

#ifdef max
#undef max
#undef min
#endif

namespace magnum {namespace math {

template<class T, int dim> class Matrix;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Vector {
friend class Matrix<T,dim>;
public:
	T		vec[dim];

public:
	static const Vector		Null;

public:
	inline Vector();
	inline explicit Vector(const T* );

	inline Vector			operator+(const Vector &u) const;
	inline Vector			operator+(const T&) const;
	inline Vector			operator-(const Vector &u) const;
	inline Vector			operator-(const T& s) const;
	inline Vector			operator-() const;
	inline T				operator*(const Vector& u) const;
	inline Vector			operator*(const T& s) const;
	inline Vector			operator/(const T& s) const;

	inline const Vector&	operator= (const T& s);
	inline const Vector&	operator+= (const T& s);
	inline const Vector&	operator+= (const Vector& v);
	inline const Vector&	operator-= (const T& s);
	inline const Vector&	operator-= (const Vector& v);
	inline const Vector&	operator*= (const T& s);
	inline const Vector&	operator/= (const T& );

	inline bool				operator==(const Vector& ) const;
	inline bool				operator!=(const Vector& ) const;

	// The following operators are intended to enable the usage of containers
	inline bool				operator<(const Vector& ) const;
	inline bool				operator>(const Vector& ) const;
	inline bool				operator<=(const Vector& ) const;
	inline bool				operator>=(const Vector& ) const;

	inline					operator const T*() const;
	inline					operator T*();

	inline T&				operator()(unsigned int );
	inline const T&			operator()(unsigned int ) const;
	
	// Type conversion routines
	template<class S> inline operator Vector<S,dim>() const;
	template<class S> inline Vector<T,dim>(const Vector<S,dim>& v);

	inline Vector&			null();

	inline Vector&			normalize();

	inline T				length() const;
	inline T				length2() const;
	
	inline Vector			abs() const;

	inline T				max() const;
	inline T				min() const;
	inline T				maxabs() const;
	inline T				minabs() const;
	
	inline T				sum() const;
	inline T				sumabs() const;
};

template<class T, int dim> Vector<T,dim> operator*(const T& ,const Vector<T,dim>& );

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class _VectorNull : public Vector<T,dim> {
public:
	_VectorNull();
};


}; };

#endif
