/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MATH_VECTOR3_H
#define __MATH_VECTOR3_H

#include "math.vector.h"
#include "math.vector2.h"

namespace magnum {namespace math {

template<class T> class Matrix3;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Vector3 {
public:
	T			x,y,z;

public:
	static Vector3			Null;

public:
	inline Vector3();
	inline explicit Vector3(const T* );
	inline Vector3(const T& ,const T& , const T& );	
	inline Vector3(const Vector2<T>& );
	inline Vector3(const Vector<T,3>& );

	inline Vector3			operator+(const Vector3 &u) const;
	inline Vector3			operator+(const T&) const;
	inline Vector3			operator-(const Vector3 &u) const;
	inline Vector3			operator-(const T& s) const;
	inline Vector3			operator-() const;
	inline T				operator*(const Vector3& s) const;
	inline Vector3			operator*(const T& s) const;
	inline Vector3			operator/(const T& s) const;

	inline Vector3			operator%(const Vector3& u) const;

	inline const Vector3&	operator+= (const T& s);
	inline const Vector3&	operator+= (const Vector3& v);
	inline const Vector3&	operator-= (const T& s);
	inline const Vector3&	operator-= (const Vector3& v);
	inline const Vector3&	operator*= (const T& s);
	inline const Vector3&	operator/= (const T& );

	inline const Vector3&	operator=(const Vector<T,3>& );
	inline const Vector3&	operator=(const Vector<T,2>& );
	inline const Vector3&	operator=(const Vector2<T>& );

	inline bool				operator==(const Vector3& v) const;
	inline bool				operator!=(const Vector3& v) const;

	// The following operators are intended to enable the usage of containers
	inline bool				operator<(const Vector3& ) const;
	inline bool				operator>(const Vector3& ) const;
	inline bool				operator<=(const Vector3& ) const;
	inline bool				operator>=(const Vector3& ) const;

	inline					operator Vector<T,3>() const;
	inline					operator const T*() const;
	inline					operator		T*();

	// Type conversion routines
	template<class S> inline operator Vector3<S>() const {
		return Vector3<S>(S(x), S(y), S(z));
	};
	template<class S> inline Vector3<T>(const Vector3<S>& v) {
		x = T(v.x);
		y = T(v.y);
		z = T(v.z);
	};

	inline Vector2<T>		xy() const;
	inline Vector2<T>		xz() const;
	inline Vector2<T>		yz() const;
							
	inline Vector3&			null();

	inline Vector3&			normalize();

	inline T				length() const;
	inline T				length2() const;

	inline T				max() const;
	inline T				min() const;
	inline T				maxabs() const;
	inline T				minabs() const;
};

template<class T> inline Vector3<T> operator*(const T& ,const Vector3<T>& );
template<class T> inline Vector3<T> operator*(const Matrix<T,4>& ,const Vector3<T>& );
template<class T> inline Vector3<T> operator*(const Matrix<T,3>& ,const Vector3<T>& );


}; };

#endif
