/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MEDIA_BUFFER_INL
#define __MEDIA_BUFFER_INL

#include <string.h>

namespace magnum { namespace media {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Buffer::Buffer(Node* owner, int data) {
	m_Buffer = NULL;
	m_BufferSize = 0;
	m_Consumer = NULL;
	m_Producer = NULL;
	m_Owner = owner;
	m_OwnerData = data;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Buffer::~Buffer() {
	if (m_Buffer != NULL) {
		delete[] (byte *)m_Buffer;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Buffer::createBuffer(int size) {
	ASSERT(size > 0);

	if (m_BufferSize >= size)
		return true;

	if (m_Buffer != NULL) {
		delete[] (byte *)m_Buffer;
	}

	m_BufferSize = size;
	m_Buffer = new byte[size];
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Buffer::releaseBuffer() {
	if (m_Buffer != NULL) {
		delete[] (byte *)m_Buffer;
	}

	m_Buffer = NULL;
	m_BufferSize = 0;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Buffer::lock() {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Buffer::unlock() {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* Buffer::getBuffer() const {
	return m_Buffer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Buffer::getBufferSize() const {
	return m_BufferSize;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Buffer::setBufferSize(int s) {
	ASSERT(s > 0);
	if (s > m_BufferSize)
		return false;
	m_BufferSize = s;
	return true;
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Buffer::getBuffer(void* dst,int offset, int size) const {
	if (m_Buffer == NULL)
		return false;
	if (size == -1)
		size = m_BufferSize - offset;
	else if ((size + offset) > m_BufferSize)
		return false;

	memcpy(dst, ((byte *)m_Buffer)+offset, size);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Buffer::setBuffer(const void* src,int offset, int size) {
	if (m_Buffer == NULL)
		return false;
	if (size == -1)
		size = m_BufferSize - offset;
	else if ((size + offset) > m_BufferSize)
		return false;

	memcpy(((byte *)m_Buffer)+offset, src, size);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputPin* Buffer::getConsumer() const {
	return m_Consumer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputPin* Buffer::getProducer() const {
	return m_Producer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node* Buffer::getOwner() const {
	return m_Owner;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Buffer::getOwnerData() const {
	return m_OwnerData;
}


}; };

#endif
