/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MEDIA_DEVICE_H
#define __MEDIA_DEVICE_H

#include "object.h"
#include "media.node.h"
#include "container.list.h"

namespace magnum { namespace media {
using namespace magnum::container;
using magnum::types::String;
using magnum::object::Object;
using magnum::system::Thread;

class Node;
class Filter;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Device : public Object {
DeclareClass(Device, Object);
public:
	static const char DefaultAudio[];
	static const char DefaultVideo[];

	typedef Node* (*NodeCreator)(Device* dev, const char* name);
	typedef Device* (*DeviceCreator)(const char* name);

	enum Type {
		Audio,
		Video,
		Mixed
	};

private:
	class DeviceInfo : public Object {
	DeclareClass(DeviceInfo, Object);
	public:
		String				m_Name;
		Type				m_Type;
		DeviceCreator		m_Creator;
	};

	static List<DeviceInfo> m_DeviceList;

private:
	class NodeInfo : public Object {
	DeclareClass(NodeInfo, Object);
	public:
		String				m_Name;
		Node::Type			m_Type;
		NodeCreator			m_Creator;
	};

	List<NodeInfo>			m_NodeList;

public:
	Node*					createNode(const char* which = NULL);
	Filter*					createFilter(const char* which = NULL);

	static Device*			createDevice(const char *which, bool exculsive = false);

protected:
	bool					publishNode(const char *nodeName, Node::Type t, Device::NodeCreator c);

	static bool				publishDevice(const char* which, Device::DeviceCreator );
};


}; };

#endif
