/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MEDIA_HANDLER_H
#define __MEDIA_HANDLER_H

#include "object.h"
#include "container.chain.h"

namespace magnum { namespace media {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chainable;

class Pin;
class InputPin;
class OutputPin;
class Node;
class Buffer;
class Format;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Message : public Object {
DeclareClass(Message, Object);
friend class Node;
private:
	Node*		m_Node;
	Pin*		m_Pin;
	int			m_Command;
	int			m_Parameter;
	void*		m_Pointer;

public:
	enum {
		Connect,
		Disconnect,
		ReceiveBuffer,
		SendBuffer,
		RequestBuffer,
		Start,
		Stop,
		Pause,
		Unpause,
		SetFormat
	};

public:
	inline Message();
	inline Message(const Message& msg);
	inline Message(int command);
	inline Message(int command, int parameter);
	inline Message(int command, int parameter, void *pointer);
	inline Message(int command, void *pointer);
	
	inline int		getCommand() const;
	inline void		setCommand(int cmd);

	inline void*	getPointer() const;
	inline void		setPointer(void* );

	inline int		getParameter() const;
	inline void		setParameter(int param);

	inline Pin*		getPin() const;
	inline void		setPin(Pin* pin);

	inline Node*	getNode() const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Handler : public Object, public Chainable<Handler> {
DeclareClass(Handler, Object);
friend class Node;
public:

protected:
	virtual bool	handleMessage(const Message& msg);

	// (Dis)Connection methods
	virtual bool	onConnect(Pin* src);
	virtual bool	onDisconnect(Pin* src);

	// Buffer routines
	virtual bool	onReceiveBuffer(InputPin* ,Buffer* );
	virtual bool	onSendBuffer(OutputPin* ,Buffer* );
	virtual bool	onRequestBuffer(OutputPin* );

	// Start/Stop/Pause - this is system and device specific
	virtual bool	onStart();
	virtual bool	onStop();
	virtual bool	onPause();
	virtual bool	onUnpause();

	// Format methods
	virtual bool	onSetFormat(Pin* ,const Format* );
	virtual bool	onAdjustFormat(Pin* ,Format* );
};

}; };

#endif
