/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MEDIA_NODE_H
#define __MEDIA_NODE_H

#include "object.h"
#include "system.thread.h"
#include "system.mutex.h"
#include "container.list.h"
#include "container.queue.h"
#include "media.handler.h"

namespace magnum { namespace media {
using namespace magnum::container;
using magnum::object::Object;
using magnum::system::Mutex;
using magnum::container::Chain;
using magnum::types::String;

class Handler;
class Device;
class Buffer;
class Graph;
class Format;
class SourceNode;
class DrainNode;
class Pin;
class InputPin;
class OutputPin;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Node : public Handler, public Chainable<Node> {
DeclareClass(Node, Handler);
friend class Graph;
friend class Pin;
friend class Buffer;
friend class InputPin;
friend class OutputPin;

public:
	static const char DefaultInput[];
	static const char DefaultOutput[];
	static const char DefaultFilter[];
	static const char DefaultNode[];

	enum Type {
		Input,
		Output,
		Filter
	};

	enum State {
		Running,
		Stopped,
		Paused
	};


private:
	class PinInfo : public Object {
	DeclareClass(PinInfo, Object);
	public:
		String			m_Name;
		Pin*			m_Pin;
	};

	Chain<Handler>			m_Handlers;
	Graph*					m_Graph;
	Device*					m_Device;
	Mutex					m_Mutex;
	String					m_Name;
	List<PinInfo>			m_InputPins;
	List<PinInfo>			m_OutputPins;
	InputPin*				m_DefaultInput;
	OutputPin*				m_DefaultOutput;
	State					m_State;
	Type					m_Type;
	// Event Queue
	// Input Buffer Queue
	// List of Pins


public:
	Node(Device* dev, const char* name, Type t);
	~Node();

	inline void			addHandler(Handler* );
	inline void			removeHandler(Handler* );

	InputPin*			getInputPin(const char* which = NULL);
	OutputPin*			getOutputPin(const char* which = NULL);

	inline Device*		getDevice() const;
	inline Graph*		getGraph() const;

	inline String		getName() const;

	inline State		getState() const;

	inline bool			lock();
	inline bool			unlock();
	inline bool			isLocked() const;

protected:
	virtual bool		adjustFormat(Pin* ,Format* );

	Buffer*				receiveBuffer(InputPin* ,bool block=true);		// Input Buffer
	bool				sendBuffer(OutputPin* ,Buffer* ,bool block=true);	// Output Buffer

	bool				publishOutputPin(OutputPin* ,bool def=true);
	bool				publishInputPin(InputPin* ,bool def=true);

	bool				sendMessage(const Message& msg);
};


}; };

#endif
