/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MEDIA_PIN_H
#define __MEDIA_PIN_H

#include "object.h"
#include "container.queue.h"
#include "system.signal.h"
#include "system.mutex.h"
#include "media.format.h"
#include "media.buffer.h"

namespace magnum { namespace media {
using namespace magnum::container;
using magnum::system::Signal;
using magnum::system::Mutex;
using magnum::container::Queue;
using magnum::object::Object;
using magnum::types::String;

class Node;
class Graph;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Pin : public Object {
DeclareClass(Pin, Object);
friend class Graph;
friend class Node;
private:
	Node*			m_Node;
	Format			m_Format;
	String			m_Name;
	Pin*			m_Connection;

public:
	static const char DefaultInput[];
	static const char DefaultOutput[];

public:
	Pin(Node* node);
	~Pin();

	bool				requestFormat(const Format* );
	bool				adjustFormat(Format* );
	bool				setFormat(const Format* );

	inline bool			getFormat(Format* ) const;
	inline const Format* getFormat() const;

	inline Graph*		getGraph() const;
	inline Pin*			getConnection() const;
	inline Node*		getNode() const;

	inline bool			isConnected() const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class InputPin : public Pin {
DeclareClass(InputPin, Pin);
friend class Node;
private:
	Queue<Buffer>		m_BufferQueue;
	Signal				m_QueueSignal;
	Mutex				m_QueueMutex;

public:
	InputPin(Node* node);

	inline OutputPin*	getConnection() const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class OutputPin : public Pin {
DeclareClass(OutputPin, Pin);
friend class Node;
public:
	OutputPin(Node* node);

	inline InputPin*	getConnection() const;
};

}; };

#endif
