/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __NET_ADDRESS_INL
#define __NET_ADDRESS_INL

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Address::Address() {
	m_Hostname = "";
	m_HostnameValid = false;
	m_Address = 0;
	m_AddressValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Address::Address(const String& hostname) {
	m_Hostname = hostname;
	m_HostnameValid = true;
	m_AddressValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Address::Address(uint32 address) {
	m_Address = htonl(address);
	m_AddressValid = true;
	m_HostnameValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Address::getHostname() const {
	if (!m_HostnameValid)
		if (!((Address *)this)->resolveHostname())
			return String();
	return m_Hostname;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline uint32 Address::getAddress() const {
	return getAddressHO();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline uint32 Address::getAddressNO() const {
	if (!m_AddressValid)
		if (!((Address *)this)->resolveAddress())
			return 0;
	return m_Address;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline uint32 Address::getAddressHO() const {
	if (!m_AddressValid)
		if (!((Address *)this)->resolveAddress())
			return 0;
	return ntohl(m_Address);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Address::setHostname(const String& hostname) {
	m_Hostname = hostname;
	m_HostnameValid = true;
	m_AddressValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Address::setAddress(uint32 address) {
	setAddressHO(address);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Address::setAddressHO(uint32 address) {
	m_Address = htonl(address);
	m_AddressValid = true;
	m_HostnameValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Address::setAddressNO(uint32 address) {
	m_Address = address;
	m_AddressValid = true;
	m_HostnameValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Address Address::getByName(const String& hostname) {
	return Address(hostname);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Address Address::getByAddress(uint32 address) {
	return Address(address);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Address::clear()  {
	m_HostnameValid = false;
	m_AddressValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Address::isValid() const {
	return (m_HostnameValid || m_AddressValid);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Address::operator==(const Address& adr) const {
	if (m_AddressValid && adr.m_AddressValid)
		return (m_Address == adr.m_Address);
	if (m_HostnameValid && adr.m_HostnameValid)
		return (m_Hostname == adr.m_Hostname);
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Address::operator!=(const Address& adr) const {
	if (m_AddressValid && adr.m_AddressValid)
		return (m_Address != adr.m_Address);
	if (m_HostnameValid && adr.m_HostnameValid)
		return (m_Hostname != adr.m_Hostname);
	return true;
}


}; };
#endif
