/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __PHYSICS_RELAXATOR_H
#define __PHYSICS_RELAXATOR_H

#include "container.array.h"
#include "container.list.h"
#include "physics.weight.h"

namespace magnum { namespace physics {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Vector;
using magnum::container::Array;
using magnum::container::List;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Relaxator : public Object {
DeclareClass(Relaxator, Object);
public:
	typedef typename magnum::container::Array<Weight<T,dim> >::Iterator  Iterator;
	
private:
	Vector<T,dim>			m_Gravity;
	Array<Weight<T,dim> >	m_Weights;
	List<Spring<T,dim> >	m_Springs;
	T						m_Constant;
	T						m_Damping;
	
public:
	Relaxator();

	void					clear();
	
	void					setSize(unsigned int );
	inline unsigned int		getSize() const;
	
	inline bool				isEmpty() const;

	inline Weight<T,dim>&		operator[](unsigned int );
	inline const Weight<T,dim>& operator[](unsigned int ) const;
	
    inline Iterator     	begin() const;
    inline Iterator     	end() const;
    inline Iterator     	at(unsigned int ) const;
	
	Spring<T,dim>*			connect(unsigned int ,unsigned int ,T=T(1));
	void					disconnect(unsigned int ,unsigned int );
	bool					isConnected(unsigned int ,unsigned int ) const;
	
	inline void				setGravity(const Vector<T,dim>& );
	inline const Vector<T,dim>& getGravity() const;

	inline void				setConstant(const T& );
	inline const T&			getConstant() const;

	inline void				setDamping(const T& );
	inline const T&			getDamping() const;

	void					reset();

	inline void				relax(T time);
	template<class F>
	void					relax(F& func,T time);

private:
	static inline void		defaultFunctor(Weight<T,dim>* ,const T& time);	
};

}; };

#endif
