/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __PHYSICS_SPRING_INL
#define __PHYSICS_SPRING_INL

namespace magnum { namespace physics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Spring<T,dim>::Spring() {
	m_Constant = T(1);
	m_Force.null();
	m_Left = m_Right = NULL;
	m_LeftNext = m_RightNext = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Spring<T,dim>::setConstant(T c) {
	m_Constant = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Spring<T,dim>::getConstant() const {
	return m_Constant;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Spring<T,dim>::setNaturalLength(T l) {
	m_NaturalLength = l;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Spring<T,dim>::getNaturalLength() const {
	return m_NaturalLength;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Weight<T,dim>* Spring<T,dim>::getLeft() const {
	return m_Left;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Weight<T,dim>* Spring<T,dim>::getRight() const {
	return m_Right;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Spring<T,dim>::getLength() const {
	return m_Length;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Spring<T,dim>::getForce() const {
	return m_Force;
}

}; };

#endif
