/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_CAMERANODE_H
#define __SCENE3D_CAMERANODE_H

#include "scene3d.graphics.h"
#include "graphics3d.camera.h"
#include "geometry.transform.h"
#include "geometry.frustum.h"

namespace magnum { namespace scene3d {
using magnum::graphics3d::Camera;
using magnum::graphics::Rectangle;
using magnum::geometry::Frustum3;
using magnum::geometry::Transform;

class Observer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class CameraNode : public Graphics {
DeclareClass(CameraNode, Graphics);
friend class Observer;
private:
	Camera				m_Camera;
	Transform3<float>	m_Transform;
	Transform3<float>	m_WorldTransform;
	Frustum3<float>		m_Frustum;

public:
	inline CameraNode();

	inline void			setTransform(const Transform3<float>& );
	inline const Transform3<float>& getTransform() const;
	inline const Transform3<float>& getWorldTransform() const;

	inline void			setViewPlane(float );
	inline float		getViewPlane() const;
	inline void			setViewWindow(const Rectangle<float>& );
	inline void			setNearPlane(float );
	inline float		getNearPlane() const;
	inline void			setFarPlane(float );
	inline float		getFarPlane() const;

	inline const Frustum3<float>& getFrustum() const;

protected:
	inline void			onSelect() const;
	virtual void		onAttachLocale(Locale* );
	virtual void		onDetachLocale(Locale* );
	virtual void		onEnable();
	virtual void		onDisable();
	virtual void		onTransform(const Transform3<float>& parent);
	
private:
	void				updateFrustum();	
};

}; };

#endif
