/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_LEAF_INL
#define __SCENE3D_LEAF_INL

namespace magnum {namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Leaf<T>::Leaf(typename Leaf<T>::Type t) : Node((Node::Type)(t | Node::LeafType)) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Bounds<float,3>& Leaf<T>::getWorldBounds() const {
	return Placeable<float, 3, T>::getBounds();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Leaf<T>::setWorldBounds(const Bounds<float,3>& b) {
	Placeable<float, 3, T>::setBounds(b);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Leaf<T>::transformWorldBounds(const Transform<float,3>& t) {
	Placeable<float, 3, T>::setBounds(Bounds<float,3>(m_Bounds).transform(t));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Bounds<float,3>& Leaf<T>::getBounds() const {
	return m_Bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Leaf<T>::setBounds(const Bounds<float,3>& b) {
	m_Bounds = b;
	applyParentTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Leaf<T>::onEnable() {
	if (getLocale() == NULL) 
		return;
	ASSERT(getSpace() == NULL);
	getLocale()->insert((T*)this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Leaf<T>::onDisable() {
	if (getLocale() == NULL) 
		return;
	ASSERT(getSpace() != NULL);
	refer();
	getLocale()->remove((T*)this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Leaf<T>::onAttachLocale(Locale* l) {
	Node::onAttachLocale(l);
	ASSERT(getSpace() == NULL);
	if (!isEnabled())
		return;
	l->insert((T*)this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Leaf<T>::onDetachLocale(Locale* l) {
	Node::onDetachLocale(l);
	if (!isEnabled())
		return;
	ASSERT(getSpace() != NULL);
	refer();
	l->remove((T*)this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Leaf<T>::onTransform(const Transform3<float>& parent) {
	transformWorldBounds(parent);
}



}; };

#endif
