/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_LOCALE_H
#define __SCENE3D_LOCALE_H

#include "object.h"
#include "container.chain.h"
#include "math.vector3.h"
#include "geometry.space.h"
#include "scene3d.leaf.h"

namespace magnum {namespace scene3d {
using magnum::container::Chainable;
using magnum::container::Chain;
using magnum::geometry::Space;
using magnum::geometry::Placeable;
using magnum::math::Vector3;

class Universe;
class Root;
class Graphics;
class Audio;
class Link;
class Behaviour;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Locale : public Object, private Chainable<Locale> {
DeclareClass(Locale, Object);
friend class Universe;
friend class Leaf<Link>;
friend class Leaf<Audio>;
friend class Leaf<Graphics>;
friend class Leaf<Behaviour>;
private:
	Universe*					m_Universe;
	Chain<Root>					m_Roots;
	Space<float, 3, Graphics>	m_GraphicsSpace;
	Space<float, 3, Audio>		m_AudioSpace;
	Space<float, 3, Behaviour>	m_BehaviourSpace;
	Vector3<double>				m_Origin;

public:
	typedef magnum::container::Chain<Root>::Iterator Iterator;

public:
	inline Locale(Universe* );
	inline Locale(Universe* ,const Vector3<double>& vec);

	void					insert(Root* );
	void					remove(Root* );
	bool					contains(Root* ) const;
	void					clear();
							
	inline bool				isEmpty() const;
							
	inline Universe*		getUniverse() const;

	inline const Space<float,3,Graphics>* getGraphicsSpace() const;
	inline const Space<float,3,Audio>* getAudioSpace() const;
	inline const Space<float,3,Behaviour>* getBehaviourSpace() const;

	inline const Vector3<double>& getOrigin() const;
	inline void				setOrigin(const Vector3<double>& );
							
	inline Iterator			begin() const;
	inline Iterator			end() const;
	Iterator				at(const char* ) const;

public:
	// Some dirty hack for traversers - we cannot make a traverser a friend
	// of a locale (although this would be very reasonable), beacause
	// the traverser is a template.
	inline void				getSpace(const Space<float, 3, Graphics>* & ) const;
	inline void				getSpace(const Space<float, 3, Audio>* & ) const;
	inline void				getSpace(const Space<float, 3, Behaviour>* & ) const;

private:
	// Some dirty hacks for Leafs
	inline void				insert(Graphics* );
	inline void				remove(Graphics* );
	inline void				insert(Audio* );
	inline void				remove(Audio* );
	inline void				insert(Behaviour* );
	inline void				remove(Behaviour* );
	inline void				insert(Link* );
	inline void				remove(Link* );
};

}; };

#endif
