/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_NODE_INL
#define __SCENE3D_NODE_INL

namespace magnum { namespace scene3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node::Type Node::getType() const {
	return m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Node::getName() const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Node::setName(const char* n) {
	m_Name = n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group* Node::getParent() const {
	return m_Parent;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Locale* Node::getLocale() const {
	return m_Locale;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Universe* Node::getUniverse() const {
	if (m_Locale == NULL)
		return NULL;
	return m_Locale->getUniverse();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Node::isEnabled() const {
	return ((m_Flags & Enabled) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	Node::getFlags() const {
	return m_Flags;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Node::setFlag(Flag f) {
	m_Flags |= f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Node::clearFlag(Flag f) {
	m_Flags &= ~f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Node::testFlag(Flag f) const {
	return ((m_Flags & f) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Node::hasParentTransform() const {
	return ((m_Flags & ParentTransform) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Node::applyParentTransform() {
	m_Flags |= DirtyTransform;
	if (m_PrevDirty)
		return;
	enqueueUpdateTransform();
}


}; };

#endif
