/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SYSTEM_PROGRAM_INL
#define __SYSTEM_PROGRAM_INL

namespace magnum {namespace system {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Program::execute() const {
	ASSERT(m_Program.getSize() > 0);
	int (*fn)();
	fn = (int (*)())m_Program.getArray();
	return fn();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Program::execute(Location l) const {
	ASSERT(m_Program.getSize() > 0);
	ASSERT(l != NULL);
	int (*fn)();
	fn = (int (*)())l;
	return fn();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Program::addByte(byte b) {
	m_Program.insert(b);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Program::addWord(word w) {
#if defined(__LSBFIRST__)
	m_Program.insert((byte)w);
	m_Program.insert((byte)(w >> 8));
#else
	m_Program.insert((byte)(w >> 8));
	m_Program.insert((byte)w);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Program::addDword(dword d)  {
#if defined(__LSBFIRST__)
	m_Program.insert((byte)d);
	m_Program.insert((byte)(d >> 8));
	m_Program.insert((byte)(d >> 16));
	m_Program.insert((byte)(d >> 24));
#else
	m_Program.insert((byte)(d >> 24));
	m_Program.insert((byte)(d >> 16));
	m_Program.insert((byte)(d >> 8));
	m_Program.insert((byte)d);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Program::addFloat(float f) {
	addDword(*(dword *)(void *)&f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Program::addDouble(double d) {
	addDword(*(dword *)(void *)&d);
	addDword(*(((dword *)(void *)&d)+1));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Array<Program::Label>& Program::getLabels() const {
	return m_Labels;
}


}; };

#endif
