/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SYSTEM_THREAD_H
#define __SYSTEM_THREAD_H

#include "object.h"
#include "base.types.h"
#include "container.chain.h"
#include "system.mutex.h"
#include "system.signal.h"

# if defined(__FORUNIX__)
#	include <pthread.h>
# endif

namespace magnum { namespace system {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chain;
using magnum::container::Chainable;
using magnum::types::dword;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Thread : public Object, private Chainable<Thread> {
DeclareClass(Thread, Object);
public:
#if defined(__FORWINDOWS__)
	typedef dword Storage;
#elif defined(__FORUNIX__)
	typedef pthread_key_t Storage;
#endif
	enum {
		StorageNone = -1,
	};

private:
	class Startup {
	public:
		Startup();
	};
	friend Startup;

	static Startup			g_Startup;
	static Chain<Thread>	g_Threads;
	static Mutex			g_Mutex;
	static Storage			g_Current;

	Signal					m_Signal;

#if defined(__FORWINDOWS__)
	HANDLE					m_ThreadHandle;
	DWORD					m_ThreadId;
#elif defined(__FORUNIX__)
	pthread_t				m_ThreadId;
#endif


public:
	Thread();
	~Thread();

	bool				start(bool sync=false);
	bool				suspend();
	bool				resume();
	bool				join() const;
	bool				join(unsigned int millis) const;
						
	inline bool			isAlive() const;
	inline bool			isInterrupted() const;
						
	inline int			getPriority() const;
	void				setPriority(int p);

	inline int			getThreadId() const;

	static bool			sleep(unsigned int millis);
	static void			exit();

	static Storage		createTLS();
	static void			releaseTLS(Storage s);
	inline static void	setTLS(Storage ,const void *); 
	inline static void*	getTLS(Storage );

	inline static Thread* getCurrent();

protected:
	virtual bool		run();

private:
	static Thread*		searchCurrent();
	static Thread*		createCurrent();

#if defined(__FORWINDOWS__)
	static DWORD WINAPI	ThreadProc(LPVOID param);
#elif defined(__FORUNIX__)
	static void* ThreadProc(void* param);
#endif
};

}; };
#endif
