/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "application.message.h"
#include "container.allocator.h"
#include "system.mutex.h"

#include "base.string.inl"
#include "object.class.inl"
#include "object.inl"
#include "application.message.inl"
#include "container.chain.inl"
#include "container.queue.inl"
#include "container.iterator.inl"
#include "container.allocator.inl"
#include "graphics.rectangle.inl"
#include "graphics.point.inl"
#include "system.mutex.inl"



namespace magnum { namespace application {
using magnum::system::Mutex;
using magnum::container::Allocator;

static Allocator<Message::Parameter>		g_ParameterAllocator;
static Allocator<Message>					g_MessageAllocator;
static Mutex								g_AllocatorMutex;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void* Message::operator new(size_t ) {
	Message* msg;
	g_AllocatorMutex.lock();
	msg = g_MessageAllocator.alloc();
	g_AllocatorMutex.unlock();
	return msg;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::operator delete(void* ptr) {
	g_AllocatorMutex.lock();
	g_MessageAllocator.free((Message*)ptr);
	g_AllocatorMutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void* Message::Parameter::operator new(size_t ) {
	Parameter* parm;
	g_AllocatorMutex.lock();
	parm = g_ParameterAllocator.alloc();
	g_AllocatorMutex.unlock();
	return parm;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::Parameter::operator delete(void* ptr) {
	g_AllocatorMutex.lock();
	g_ParameterAllocator.free((Parameter*)ptr);
	g_AllocatorMutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addPointer(const char* name,void* value) {
	Parameter *param = new Parameter();
	param->m_Type = Pointer;
	param->m_Pointer = value;
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addInteger(const char* name,int value) {
	Parameter *param = new Parameter();
	param->m_Type = Integer;
	param->m_Integer = value;
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addFloat(const char* name,float value) {
	Parameter *param = new Parameter();
	param->m_Type = Float;
	param->m_Float = value;
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addDouble(const char* name,double value) {
	Parameter *param = new Parameter();
	param->m_Type = Float;
	param->m_Double = value;
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addRectangle(const char* name, const Rectangle<int>& value) {
	Parameter *param = new Parameter();
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	param->m_Type = RectangleInt;
	param->m_RectangleInt[0] = value.getX();
	param->m_RectangleInt[1] = value.getY();
	param->m_RectangleInt[2] = value.getWidth();
	param->m_RectangleInt[3] = value.getHeight();
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addRectangle(const char* name, const Rectangle<float>& value) {
	Parameter *param = new Parameter();
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	param->m_Type = RectangleFloat;
	param->m_RectangleFloat[0] = value.getX();
	param->m_RectangleFloat[1] = value.getY();
	param->m_RectangleFloat[2] = value.getWidth();
	param->m_RectangleFloat[3] = value.getHeight();
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addPoint(const char* name, const Point<int>& value) {
	Parameter *param = new Parameter();
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	param->m_Type = PointInt;
	param->m_PointInt[0] = value.getX();
	param->m_PointInt[1] = value.getY();
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::addPoint(const char* name, const Point<float>& value) {
	Parameter *param = new Parameter();
	strncpy(param->m_Name, name, sizeof(param->m_Name)/sizeof(char));
	param->m_Type = PointFloat;
	param->m_PointFloat[0] = value.getX();
	param->m_PointFloat[1] = value.getY();
	m_Parameters.insert(param);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Message::Parameter* Message::findParameter(const char* name) const {
	Iterator iter = m_Parameters.begin();
	Iterator end = m_Parameters.end();
	Parameter* data;

	while (iter != end) {
		data = &(*iter);
		if (strncmp(data->m_Name, name, sizeof(data->m_Name)/sizeof(char)) == 0)
			return data;
		iter++;
	}

	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Message::cloneParameters(const Chain<Parameter>& chain) {
	Iterator iter = chain.begin();
	Iterator end = chain.end();

	while (iter != end) {
		Parameter* param = new Parameter;
		strncpy(param->m_Name, (*iter).m_Name, sizeof(param->m_Name)/sizeof(char));
		param->m_Type = (*iter).m_Type;
		switch (param->m_Type) {
			case Pointer:
				param->m_Pointer = (*iter).m_Pointer;
				break;
			case Integer:
				param->m_Integer = (*iter).m_Integer;
				break;
			case Float:
				param->m_Float = (*iter).m_Float;
				break;
			case Double:
				param->m_Double = (*iter).m_Double;
				break;
			case Text:
				param->m_String = (*iter).m_String;
				param->m_String->refer();
				break;
			case PointInt:
				param->m_PointInt[0] = (*iter).m_PointInt[0];
				param->m_PointInt[1] = (*iter).m_PointInt[1];
				break;
			case PointFloat:
				param->m_PointFloat[0] = (*iter).m_PointFloat[0];
				param->m_PointFloat[1] = (*iter).m_PointFloat[1];
				break;
			case RectangleInt:
				param->m_RectangleInt[0] = (*iter).m_RectangleInt[0];
				param->m_RectangleInt[1] = (*iter).m_RectangleInt[1];
				param->m_RectangleInt[2] = (*iter).m_RectangleInt[2];
				param->m_RectangleInt[3] = (*iter).m_RectangleInt[3];
				break;
			case RectangleFloat:
				param->m_RectangleFloat[0] = (*iter).m_RectangleFloat[0];
				param->m_RectangleFloat[1] = (*iter).m_RectangleFloat[1];
				param->m_RectangleFloat[2] = (*iter).m_RectangleFloat[2];
				param->m_RectangleFloat[3] = (*iter).m_RectangleFloat[3];
				break;
		}
		m_Parameters.insert(param);
		iter++;
	}
}


}; };
