/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "io.exception.h"
#include "graphics.translator.dds.h"

#include "object.inl"
#include "object.exception.inl"
#include "base.string.inl"
#include "graphics.translator.inl"
#include "graphics.palette.inl"
#include "graphics.colour.inl"
#include "graphics.image.inl"
#include "graphics.dimension.inl"


namespace magnum { namespace graphics {

struct DdsHeader {
	unsigned int	ddsIdentifier;
	unsigned int	headerSize;
	unsigned int	flags;
	unsigned int	height;
	unsigned int	width;
	unsigned int	linesize;
	unsigned int	depth;
	unsigned int	nMipMaps;
	
	unsigned char	junk3[52];
	unsigned int	fourCC;
	unsigned int	bpp;
	unsigned char	junk4[36];
};


#define MCHAR2(a, b) (a | (b << 8))
#define MCHAR4(a, b, c, d) (a | (b << 8) | (c << 16) | (d << 24))



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* TranslatorDDS::onDecode(InputStream* in, long size) {
	DdsHeader header;
	Image::Type type;
	Image* img, *nextImg, *root;
	int width, height, nMipMaps;

	if (size == Autosize)
		size = in->available();

	in->read(&header, sizeof(header));
	if (header.ddsIdentifier != MCHAR4('D', 'D', 'S', ' '))
		throw magnum::io::TranslatorFormatException();

	width = header.width;
	height = header.height;
	nMipMaps = header.nMipMaps;
	if (nMipMaps <= 0) {
		nMipMaps = 1; 
	}

	switch(header.fourCC){
		case MCHAR4('D', 'X', 'T', '1'):
			type = Image::DXT1;
			break;
		case MCHAR4('D', 'X', 'T', '3'):
			type = Image::DXT3;
			break;
		case MCHAR4('D', 'X', 'T', '5'):
			type = Image::DXT5;
			break;
		default:
			switch(header.bpp){
				case 16:
					type = Image::R5G6B5;
					break;
				case 24:
					type = Image::BGR8;
					break;
				case 32:
					type = Image::BGRA8;
					break;
				default:
					throw magnum::io::TranslatorFormatException();
			}
	}
	
	// Calculate how large buffer we need possibly including mipmaps
	root = NULL;
	while (nMipMaps > 0) {
		nextImg = new Image(type, width, height);
		if (root != NULL) {
			img->attachNext(nextImg);
			nextImg->release();
		} else {
			root = nextImg;
		}
		img = nextImg;

		void* data = img->lock();
		int readsize = img->getLinesize() * img->getHeight();

		in->read(data, readsize);

/*		if (useRGBA && isPlainFormat(format)){
			int nChannels = getChannels(format);
			flipChannels(pixels, readSize / nChannels, nChannels);
		}
*/		img->unlock();

		nMipMaps--;
		if (width > 1)
			width >>= 1;
		if (height > 1)
			height >>= 1;
	}

	return root;
}

 

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
long TranslatorDDS::onEncode(OutputStream* out, const Image* img) {
	return 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool TranslatorDDS::onIdentify(File* in) {
	dword id;

	try {
		unsigned long oldpos;
		oldpos = in->getPosition();	
		in->read(&id, sizeof(id));
		in->setPosition(oldpos, File::Absolute);
	} catch (...) {
		return false;
	}

	if (id != MCHAR4('D', 'D', 'S', ' '))
		return false;

	return true;
} 


}; };
