/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics3d.mode.h"
#include "graphics3d.modestack.h"
#include "graphics3d.context.h"
#include "graphics3d.vertexformat.h"

#include "base.string.inl"
#include "object.inl"
#include "math.range.inl"
#include "graphics.colour.inl"
#include "graphics3d.mode.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.context.inl"


namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mode::Mode() {
	m_DepthRange.setStart(0.0f);
	m_DepthRange.setEnd(1.0f);
	m_DepthTest = LessEqual;
	m_DepthWrite = false;
	m_StencilFail = m_StencilZFail = m_StencilPass = Keep;
	m_StencilTest = Disable;
	m_StencilRef = 0;
	m_StencilMask = 0xffffffff;
	m_StencilWrite = 0;
	m_BlendSource = One;
	m_BlendDestination = Zero;
	m_AlphaTest = Disable;
	m_AlphaRef = 0.0f;
	m_RedWrite = m_GreenWrite = m_BlueWrite = m_AlphaWrite = true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Mode::onSelect() const {
	ModeStack* ms = getContext()->getModeStack();

	// Colour buffer
	glColorMask(m_RedWrite, m_GreenWrite, m_BlueWrite, m_AlphaWrite);

	// Depth test realted settings
	if (m_DepthTest == Disable) {
		glDisable(GL_DEPTH_TEST);
	} else {
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(m_DepthTest);
	}
	glDepthMask(m_DepthWrite);
	glDepthRange(m_DepthRange.getStart(), m_DepthRange.getEnd());

	// Stenvil Buffer related settings
	if (m_StencilTest == Disable) {
		glDisable(GL_STENCIL_TEST);
	} else {
		glEnable(GL_STENCIL_TEST);
		glStencilFunc(m_StencilTest, m_StencilRef, m_StencilMask);
	}
	glStencilMask(m_StencilWrite);
	glStencilOp(m_StencilFail, m_StencilZFail, m_StencilPass);
	ms->m_StencilTest = m_StencilTest;
	ms->m_StencilRef = m_StencilRef;
	ms->m_StencilMask = m_StencilMask;

	// Blending
	if ((m_BlendSource == One) && (m_BlendDestination == Zero)) {
		glDisable(GL_BLEND);
	} else {
		glEnable(GL_BLEND);
		glBlendFunc(m_BlendSource ,m_BlendDestination);
	}
	if (glBlendEquationEXT)
		glBlendEquationEXT(m_BlendEquation);
	if (glBlendColorEXT)
		glBlendColorEXT(m_BlendColour.R, m_BlendColour.G, m_BlendColour.B, m_BlendColour.A);

	// Alpha test related settings
	if (m_AlphaTest == Disable) {
		glDisable(GL_ALPHA_TEST);
	} else {
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(m_AlphaTest, m_AlphaRef);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Mode::onDeselect() const {
}

}; };


