#include "graphics3d.parameterformat.h"

#include "container.array.inl"
#include "graphics3d.parameterformat.inl"


namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ParameterFormat::shrink() {
	for (unsigned int i = 0; i < m_Components.getSize(); i++) {
		if (m_Components[i].m_Type == 0) {
			m_Components.remove(i);
			i--;
		}
	}
	m_Components.shrink();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ParameterFormat::append(const ParameterFormat& fmt) {
	unsigned int i = m_Components.getSize();
	unsigned int offset = m_Length;

	m_Components.append(fmt.m_Components);
	m_Length += fmt.m_Length;
	for (; i < m_Components.getSize(); i++) {
		m_Components[i].m_Location += offset;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ParameterFormat::insert(const char* name, const char* desc, Types::Type t, ParameterBuffer::Location l) {
	Component cmp;
	strncpy(cmp.m_Description, desc, sizeof(cmp.m_Description));
	strncpy(cmp.m_Name, name, sizeof(cmp.m_Name));
	cmp.m_Type = t;
	cmp.m_Location = l;
	m_Length += ((t >> Types::SizeShift) & Types::SizeMask);
	m_Components.insert(cmp);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ParameterFormat::insert(const char* name, Types::Type t, ParameterBuffer::Location l) {
	Component cmp;
	strncpy(cmp.m_Name, name, sizeof(cmp.m_Name));
	cmp.m_Description[0] = 0;
	cmp.m_Type = t;
	cmp.m_Location = l;
	m_Length += ((t >> Types::SizeShift) & Types::SizeMask);
	m_Components.insert(cmp);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ParameterFormat::remove(const char* name) {
	unsigned int size = 0;
	for (unsigned int i = 0; i < m_Components.getSize(); i++) {
		if (strncmp(m_Components[i].m_Name, name, sizeof(m_Components[i].m_Name)) == 0) {
			size = ((m_Components[i].m_Type >> Types::SizeShift) & Types::SizeMask);
			m_Length -= size;
			m_Components.remove(i);
			break;
		}
	}
	for (;i < m_Components.getSize(); i++) {
		m_Components[i].m_Location -= size;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ParameterFormat::contains(const char* name) const {
	for (unsigned int i = 0; i < m_Components.getSize(); i++) {
		if (strncmp(m_Components[i].m_Name, name, sizeof(m_Components[i].m_Name)) == 0)
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ParameterFormat::contains(ParameterBuffer::Location l) const {
	for (unsigned int i = 0; i < m_Components.getSize(); i++) {
		if (m_Components[i].m_Location == (unsigned int)l)
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ParameterFormat::Iterator ParameterFormat::at(const char* name) const {
	Iterator iter;
	for (iter = m_Components.begin(); iter != m_Components.end(); iter++) {
		if (strncmp(iter->m_Name, name, sizeof(iter->m_Name)) == 0)
			break;
	}
	return iter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ParameterFormat::Iterator ParameterFormat::at(ParameterBuffer::Location l) const {
	Iterator iter;
	for (iter = m_Components.begin(); iter != m_Components.end(); iter++) {
		if (iter->m_Location == (unsigned int)l)
			break;
	}
	return iter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
const ParameterFormat::Component& ParameterFormat::operator[](const char* name) const {
	for (unsigned int i = 0; i < m_Components.getSize(); i++) {
		if (strncmp(m_Components[i].m_Name, name, sizeof(m_Components[i].m_Name)) == 0)
			return m_Components[i];
	}
	ASSERT(false);
	return *(Component *)NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
const ParameterFormat::Component& ParameterFormat::operator[](ParameterBuffer::Location l) const {
	for (unsigned int i = 0; i < m_Components.getSize(); i++) {
		if (m_Components[i].m_Location == (unsigned int)l)
			return m_Components[i];
	}
	ASSERT(false);
	return *(Component *)NULL;
}

}; };
