/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.image.h"
#include "graphics3d.context.h"
#include "graphics3d.texture.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "graphics3d.context.inl"
#include "graphics3d.texture.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"


namespace magnum { namespace graphics3d {
using magnum::graphics::Image;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Texture::Texture(unsigned int tex, unsigned int target) {
	int tmp;

	m_OglTexture = tex;
	m_OglTarget = target;
	m_MagFilter = Nearest;
	m_MinFilter = Nearest;
	m_WrapS = Repeat;
	m_WrapT = Repeat;
	m_WrapU = Repeat;

	glBindTexture(m_OglTarget, tex);
	glGetTexLevelParameteriv(m_OglTarget, 0, GL_TEXTURE_INTERNAL_FORMAT, &tmp);
	switch (tmp) {
		case 1: m_Type = Image::Index8; break;
		case GL_RGB8: m_Type = Image::RGB8; break;
		case GL_RGBA8: m_Type = Image::RGBA8; break;
		default: m_Type = Image::Unknown;
	}
	glTexParameteri(m_OglTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(m_OglTarget, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_R, GL_REPEAT);
	glBindTexture(m_OglTarget, 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Texture::compileSelect(Assembler& as, unsigned int stage) const {
	as.asmCall((void (Texture::*)(unsigned int ) const)Texture::forceSelect, this, stage);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int Texture::mapOutputFormat(Type t) {
	switch (t) {
		case Image::BGRA8:
			return GL_RGBA8;
		case Image::BGR8:
			return GL_RGB8;
		case Image::RGBA8:
			return GL_RGBA8;
		case Image::RGB8:
			return GL_RGB8;
		case Image::R5G6B5:
			return GL_RGB5;
		case Image::RGB5:
			return GL_RGB5;
		case Image::Index8:
			return 1;
		case Image::A8:
			return GL_ALPHA8;
		case Image::DUDV8:
			return GL_DU8DV8_ATI;
		default:
			return -1;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int Texture::mapInputFormat(Type t) {
	switch (t) {
		case Image::BGRA8:
			return GL_BGRA_EXT;
		case Image::BGR8:
			return GL_BGR_EXT;
		case Image::RGBA8:
			return GL_RGBA;
		case Image::RGB8:
			return GL_RGB;
		case Image::R5G6B5:
			return GL_RGB;
		case Image::RGB5:
			return GL_RGB;
		case Image::Index8:
			return GL_COLOR_INDEX;
		case Image::DUDV8:
			return GL_DUDV_ATI;
		case Image::A8:
			return GL_ALPHA;
		default:
			return -1;
	}
}

}; };
