/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "graphics3d.vertexformat.inl"

namespace magnum { namespace graphics3d { 


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void VertexFormat::set(unsigned int i, VertexFormat::Type t, VertexFormat::Attribute m) {
	ASSERT(i < MaxComponents);

	int sdiff;
	sdiff = ((t >> Types::SizeShift) & Types::SizeMask) 
		- ((m_Component[i].m_Type >> Types::SizeShift) & Types::SizeMask);

	m_Component[i].m_Attribute = m;
	m_Component[i].m_Type = t;

	if (m_Size <= i) {
		for (;m_Size <= i; m_Size++)
			m_Component[m_Size].m_Offset = m_Stride;
	} else {
		for (i++; i < m_Size; i++)
			m_Component[i].m_Offset += sdiff;
	}

	m_Stride += sdiff;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
unsigned int VertexFormat::insert(Type t, Attribute m) {
	unsigned int i,ret;

	for (i = 0; i < m_Size; i++) {
		if (m_Component[i].m_Attribute == 0) 
			break;
	}

	if (i == MaxComponents)
		return -1;

	ret = i;
	m_Component[i].m_Attribute = m;
	m_Component[i].m_Type = t;
	m_Component[i].m_Offset = (i == 0) ? 0 : 
		m_Component[i-1].m_Offset + ((m_Component[i-1].m_Type >> Types::SizeShift) & Types::SizeMask);

	if (i == m_Size) {
		m_Size++;
	} else {
		for (i++; i < m_Size; i++) {
			m_Component[i].m_Offset += (t >> Types::SizeShift) & Types::SizeMask;
		}
	}

	m_Stride += (t >> Types::SizeShift) & Types::SizeMask;

	return ret;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void VertexFormat::remove(unsigned int i) {
	ASSERT(i < MaxComponents);

	int sdiff = (m_Component[i].m_Type >> Types::SizeShift) & Types::SizeMask;
	m_Stride -= sdiff;

	m_Component[i].m_Attribute = (Attribute)0;
	m_Component[i].m_Type = (Type)0;

	for (i++; i < m_Size; i++) {
		m_Component[i].m_Offset -= sdiff;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
unsigned int VertexFormat::shrink() {
	unsigned int src = 0, dst = 0, del=0;
	while (dst < m_Size) {
		if ((m_Component[src].m_Type != 0) || (m_Component[src].m_Attribute != 0)) {
			if (dst != src)
				memcpy(&m_Component[dst], &m_Component[src], sizeof(Component));
			dst++;
		} else {
			m_Size--;
		}
		src++;
	}

	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void VertexFormat::append(const VertexFormat& fmt) {
	unsigned int i;
	for (i = 0; i < fmt.m_Size; i++) {
		if ((fmt[i].m_Type != 0) || (fmt[i].m_Attribute != 0)) {
			insert(fmt[i].m_Type, fmt[i].m_Attribute);
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool VertexFormat::operator==(const VertexFormat& fmt) const {
	unsigned int i;
	for (i = 0; i < MaxComponents; i++) {
		if ((m_Component[i].m_Attribute == 0) && (fmt.m_Component[i].m_Attribute == 0))
			continue;
		if ((m_Component[i].m_Attribute != fmt.m_Component[i].m_Attribute)
		 || (m_Component[i].m_Type != fmt.m_Component[i].m_Type)
		 || (m_Component[i].m_Offset != fmt.m_Component[i].m_Offset))
			return false;
	}

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool VertexFormat::operator!=(const VertexFormat& fmt) const {
	unsigned int i;
	for (i = 0; i < MaxComponents; i++) {
		if ((m_Component[i].m_Attribute == 0) && (fmt.m_Component[i].m_Attribute == 0))
			continue;
		if ((m_Component[i].m_Attribute != fmt.m_Component[i].m_Attribute)
		 || (m_Component[i].m_Type != fmt.m_Component[i].m_Type)
		 || (m_Component[i].m_Offset != fmt.m_Component[i].m_Offset))
			return true;
	}

	return false;
}

}; };

