/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "gui.view.h"
#include "gui.window.h"
#include "container.chain.h"
#include "system.thread.h"
#include "application.looper.h"
#include "application.message.h"
#include "system.h"
#include "application.keyboard.h"

#include "container.chain.inl"
#include "container.iterator.inl"
#include "system.system.inl"
#include "system.thread.inl"
#include "base.string.inl"
#include "application.looper.inl"
#include "application.message.inl"
#include "gui.view.inl"
#include "gui.window.inl"
#include "gui.point.inl"
#include "gui.dimension.inl"
#include "gui.rectangle.inl"

#include <memory.h>

#ifndef __FORX11__
#	error "This file is only for X11"
#endif

namespace magnum { namespace ui {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::windowCreate() {
	// Default Event Mask that will capture everything
	static int mask =
		SubstructureRedirectMask |
		KeyPressMask |     
		KeyReleaseMask |                 
		ButtonPressMask |               
		ButtonReleaseMask |              
		EnterWindowMask |            
		LeaveWindowMask |               
		PointerMotionMask |             
		KeymapStateMask |               
		ExposureMask |                  
		VisibilityChangeMask |          
		StructureNotifyMask |           
		FocusChangeMask |               
		ColormapChangeMask |            
		OwnerGrabButtonMask;

	int width = m_Bounds.getWidth();
	int height = m_Bounds.getHeight();

	Window root = XDefaultRootWindow(m_Display);
	int screen = XDefaultScreen(m_Display);
	Visual *visual = XDefaultVisual(m_Display, screen);
	int depth = XDefaultDepth(m_Display, screen);

	// Fill out window attributes...
	XSetWindowAttributes attribs;
	attribs.background_pixmap = None;
	attribs.background_pixel = 0;
	attribs.border_pixmap = CopyFromParent;
	attribs.border_pixel = 0;
	attribs.backing_store = Always;
	attribs.event_mask = mask;

	// CReate the window
	m_WindowHandle = XCreateWindow(m_Display, root,
					0, 0, width, height,
					CopyFromParent,
					depth, 
					InputOutput,
					visual,
					CWBackPixmap | CWBorderPixel | CWBackingStore,
					&attribs);

	// Create Device Handle
	m_DeviceHandle = XCreateC(m_Display, m_WindowHandle, 0, NULL);
          
	// Set window title
	XStoreName(m_Display, m_WindowHandle, m_Title.getString());

	// Select input
	XSelectInput(m_Display, m_WindowHandle, mask);

	// Set window property...
	void* data = this;
	XChangeProperty(m_Display, m_WindowHandle, Looper::g_WindowPointer, GLooper::g_IntegerType, 32, PropModeReplace, (unsigned char*)&data, sizeof(data)/4);

	// Create a palette
  	m_PaletteHandle = XCreateColormap(m_Display, m_WindowHandle, visual, AllocNone);
  	m_PaletteColours = new XColor[256];

	return true;

/*	int i;
  	for(i=0;i<256;i++) {
  		if(!XAllocColorCells(m_Display,m_PaletteHandle,0,0,0,&m_PaletteColours[i],1)) {
   			return(0);
  		} else {
	    	m_PaletteColours[i].flags = DoRed | DoGreen | DoBlue;
	    	m_PaletteColours[i].red = i;
	    	m_PaletteColours[i].green = i;
	    	m_PaletteColours[i].blue = i;
	    	m_PaletteColours[i].pixel = m_Colours[i];
	   	}
   	}

	return true; */
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::windowDestroy() {
	XUnmapWindow(m_Display, m_WindowHandle);
	XFreeC(m_Display, m_DeviceHandle);
	XFreeColormap(m_Display, m_PaletteHandle);
	XDestroyWindow(m_Display, m_WindowHandle);
	delete[] m_PaletteColours;

	m_WindowHandle = 0;
	m_DeviceHandle = 0;
	m_PaletteHandle = 0;
    m_PaletteColours = NULL;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::windowShow() {
	if (m_Display == NULL)
		return false;

	XMapWindow(m_Display, m_WindowHandle);
	XClearArea(m_Display, m_WindowHandle, 0, 0, m_Bounds.getWidth(), m_Bounds.getHeight(), True);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::dispatchEvent(const XEvent& event) {
	int flags, len;
    KeySym keysym;
	Rectangle<int> rect;
	char buffer[128], *ptr;

	// This is a foreign event!
	if (event.xany.window != m_WindowHandle) {
PRINT("Window::dispatchEvent: Wrong Window!");
		Window* window;
		View* view;
		Atom returnType;
		int returnFormat;
		unsigned long returnN = 0;
		unsigned long returnRemaining = 0;
		unsigned char *returnProp = NULL;

		if (event.xany.window == 0)
			return true;

		if (XGetWindowProperty(m_Display, event.xany.window, Looper::g_ViewPointer, 0, sizeof(window)/4, False, AnyPropertyType, &returnType, &returnFormat, &returnN, &returnRemaining, &returnProp) == Success) {
			view = (GView*)returnProp;
PRINT1("Propagating to View at 0x%x...",view);
			return view->dispatchEvent(event);			
		}

		return false;
	}

	// Here is the right stuff...
	switch(event.type) {
		case KeymapNotify:
PRINT("Window::dispatchEvent: KeymapNotify");
			return true;

		case KeyPress:
PRINT("Window::dispatchEvent: KeyPress");
			m_MousePosition.setX(event.xkey.x);
			m_MousePosition.setY(event.xkey.y);
			keysym = XLookupKeysym((XKeyEvent *)&event.xkey, 0);
			onKeyDown((GKeyboard::GKey)keysym);
			len = XLookupString((XKeyEvent *)&event.xkey, buffer, 128, NULL, NULL);
			ptr = buffer;
			while (len > 0) {
				onChar((Character::Code)*ptr, (Character::Modifier)m_KeyboardModifiers);
				ptr++;
				len--;
			}
			return true;

		case KeyRelease:
PRINT("Window::dispatchEvent: KeyRelease");
			m_MousePosition.setX(event.xkey.x);
			m_MousePosition.setY(event.xkey.y);
			keysym = XLookupKeysym((XKeyEvent *)&event.xkey, 0);
			onKeyUp((GKeyboard::GKey)keysym);
			return true;

		case ButtonPress:
PRINT("Window::dispatchEvent: ButtonPress");
			m_KeyboardModifiers = 
				((event.xbutton.state & ShiftMask) ? Character::Shift : 0) |
				((event.xbutton.state & ControlMask) ? Character::Control : 0);
			m_MousePosition.setX(event.xbutton.x);
			m_MousePosition.setY(event.xbutton.y);
			if (event.xbutton.button == Button1) {
				m_MouseButtons |= Mouse::LeftDown;
				m_MouseButtons &= ~Mouse::LeftUp;
				onMouseButton(m_MousePosition, Mouse::LeftDown);
				return true;
			} else if (event.xbutton.button == Button2) {
				m_MouseButtons |= Mouse::MiddleDown;
				m_MouseButtons &= ~Mouse::MiddleUp;
				onMouseButton(m_MousePosition, Mouse::MiddleDown);
				return true;
			} else if (event.xbutton.button == Button3) {
				m_MouseButtons |= Mouse::RightDown;
				m_MouseButtons &= ~Mouse::RightUp;
				onMouseButton(m_MousePosition, Mouse::RightDown);
				return true;
			}
			return false;

		case ButtonRelease:
PRINT("Window::dispatchEvent: ButtonRelease");
			m_KeyboardModifiers = 
				((event.xbutton.state & ShiftMask) ? Character::Shift : 0) |
				((event.xbutton.state & ControlMask) ? Character::Control : 0);
			m_MousePosition.setX(event.xbutton.x);
			m_MousePosition.setY(event.xbutton.y);
			if (event.xbutton.button == Button1) {
				m_MouseButtons |= Mouse::LeftUp;
				m_MouseButtons &= ~Mouse::LeftDown;
				onMouseButton(m_MousePosition, Mouse::LeftUp);
				return true;
			} else if (event.xbutton.button == Button2) {
				m_MouseButtons |= Mouse::MiddleUp;
				m_MouseButtons &= ~Mouse::MiddleDown;
				onMouseButton(m_MousePosition, Mouse::MiddleUp);
				return true;
			} else if (event.xbutton.button == Button3) {
				m_MouseButtons |= Mouse::RightUp;
				m_MouseButtons &= ~Mouse::RightDown;
				onMouseButton(m_MousePosition, Mouse::RightUp);
				return true;
			}
			return false;

		case MotionNotify:
PRINT("Window::dispatchEvent: MotionNotify");
			m_KeyboardModifiers = 
				((event.xbutton.state & ShiftMask) ? Character::Shift : 0) |
				((event.xbutton.state & ControlMask) ? Character::Control : 0);
			m_MouseButtons = 
				((event.xmotion.state & Button1Mask) ? Mouse::LeftDown : Mouse::LeftUp) |
				((event.xmotion.state & Button2Mask) ? Mouse::MiddleDown : Mouse::MiddleUp) |
				((event.xmotion.state & Button3Mask) ? Mouse::RightDown : Mouse::RightUp);
			m_MousePosition.setX(event.xmotion.x);
			m_MousePosition.setY(event.xmotion.y);

			// Get other mouse-movements...
			while (XPending(m_Display) > 0) {
				XEvent event;
				XNextEvent(m_Display, &event);
				if ((event.type != MotionNotify) || (event.xany.window != m_WindowHandle)) {
					XPutBackEvent(m_Display, &event);
					break;
				} else {
					m_MousePosition.setX(event.xmotion.x);
					m_MousePosition.setY(event.xmotion.y);
				}
			}

			onMouseMove(m_MousePosition, m_MouseButtons);
			return true;

		case ConfigureNotify:
PRINT("Window::dispatchEvent: ConfigureNotify");
			if ((event.xconfigure.width != m_Bounds.getWidth()) 
			 || (event.xconfigure.height != m_Bounds.getHeight())) {
				m_Bounds.setX(event.xconfigure.x + event.xconfigure.border_width);
				m_Bounds.setY(event.xconfigure.y + event.xconfigure.border_width);
				m_Bounds.setWidth(event.xconfigure.width);
				m_Bounds.setHeight(event.xconfigure.height);
				onResize(m_Bounds.getSize());
				return true;
			}
			if (((event.xconfigure.x + event.xconfigure.border_width) != m_Bounds.getX()) 
			 || ((event.xconfigure.y + event.xconfigure.border_width) != m_Bounds.getY())) {
				m_Bounds.setX(event.xconfigure.x + event.xconfigure.border_width);
				m_Bounds.setY(event.xconfigure.y + event.xconfigure.border_width);
				onMove(m_Bounds.getLocation());
				return true;
			}
			return false;

		case FocusIn:
PRINT("Window::dispatchEvent: FocusIn");
			onActivate();
			return true;

		case FocusOut:
PRINT("Window::dispatchEvent: FocusOut");
			onDeactivate();
			return true;

		case MappingNotify:
PRINT("Window::dispatchEvent: MappingNotify");
			XRefreshKeyboardMapping((XMappingEvent *)&event.xmapping);
			return true;

		case Expose:
PRINT("Window::dispatchEvent: Expose");
			// Collect some Expose events for performance reasons...
			rect = GRectangle<int>(event.xexpose.x, event.xexpose.y, event.xexpose.width, event.xexpose.height);
			len = event.xexpose.count;
			while ((len > 0) && (XPending(m_Display) > 0)) {
				XEvent event;
				XNextEvent(m_Display, &event);
				if ((event.type != Expose) || (event.xany.window != m_WindowHandle)) {
					XPutBackEvent(m_Display, &event);
					break;
				}
				rect |= GRectangle<int>(event.xexpose.x, event.xexpose.y, event.xexpose.width, event.xexpose.height);
				len = event.xexpose.count;
			}
			onPaint(rect);
			return true;

		case GraphicsExpose:
PRINT("Window::dispatchEvent: GraphicsExpose");
			return true;

		case NoExpose:
PRINT("Window::dispatchEvent: NoExpose");
			return true;

		case EnterNotify:
		case LeaveNotify:

		case CirculateRequest:
		case ConfigureRequest:
		case MapRequest:
		case ResizeRequest:


		case CirculateNotify: 
		case CreateNotify:
		case DestroyNotify:
		case GravityNotify:
		case MapNotify:
		case ReparentNotify:
		case UnmapNotify:
		case VisibilityNotify:

		case ColormapNotify:  

/*		case WM_CREATE:
			return 0;
		case WM_CLOSE:
			close();
			return 0;*/
		default:
			return false;
	}
	return true;
}

}; };
