/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "gui3d.checkbox.h"
#include "gui3d.context.h"
#include "gui3d.fontmap.h"
#include "gui3d.root.h"

#include "object.pointer.inl"
#include "graphics.dimension.inl"
#include "freetype.font.inl"
#include "freetype.descriptor.inl"
#include "gui3d.checkbox.inl"
#include "gui3d.control.inl"
#include "gui3d.context.inl"
#include "gui3d.fontmap.inl"
#include "gui3d.view.inl"


namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Checkbox::Checkbox(const Rectangle<float>& r,const Message& msg) : Control(r,msg) {
	init();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Checkbox::Checkbox(const Rectangle<float>& r,const Message& msg,const char* str) : Control(r,msg) {
	m_Text = str;
	init();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Checkbox::init() {
	m_State = NormalState;
	m_Font = NULL;
	m_Checked = false;

	m_TextColour = Colour<float>(0.9f, 0.9f, 0.9f);
	m_BackColour = Colour<float>(0.2f, 0.2f, 0.2f);
	m_BorderColour = Colour<float>(0.3f, 0.3f, 0.3f);

	m_HoverTextColour = Colour<float>(1.0f, 1.0f, 1.0f);
	m_HoverBackColour = Colour<float>(0.3f, 0.3f, 0.3f);
	m_HoverBorderColour = Colour<float>(0.6f, 0.6f, 0.6f);

	m_PressedTextColour = Colour<float>(1.0f, 1.0f, 1.0f);
	m_PressedBackColour = Colour<float>(0.8f, 0.6f, 0.2f);
	m_PressedBorderColour = Colour<float>(1.0f, 0.8f, 0.2f);

	m_DisabledTextColour = Colour<float>(0.4f, 0.4f, 0.4f);
	m_DisabledBackColour = Colour<float>(0.2f, 0.2f, 0.2f);
	m_DisabledBorderColour = Colour<float>(0.3f, 0.3f, 0.3f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onPaint() {
	Context* context = getContext();
	if (m_FontMap == NULL) {
		if (m_Font == NULL)
			return true;
		m_FontMap = context->createFont(m_Font);
	}

	Dimension<float> size = m_Font->getSize();
	Rectangle<float> check(0.0f, size.getHeight()*0.1f, size.getWidth()*0.8f, size.getHeight()*0.8f);
	if (m_State == NormalState) {
		context->setColour(m_BackColour);
		context->fillRectangle(check);
		context->setColour(m_BorderColour);
		context->drawRectangle(check);
		context->setColour(m_TextColour);
	} else if (m_State == PressedState) {
		context->setColour(m_PressedBackColour);
		context->fillRectangle(check);
		context->setColour(m_PressedBorderColour);
		context->drawRectangle(check);
		context->setColour(m_PressedTextColour);
	} else if (m_State == HoveringState) {
		context->setColour(m_HoverBackColour);
		context->fillRectangle(check);
		context->setColour(m_HoverBorderColour);
		context->drawRectangle(check);
		context->setColour(m_HoverTextColour);
	} else if (m_State == DisabledState) {
		context->setColour(m_DisabledBackColour);
		context->fillRectangle(check);
		context->setColour(m_DisabledBorderColour);
		context->drawRectangle(check);
		context->setColour(m_DisabledTextColour);
	}
	if (m_Checked) {
		float x0 = size.getWidth()*0.1f;
		float x1 = size.getWidth()*0.7f;
		float y0 = size.getHeight()*0.2f;
		float y1 = size.getHeight()*0.8f;
		context->setLineWidth(2.0f);
		context->drawLine(Point<float>(x0,y0),Point<float>(x1,y1));
		context->drawLine(Point<float>(x1,y0),Point<float>(x0,y1));
		context->setLineWidth(1.0f);
	}
	Point<float> p;
	p.setX(size.getWidth());
	p.setY(size.getHeight()*0.8f);
	context->selectFont(m_FontMap);
	context->drawString(p, m_Text);
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onEnable() {
	m_State = NormalState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onDisable() {
	m_State = DisabledState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onMouseMove(const Point<float>& pos, Mouse::Button button) {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onMouseEnter(const Point<float>&, Mouse::Button buttons) {
	m_State = HoveringState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onMouseButton(const Point<float>&, Mouse::Button buttons) {
	if (buttons & Mouse::LeftDown) {
		if (m_State != PressedState) {
			m_State = PressedState;
			onPush();
		}
		return true;
	}
	if (buttons & Mouse::LeftUp) {
		if (m_State == PressedState) {
			m_Checked = !m_Checked;
			onRelease();
			onAction();
		}
		m_State = HoveringState;
		return true;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onMouseLeave() {
	if (m_State == PressedState)
		onRelease();
	m_State = NormalState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onPush() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Checkbox::onRelease() {
	return true;
}

}; };
