/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "io.buffer.h"

#include "io.buffer.inl"


namespace magnum { namespace io {
using magnum::math::min;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
unsigned long InputBuffer::read(void *buffer, unsigned long size) {
	ASSERT(m_Stream != NULL);

	byte* dst = (byte *)buffer;
	unsigned long read;
	unsigned long totalRead = 0;

	while(size > 0) {
		if (m_BufferLeft == 0) {
			m_BufferCurrent = m_Buffer;
			m_BufferLeft = m_Stream->read(m_Buffer, m_BufferSize);
			if (m_BufferLeft == 0)
				return totalRead;
		}
		read = min((int)size, m_BufferLeft);
		memcpy(dst, m_BufferCurrent, (int)read);
		m_BufferCurrent += read;
		dst += read;
		m_BufferLeft -= read;
		size -= read;
		totalRead += read;
	}

	return totalRead;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
unsigned long InputBuffer::skip(unsigned long size) {
	ASSERT(m_Stream != NULL);

	unsigned long read;
	unsigned long totalRead = 0;

	while(size > 0) {
		if (m_BufferLeft == 0) {
			m_BufferCurrent = m_Buffer;
			m_BufferLeft = m_Stream->read(m_Buffer, m_BufferSize);
			if (m_BufferLeft == 0)
				return totalRead;
		}
		read = min((int)size, m_BufferLeft);
		m_BufferCurrent += read;
		m_BufferLeft -= read;
		size -= read;
		totalRead += read;
	}

	return totalRead;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
unsigned long InputBuffer::available() {
	return m_BufferLeft + m_Stream->available();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool InputBuffer::close() {
	if (m_Stream != NULL) {
		m_Stream->release();
		m_Stream = NULL;
	}

	return true;
}


}; };
