/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "io.directory.h"

#include "container.array.inl"
#include "io.directory.inl"

namespace magnum { namespace io {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Directory::create(const char* name) {
#if defined(__FORWINDOWS__)
	return (CreateDirectory(name, NULL) != 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Directory::remove(const char* name) {
#if defined(__FORWINDOWS__)
	return (RemoveDirectory(name) != 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Directory::clear() {
	m_Nodes.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Directory::refresh() {
	HANDLE hnd;
	DirectoryNode node;
	
	m_Nodes.clear();
	
	hnd = FindFirstFile(m_Directory, &node.m_Entry);
	if (hnd == INVALID_HANDLE_VALUE) {
		if (GetLastError() == ERROR_NO_MORE_FILES)	
			return true;
		return false;
	}
	m_Nodes.insert(node);
	
	for(;;) {
		if (FindNextFile(hnd, &node.m_Entry) == 0)
			break;
		m_Nodes.insert(node);
	}
	
	FindClose(hnd);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Directory::open(const char* dir) {
	ASSERT(dir != NULL);
	m_Directory = dir;
	return refresh();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
String Directory::getCurrent() {
	char buffer[MAX_PATH];
	DWORD len;
	len = GetCurrentDirectory(MAX_PATH, buffer);
	return String(buffer);
}
}; };
