/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "media.node.h"
#include "media.pin.h"
#include "media.buffer.h"

#include "base.string.inl"
#include "object.class.inl"
#include "object.inl"
#include "system.thread.inl"
#include "container.queue.inl"
#include "media.node.inl"
#include "media.pin.inl"
#include "media.buffer.inl"
#include "media.handler.inl"

namespace magnum { namespace media {

const char Pin::DefaultInput[] = "DefaultInput";
const char Pin::DefaultOutput[] = "DefaultOutput";


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Pin::adjustFormat(Format* fmt) {
	ASSERT(m_Node != NULL);
	return m_Node->onAdjustFormat(this, fmt);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Pin::requestFormat(const Format* fmt) {
	ASSERT(m_Node != NULL);
	Format lfmt = *fmt;

	// Request new format
	if (!m_Node->onAdjustFormat(this, &lfmt))
		return false;
	if (!m_Node->onSetFormat(this, &lfmt))
		return false;

	m_Format = lfmt;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Pin::setFormat(const Format* fmt) {
	ASSERT(m_Node != NULL);
	// Request new format
	if (!m_Node->onSetFormat(this, fmt))
		return false;

	m_Format = *fmt;

	// fire handler
	Message msg(Message::SetFormat);
	msg.setPin(this);
	msg.setPointer(&m_Format);
	m_Node->sendMessage(msg);
	return true;
}


}; };
