/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "media.wavedevice.win.h"
#include "media.waveproducer.win.h"
#include "media.waveconsumer.win.h"

#include "base.string.inl"
#include "object.class.inl"
#include "object.inl"
#include "system.thread.inl"
#include "system.signal.inl"
#include "container.queue.inl"
#include "media.node.inl"
#include "media.pin.inl"
#include "media.buffer.inl"
#include "media.audio.inl"

#include "media.wavedevice.win.inl"
#include "media.waveproducer.win.inl"
#include "media.waveconsumer.win.inl"

#ifndef __FORWINDOWS__
#	error "This file is only for Windows!"
#endif


namespace magnum { namespace media { namespace _windows {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveProducer::WaveProducer(WaveDevice* dev) : AudioProducer(dev, Node::DefaultInput) , m_OutputPin(this) {
	m_Thread = NULL;
	publishOutputPin(&m_OutputPin);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveProducer::~WaveProducer() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::addBuffer(WaveBuffer* buf) {
	ASSERT(buf->getBufferSize() != 0)

	buf->m_WaveHeader.lpData = (LPSTR)buf->m_Buffer;
	buf->m_WaveHeader.dwBufferLength = buf->m_BufferSize;
	buf->m_WaveHeader.dwUser = (DWORD)buf;
	buf->m_WaveHeader.dwFlags = 0;
	buf->m_WaveHeader.dwLoops = 0;

	if (waveInPrepareHeader(m_WaveHandle, &buf->m_WaveHeader, sizeof(buf->m_WaveHeader)) != MMSYSERR_NOERROR)
		return false;
	if (waveInAddBuffer(m_WaveHandle, &buf->m_WaveHeader, sizeof(buf->m_WaveHeader)) != MMSYSERR_NOERROR)
		return false;

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::onStart() {
	MMRESULT result;
	int i;

	if (m_Thread != NULL)
		return false;

	m_Thread = new WaveThread(this);
	m_Thread->start();

	result = waveInOpen(&m_WaveHandle, WAVE_MAPPER, &m_WaveFormat, m_Thread->getThreadId(), (DWORD)this, CALLBACK_THREAD | WAVE_FORMAT_DIRECT);
	if (result != MMSYSERR_NOERROR)
		goto errorOpen;

	// Add some buffers
	for (i=0; i<3 ;i++) {
		WaveBuffer* buffer;
		buffer = new WaveBuffer(this);
		buffer->createBuffer(m_BufferSize);
		if (!addBuffer(buffer))
			goto errorStart;
	}

	// Start recording
	if (waveInStart(m_WaveHandle) != MMSYSERR_NOERROR)
		goto errorStart;

	return true;

errorStart:
	waveInClose(m_WaveHandle);
errorOpen:
	PostThreadMessage((DWORD)m_Thread->getThreadId(), WM_QUIT, NULL, NULL);
	m_Thread->join();
	m_Thread->release();
	m_Thread = NULL;
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::onStop() {
	if (m_Thread == NULL)
		return false;

	if (waveInStop(m_WaveHandle) != MMSYSERR_NOERROR)
		return false;
	if (waveInReset(m_WaveHandle) != MMSYSERR_NOERROR)
		return false;
	if (waveInClose(m_WaveHandle) != MMSYSERR_NOERROR)
		return false;
	PostThreadMessage((DWORD)m_Thread->getThreadId(), WM_QUIT, NULL, NULL);
	m_Thread->join();
	m_Thread->release();
	m_Thread = NULL;

	return true;	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::onPause() {
	return (waveInStop(m_WaveHandle) == MMSYSERR_NOERROR);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::onUnpause() {
	return (waveInStart(m_WaveHandle) == MMSYSERR_NOERROR);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::onSetFormat(Pin* pin, const Format* fmt) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::onAdjustFormat(Pin* pin, Format* fmt) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveProducer::WaveThread::WaveThread(WaveProducer *node) {
	m_Node = node;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveProducer::WaveThread::~WaveThread() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveProducer::WaveThread::run() {
	MSG Message;
	WAVEHDR* hdr;
	WaveBuffer* mbuf;
	PeekMessage(&Message, NULL, WM_USER, WM_USER, PM_NOREMOVE);

	while (::GetMessage(&Message, NULL, 0, 0)) {
		switch (Message.message) {
			case WIM_OPEN:
			case WIM_CLOSE:
				break;
			case WIM_DATA:
				hdr = (WAVEHDR *)Message.wParam;
				waveInUnprepareHeader(m_Node->m_WaveHandle, hdr, sizeof(WAVEHDR));
				mbuf = (WaveBuffer *)hdr->dwUser;
				mbuf->setBufferSize(hdr->dwBytesRecorded);
				m_Node->sendBuffer(&m_Node->m_OutputPin, mbuf, false);
				mbuf = new WaveBuffer(m_Node);
				mbuf->createBuffer(m_Node->m_BufferSize);
				m_Node->addBuffer(mbuf);
				break;
			default:
				TranslateMessage(&Message);
				DispatchMessage(&Message);
		}
	}

	return true;
}

}; }; };
