/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "net.address.h"
#include "net.exception.h"
#include "object.class.h"
#include "net.socket.h"

#include "net.socket.inl"
#include "net.address.inl"
#include "base.string.inl"
#include "object.inl"
#include "object.exception.inl"
#include "object.class.inl"

#include <stdio.h>

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Address Address::getLocalHost() {
	char buffer[1024];
	if (gethostname(buffer,1024) == 0) {
		return Address(String(buffer));
	}

	return Address();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Address::resolveHostname() {
	if (!m_AddressValid)
		throw UnknownHostException(Socket::ExceptionInvalidAddress);

	struct hostent *info;
	uint32 adr = m_Address; 
	info = gethostbyaddr((const char *)&adr, 4, PF_INET);
	if (info == NULL) {
		char str[30];
		adr = ntohl((unsigned int)adr);
		sprintf(str,"%d.%d.%d.%d",(adr >> 24), (adr >> 16) & 255, (adr >> 8) & 255, adr & 255);
		m_Hostname = str;
		m_HostnameValid = true;
		return true;
//		m_HostnameValid = false;
//		m_AddressValid = false;
//		throw UnknownHostException(Socket::ExceptionUnknownHost);
	}

	m_Hostname = info->h_name;
	m_HostnameValid = true;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Address::resolveAddress() {
	if (!m_HostnameValid)
		throw UnknownHostException(Socket::ExceptionInvalidAddress);

	struct hostent *info;
	info = gethostbyname(m_Hostname.getString());
	if (info == NULL) {
		m_HostnameValid = false;
		m_AddressValid = false;
		throw UnknownHostException(Socket::ExceptionUnknownHost);
	}

	m_Address = *(uint32 *)info->h_addr_list[0];
	m_AddressValid = true;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
String Address::toString() const {
	if (m_HostnameValid)
		return m_Hostname;

	if (!m_AddressValid)
		return String("Invalid IP-address");

	struct hostent *info;
	uint32 adr = m_Address; 
	info = gethostbyaddr((const char *)&adr, 4, PF_INET);
	if (info == NULL) {
		char str[30];
		adr = ntohl((unsigned int)adr);
		sprintf(str,"%d.%d.%d.%d",(adr >> 24), (adr >> 16) & 255, (adr >> 8) & 255, adr & 255);
		return String(str);
	} 

	return String(info->h_name);;
}

}; };
