/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "net.multicastsocket.h"
#include "net.exception.h"

#include "object.inl"
#include "base.string.inl"
#include "object.class.inl"
#include "object.exception.inl"
#include "net.socket.inl"
#include "net.multicastsocket.inl"

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool MulticastSocket::joinGroup(const Address& address) {
	in_addr iadr;
	ip_mreq mreq;
	int optlen = sizeof(iadr);
	int err;
	Address adr = address;

	mreq.imr_multiaddr.s_addr = adr.getAddressNO();
	mreq.imr_interface.s_addr = INADDR_ANY;
	err = setsockopt(m_Socket, IPPROTO_IP, IP_ADD_MEMBERSHIP,(char*)&mreq, sizeof(mreq));
	if (err == SOCKET_ERROR) {
		throwSocketException();
	}

	getsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_IF, (char *)&iadr, (int *)&optlen);
#if defined(__FORWINDOWS__)
	m_Interface.setAddressNO(iadr.S_un.S_addr);
#else
	m_Interface.setAddressNO(iadr.s_addr);
#endif
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool MulticastSocket::leaveGroup(const Address& address) {
	in_addr iadr;
	ip_mreq mreq;
	int optlen = sizeof(iadr);
	int err;
	Address adr = address;

	mreq.imr_multiaddr.s_addr = adr.getAddressNO();
	mreq.imr_interface.s_addr = INADDR_ANY;
	err = setsockopt(m_Socket, IPPROTO_IP, IP_DROP_MEMBERSHIP,(char*)&mreq, sizeof(mreq));
	if (err == SOCKET_ERROR) {
		throwSocketException();
	}

	getsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_IF, (char *)&iadr, (int *)&optlen);
#if defined(__FORWINDOWS__)
	m_Interface.setAddressNO(iadr.S_un.S_addr);
#else
	m_Interface.setAddressNO(iadr.s_addr);
#endif
	
	return true;}

}; };
