/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "net/socket.h"
#include "object/class.h"
#include "net/exception.h"

#include "net/socket.inl"
#include "io/stream.inl"
#include "io/inputstream.inl"
#include "io/outputstream.inl"
#include "base/string.inl"
#include "object/object.inl"
#include "net/address.inl"
#include "object/exception.inl"
#include "object/class.inl"


#ifndef __FORUNIX__
#error "This file is only for Unix"
#endif


namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Socket::throwSocketException(int err) {
	switch(err) {
		case EBADF:
		case ENOTSOCK:
			throw SocketException(ExceptionDescriptor);
		case ENETDOWN:
			throw SocketException(ExceptionNetDown);
		case ENETRESET:  
			throw SocketException(ExceptionNetReset);
		case EINTR:
			throw SocketException(ExceptionInterrupted);
		case EADDRINUSE:
			throw SocketException(ExceptionAddressInUse);
		case EALREADY:
		case EISCONN:
			throw SocketException(ExceptionAlreadyConnected);
		case ENOTCONN:  
			throw SocketException(ExceptionNotConnected);
		case EFAULT:
			throw SocketException(ExceptionAddressFault);
		case ESHUTDOWN:
			throw SocketException(ExceptionShutdown);
		case EMSSIZE:
			throw SocketException(ExceptionMessageSize);
		case EINVAL:
			throw SocketException(ExceptionAlreadyConnected);
		case EACCES:
			throw SocketException(ExceptionInterrupted);
		case EADDRNOTAVAIL:
			throw SocketException(ExceptionAddressNotAvailable);
		case EPIPE:
			throw SocketException(ExceptionBrokenPipe);

		case ECONNREFUSED:
			throw ConnectionRefusedException(ExceptionConnectionRefused);
		case EWOULDBLOCK:
			throw ConnectException(ExceptionWouldBlock);
		case ECONNRESET: 
			throw ConnectionResetException(ExceptionConnectionReset);
		case ECONNABORTED:
			throw TimeOutException(ExceptionConnectionAborted);
		case ENETUNREACH:
			throw NoRouteToHostException(ExceptionNoRouteToHost);

		default:
			throw SocketException(ExceptionUnknown);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Socket::throwBindException(int err) {
	switch(err) {
		case EADDRINUSE:
			throw BindException(ExceptionAddressInUse);
		case EADDRNOTAVAIL:
			throw BindException(ExceptionAddressNotAvailable);
		case EALREADY:
		case EISCONN:
			throw BindException(ExceptionAlreadyConnected);
		case EFAULT:
			throw BindException(Socket::ExceptionAddressFault);
		default:
			throwSocketException(err);
	}
}

}; };
