/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "object.class.h"
#include "net.socket.h"
#include "net.exception.h"

#include "object.inl"
#include "object.class.inl"
#include "object.exception.inl"
#include "io.stream.inl"
#include "io.inputstream.inl"
#include "io.outputstream.inl"
#include "base.string.inl"
#include "net.socket.inl"
#include "net.address.inl"

#ifndef __FORWINDOWS__
#error "This file is only for Windows"
#endif


namespace magnum { namespace net {

Socket::Startup Socket::m_Startup;
WSAPROTOCOL_INFO* Socket::m_ProtocolInfo;
unsigned int Socket::m_ProtocolInfoCount;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Socket::Startup::Startup() {
	int nRet;
	DWORD dwBufLen;
	WSADATA wsaData;

	Socket::m_ProtocolInfo = NULL;
	Socket::m_ProtocolInfoCount = 0;
	dwBufLen = 0;

	// Init the WSA...
	WSAStartup(MAKEWORD( 1, 1 ), &wsaData);

    // first have WSAEnumProtocols tell us how big a buffer is needed
    nRet = WSAEnumProtocols(NULL, Socket::m_ProtocolInfo, &dwBufLen);
    if (nRet != SOCKET_ERROR)
		return;
    if (WSAGetLastError() != WSAENOBUFS)
		return;

    // WSAEnumProtocols failed for the "expected" reason, allocate a buffer
    // of the needed size
    Socket::m_ProtocolInfo = (WSAPROTOCOL_INFO *)new BYTE[dwBufLen];

    // now we can call WSAEnumProtocols again with the expectation it will
    // succeed because we have allocated a big enough buffer.
    nRet = WSAEnumProtocols(NULL, Socket::m_ProtocolInfo, &dwBufLen);
	if (nRet == SOCKET_ERROR)
		return;

	Socket::m_ProtocolInfoCount = nRet;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Socket::Startup::~Startup() {
	if (Socket::m_ProtocolInfo != NULL)
		delete[] (BYTE *)Socket::m_ProtocolInfo;
	WSACleanup();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Socket::throwSocketException(int err) {
	switch(err) {
		case WSAENOTSOCK:
			throw SocketException(ExceptionDescriptor);
		case WSANOTINITIALISED:
			throw SocketException(ExceptionNotInitialised);
		case WSAENETDOWN:
			throw SocketException(ExceptionNetDown);
		case WSAENETRESET:  
			throw SocketException(ExceptionNetReset);
		case WSAEADDRINUSE:
			throw SocketException(ExceptionAddressInUse);
		case WSAEINTR:
			throw SocketException(ExceptionInterrupted);
		case WSAEALREADY:
		case WSAEISCONN:
			throw SocketException(ExceptionAlreadyConnected);
		case WSAENOTCONN:  
			throw SocketException(ExceptionNotConnected);
		case WSAEFAULT:
			throw SocketException(ExceptionAddressFault);
		case WSAESHUTDOWN:
			throw SocketException(ExceptionShutdown);
		case WSAEMSGSIZE:
			throw SocketException(ExceptionMessageSize);
		case WSAEINVAL: 
			throw SocketException(ExceptionBlocked);
		case WSAEACCES:
			throw SocketException(ExceptionInterrupted);
		case WSA_IO_PENDING:
			throw SocketException(ExceptionIOPending);
		case WSA_OPERATION_ABORTED:
			throw SocketException(ExceptionIOAborted);
		case WSAEADDRNOTAVAIL:
			throw SocketException(ExceptionAddressNotAvailable);

		case WSAECONNREFUSED:
			throw ConnectionRefusedException(ExceptionConnectionRefused);
		case WSAEDISCON:
			throw ConnectionClosedException(ExceptionConnectionClosed);
		case WSAEWOULDBLOCK:
			throw ConnectException(ExceptionWouldBlock);
		case WSAECONNRESET: 
			throw ConnectionResetException(ExceptionConnectionReset);
		case WSAECONNABORTED:
			throw TimeOutException(ExceptionConnectionAborted);
		case WSAENETUNREACH:
			throw NoRouteToHostException(ExceptionNoRouteToHost);
		default:
			throw SocketException(Socket::ExceptionUnknown);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Socket::throwBindException(int err) {
	switch(err) {
		case WSAEADDRINUSE:
			throw BindException(ExceptionAddressInUse);
		case WSAEALREADY:
		case WSAEISCONN:
			throw BindException(ExceptionAlreadyConnected);
		case WSAEFAULT:
			throw BindException(Socket::ExceptionAddressFault);
		case WSAEADDRNOTAVAIL:
			throw BindException(ExceptionAddressNotAvailable);
		default:
			throwSocketException(err);
	}
}

}; };