/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "scene3d.geometry.h"
#include "scene3d.locale.h"
#include "scene3d.universe.h"
#include "scene3d.observer.h"
#include "scene3d.root.h"
#include "scene3d.soundnode.h"
#include "scene3d.behaviour.h"
#include "scene3d.modifier.h"

#include "container.allocator.inl"
#include "container.chain.inl"
#include "geometry.space.inl"
#include "geometry.cube.inl"
#include "geometry.box.inl"
#include "geometry.bounds.inl"
#include "geometry.transform.inl"
#include "math.vector.inl"
#include "scene3d.geometry.inl"
#include "scene3d.observer.inl"
#include "scene3d.node.inl"
#include "scene3d.leaf.inl"
#include "scene3d.locale.inl"
#include "scene3d.renderqueue.inl"


namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Geometry::onTransform(const Transform3<float>& parent) {
	m_WorldTransform = parent * m_Transform;
	setWorldBounds(m_WorldTransform * getBounds());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Geometry::onSchedule(RenderQueue* rq, const Transform3<float>& ) {
	rq->insert(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Geometry::onAttachLocale(Locale* l) {
	Graphics::onAttachLocale(l);
	clearFlag(ActiveGeometry);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Geometry::onDetachLocale(Locale* l) {
	Graphics::onDetachLocale(l);
	if (getChain() != NULL)
		getChain()->remove(this);
	clearFlag(ActiveGeometry);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Geometry::onEnable() {
	Graphics::onEnable();
	clearFlag(ActiveGeometry);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Geometry::onDisable() {
	Graphics::onDisable();
	if (getChain() == NULL)
		return;
	getChain()->remove(this);
	clearFlag(ActiveGeometry);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Geometry::onRender() const {
}


}; }; 
