/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "scene3d.renderqueue.h"
#include "scene3d.geometry.h"
#include "scene3d.modifier.h"
#include "scene3d.soundnode.h"

#include "container.iterator.inl"
#include "container.chain.inl"
#include "scene3d.renderqueue.inl"

namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
RenderQueue::RenderQueue(Observer* o) {
	ASSERT(o != NULL);
	m_Observer = o;
	m_Universe = o->getUniverse();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
RenderQueue::~RenderQueue() {
	clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void RenderQueue::view() {
	// Process all geometry nodes
	magnum::container::Chain<Geometry>::Iterator iter;
	iter = m_ActiveGeometries.begin();
	while(iter != m_ActiveGeometries.end()) {
		Geometry* g = (Geometry*)iter;
		iter++;

		// Is this geometry node still active?
		if (g->testFlag(Node::ActiveGeometry)) {
			g->clearFlag(Node::ActiveGeometry);
			g->onRender();
		} else {
			m_ActiveGeometries.remove(g);
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void RenderQueue::clearGeometries() {
	while(!m_ActiveGeometries.isEmpty()) {
		Geometry* g = (Geometry*)m_ActiveGeometries.begin();
		g->clearFlag(Node::ActiveGeometry);
		m_ActiveGeometries.remove(g);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void RenderQueue::clearModifiers() {
	while(!m_ActiveModifiers.isEmpty()) {
		Modifier* b = (Modifier*)m_ActiveModifiers.begin();
		b->clearFlag(Node::ActiveModifier);
		m_ActiveModifiers.remove(b);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void RenderQueue::clear() {
	clearGeometries();
	clearModifiers();
}

}; };
