/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "system.ticker.h"

#include "system.ticker.inl"
#include "system.thread.inl"


namespace magnum { namespace system {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Ticker::Ticker() {
	m_Thread.m_Ticker = this;
	m_Alive = false;
	m_Frequency = 1000;
	m_Timer = CreateWaitableTimer(NULL, FALSE, NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Ticker::~Ticker() {
	kill();
	CloseHandle(m_Timer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Ticker::start() {
	m_Alive = true;
	if (!m_Thread.isAlive())
		m_Thread.start();
	LARGE_INTEGER starttime;
	starttime.QuadPart = -1;
	SetWaitableTimer(m_Timer, &starttime, m_Frequency, NULL, NULL, FALSE); 
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Ticker::stop() {
	if (!m_Thread.isAlive())
		return;
	CancelWaitableTimer(m_Timer);
	m_Alive = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Ticker::kill() {
	if (!m_Thread.isAlive())
		return;
	CancelWaitableTimer(m_Timer);
	m_Alive = false;
	LARGE_INTEGER starttime;
	starttime.QuadPart = -1;
	SetWaitableTimer(m_Timer, &starttime, 1, NULL, NULL, FALSE); 
	m_Thread.join();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Ticker::setFrequency(unsigned int f) {
	m_Frequency = f;
	if (!m_Alive)
		return;
	LARGE_INTEGER starttime;
	starttime.QuadPart = -1;
	SetWaitableTimer(m_Timer, &starttime, m_Frequency, NULL, NULL, FALSE); 
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Ticker::onTick() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Ticker::TickerThread::run() {
	while(true) {
		WaitForSingleObject(m_Ticker->m_Timer, INFINITE);
		if (!m_Ticker->m_Alive)
			break;
		m_Ticker->onTick();
	}
	CancelWaitableTimer(m_Ticker->m_Timer);
	return true;
}

}; };
